package ch.dlcm.assembling.zip;

import java.io.File;
import java.net.URI;
import java.net.URISyntaxException;
import java.nio.file.Paths;
import java.util.List;

import javax.annotation.PostConstruct;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import ch.dlcm.assembling.AssemblingService;
import ch.dlcm.exception.DLCMException;
import ch.dlcm.model.Status.StatusValue;
import ch.dlcm.model.oais.ArchivalInfoPackage;
import ch.dlcm.model.oais.DisseminationInfoPackage;
import ch.dlcm.service.DLCMService;
import ch.dlcm.service.DipMgmt;
import ch.dlcm.tool.FileTool;
import ch.dlcm.tool.ZipTool;

@Service
public class ZipAssemblingService extends DLCMService implements AssemblingService {

  private String assemblingPath;
  
  @Autowired
  DipMgmt dipMgmt;

  @PostConstruct
  @Override
  protected void loadConfig() {
    this.assemblingPath = this.dlcmConfig.getOrderLocation();
  }

  @Override
  public boolean checkPackage(String id) {
    // Check if Order exists
    if (Paths.get(this.getPackageFolder(id)).toFile().exists()) {
      return false;
    }
    // Check if Order archive exists
    if (!Paths.get(this.getPackageUri(id)).toFile().exists()) {
      return false;
    }
    // Check order archive
    ZipTool zipTool = new ZipTool(this.getPackageUri(id));
    return zipTool.checkZip();
  }

  @Override
  public String submitPackage(String id, String name) {
    // Create order archive (i.e. DIP)
    ZipTool zipTool = new ZipTool(this.getPackageUri(id));
    if (zipTool.zipFiles(this.getPackageFolder(id))) {
      // Purge working files
      if (FileTool.deleteDirectory(new File(this.getPackageFolder(id)))) {
        return id;
      }
    }
    return null;
  }

  @Override
  public StatusValue getStatus(String id) {
    if (!this.checkPackage(id)) {
      return StatusValue.InProgress;
    }
    return StatusValue.Completed;
  }

  @Override
  public String getPackageUri(String id) {
    return this.getPackageFolder(id) + ".zip";
  }

  private String getPackageFolder(String id) {
    return this.assemblingPath + "/" + id;
  }

  @Override
  public String sendPackage(String id, String name) {
    throw new UnsupportedOperationException();
  }

  @Override
  public List<?> listPackages() {
    throw new UnsupportedOperationException();
  }

  @Override
  public String preparePackage(String id, List<ArchivalInfoPackage> aips) throws DLCMException {
    if (aips.size() == 1) {
      if (!extractArchive(aips.get(0).getDownloadUri(), this.getPackageFolder(id))) {
        return null;
      }
    } else {
      for (ArchivalInfoPackage aip : aips) {
        if (!extractArchive(aip.getDownloadUri(), this.getPackageFolder(id) + "/data/" + aip.getDlcmId())) {
          return null;
        }
      }
    }
    return id;
  }

  private boolean extractArchive(URI archive, String destination) {
    ZipTool zipAip = new ZipTool(archive);
    return zipAip.unzipFiles(destination);
  }

  @Override
  public DisseminationInfoPackage createPackage(String id, String name, List<ArchivalInfoPackage> aips) throws URISyntaxException {
    return this.dipMgmt.createDIP(id, name, aips, new URI("file:"+this.getPackageUri(id)));
  }

}
