package ch.dlcm.controller.access;

import static org.springframework.hateoas.mvc.ControllerLinkBuilder.linkTo;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.methodOn;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.InputStreamResource;
import org.springframework.hateoas.ResourceSupport;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import ch.dlcm.config.ContextHelper;
import ch.dlcm.controller.ResourceController;
import ch.dlcm.exception.DLCMException;
import ch.dlcm.exception.DLCMResourceNotFoundException;
import ch.dlcm.message.AipDownloadMessage;
import ch.dlcm.model.Status;
import ch.dlcm.model.Status.StatusType;
import ch.dlcm.model.Status.StatusValue;
import ch.dlcm.model.oais.ArchivalInfoPackage;
import ch.dlcm.rest.RESTAction;
import ch.dlcm.rest.RESTName;
import ch.dlcm.rest.RESTResult;
import ch.dlcm.rest.RESTResult.ActionStatus;
import ch.dlcm.service.SearchMgmt;

@RestController
@RequestMapping("/" + RESTName.ACCESS + "/" + RESTName.AIP)
public class AIPDownloadController extends ResourceController<ArchivalInfoPackage> {

  @Autowired
  SearchMgmt searchMgmt;

  @RequestMapping(method = RequestMethod.POST)
  public HttpEntity<ArchivalInfoPackage> create(@RequestBody ArchivalInfoPackage aipToGet) throws DLCMException {
    ArchivalInfoPackage aip;
    try {
      aip = this.searchMgmt.getAIP(aipToGet.getDlcmId());
    } catch (DLCMResourceNotFoundException e) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }
    if (aip != null) {
      if (aip.getInfo().getStatus().getValue() != StatusValue.Completed) {
        return new ResponseEntity<>(HttpStatus.NOT_MODIFIED);
      }
      if (this.itemRepository.findOne(aip.getDlcmId()) == null) {
        aip.reset();
        if (this.itemRepository.save(aip) == null) {
          return new ResponseEntity<>(HttpStatus.NOT_MODIFIED);
        }
        ContextHelper.getPublisher().publishEvent(new AipDownloadMessage(aip.getDlcmId()));
        this.addLinks(aip);
        return new ResponseEntity<>(aip, HttpStatus.CREATED);
      }
    }
    return new ResponseEntity<>(HttpStatus.NOT_FOUND);
  }

  @RequestMapping(value = "/{id}/" + RESTName.DL, method = RequestMethod.GET)
  public HttpEntity<InputStreamResource> getFiles(@PathVariable String id) throws DLCMException {
    ArchivalInfoPackage item = itemRepository.findOne(id);
    if (item == null) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }
    return download(item.getClass().getName(), id, item.getDownloadUri(), id + ".aip");
  }

  @RequestMapping(value = "/{id}/" + RESTName.RESUME, method = RequestMethod.GET)
  public HttpEntity<RESTResult> resume(@PathVariable String id) {
    ArchivalInfoPackage item = itemRepository.findOne(id);
    if (item == null) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }
    RESTResult res = new RESTResult(item.getDlcmId());
    res.setStatus(ActionStatus.NotExecuted);
    if (item.getInfo().getStatus().getValue() == StatusValue.Completed) {
      res.setStatus(ActionStatus.NonApplicable);
      res.setMesssage(messageService.get("package.completed", new Object[] { id }));
    } else if (item.getInfo().getStatus().getValue() == StatusValue.Downloading) {
      res.setMesssage(messageService.get("package.inprogress", new Object[] { id }));
    } else {
      item.setPackageStatus(StatusValue.InProgress);
      ;
      if (itemRepository.save(item) == null) {
        res.setMesssage(messageService.get("package.cannotupdate", new Object[] { id }));
      } else {
        res.setStatus(ActionStatus.Executed);
        res.setMesssage(messageService.get("package.download", new Object[] { id }));
        ContextHelper.getPublisher().publishEvent(new AipDownloadMessage(id));
      }
    }
    res.add(linkTo(methodOn(this.getClass()).resume(id)).withSelfRel());
    res.add(linkTo(methodOn(this.getClass()).list(id)).withRel(RESTAction.PARENT));
    return new ResponseEntity<>(res, HttpStatus.OK);
  }

  @RequestMapping(value = "/" + RESTName.STATUS, method = RequestMethod.GET)
  public HttpEntity<StatusValue[]> listStatus() {
    return new ResponseEntity<>(Status.listStatusValues(StatusType.Package), HttpStatus.OK);
  }

  @Override
  protected boolean addLinks(ArchivalInfoPackage aip) {
    aip.removeLinks();
    aip.addLinksForAccess(linkTo(this.getClass()), true, true);
    return true;
  }

  @Override
  protected <W extends ResourceSupport> boolean addOthersLinks(W w) {
    w.add(linkTo(methodOn(this.getClass()).listStatus()).withRel(RESTAction.VALUES));
    return true;
  }

}