package ch.dlcm.controller.access;

import static org.springframework.hateoas.mvc.ControllerLinkBuilder.linkTo;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.methodOn;

import ch.dlcm.exception.DLCMException;
import org.springframework.core.io.InputStreamResource;
import org.springframework.hateoas.ResourceSupport;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import ch.dlcm.config.ContextHelper;
import ch.dlcm.controller.ResourceController;
import ch.dlcm.message.DipMessage;
import ch.dlcm.model.Status;
import ch.dlcm.model.Status.StatusType;
import ch.dlcm.model.Status.StatusValue;
import ch.dlcm.model.oais.DisseminationInfoPackage;
import ch.dlcm.rest.RESTAction;
import ch.dlcm.rest.RESTName;
import ch.dlcm.rest.RESTResult;
import ch.dlcm.rest.RESTResult.ActionStatus;

@RestController
@RequestMapping("/" + RESTName.ACCESS + "/" + RESTName.DIP)
public class DIPController extends ResourceController<DisseminationInfoPackage> {

	@RequestMapping(value = "/{id}/" + RESTName.DL, method = RequestMethod.GET)
	public HttpEntity<InputStreamResource> getFiles(@PathVariable String id) throws DLCMException {
		DisseminationInfoPackage item = itemRepository.findOne(id);
		if (item == null) {
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}
		return download(item.getClass().getName(), id,item.getDownloadUri(),id+".dip");
	}

	@RequestMapping(value = "/{id}/" + RESTName.RESUME, method = RequestMethod.GET)
	public HttpEntity<RESTResult> resume(@PathVariable String id) {
		RESTResult res = resumePackage(id, itemRepository);
		if (res == null) {
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}
		if (res.getStatus() == ActionStatus.Executed){
			ContextHelper.getPublisher().publishEvent(new DipMessage(id));
		}
		res.add(linkTo(methodOn(this.getClass()).resume(id)).withSelfRel());
		res.add(linkTo(methodOn(this.getClass()).list(id)).withRel(RESTAction.PARENT));
		return new ResponseEntity<>(res, HttpStatus.OK);
	}
	

	@RequestMapping(value = "/" + RESTName.STATUS, method = RequestMethod.GET)
	public HttpEntity<StatusValue[]> listStatus() {
		return new ResponseEntity<>(Status.listStatusValues(StatusType.Package), HttpStatus.OK);
	}

	@Override
	protected <W extends ResourceSupport> boolean addOthersLinks(W w) {
		w.add(linkTo(methodOn(this.getClass()).listStatus()).withRel(RESTAction.VALUES));
		return true;
	}

}