package ch.dlcm.controller.access;

import static org.springframework.hateoas.mvc.ControllerLinkBuilder.linkTo;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.methodOn;

import java.util.List;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.InputStreamResource;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.hateoas.ResourceSupport;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import ch.dlcm.config.ContextHelper;
import ch.dlcm.controller.ResourceController;
import ch.dlcm.exception.DLCMException;
import ch.dlcm.message.AipDownloadMessage;
import ch.dlcm.model.Order;
import ch.dlcm.model.OrderArchive;
import ch.dlcm.model.QueryType;
import ch.dlcm.model.Status;
import ch.dlcm.model.Status.StatusType;
import ch.dlcm.model.Status.StatusValue;
import ch.dlcm.model.oais.DisseminationInfoPackage;
import ch.dlcm.repository.ArchivalInfoPackageRepository;
import ch.dlcm.repository.OrderRepository;
import ch.dlcm.rest.RESTAction;
import ch.dlcm.rest.RESTCollection;
import ch.dlcm.rest.RESTName;
import ch.dlcm.rest.RESTPage;
import ch.dlcm.service.SearchMgmt;
import ch.dlcm.tool.CollectionTool;

@RestController
@RequestMapping("/" + RESTName.ACCESS + "/" + RESTName.ORDER)
public class OrderController extends ResourceController<Order> {
  private static final Logger log = Logger.getLogger(OrderController.class);

  @Autowired
  SearchMgmt searchMgmt;

  @Autowired
  ArchivalInfoPackageRepository aipRepository;

  @RequestMapping(value = "/{id}/" + RESTName.VIEW, method = RequestMethod.GET)
  public HttpEntity<RESTCollection<OrderArchive>> view(@PathVariable String id,
          @RequestParam(defaultValue = "false") boolean allInfo, Pageable pageable)
          throws DLCMException {
    Order item = this.itemRepository.findOne(id);
    if (item == null) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }
    if (item.isEmptyQuery()) {
      return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }
    RESTCollection<OrderArchive> collection = this.searchMgmt.search(item, allInfo, pageable);
    collection.add(linkTo(methodOn(this.getClass()).view(id, allInfo, pageable)).withSelfRel());
    collection.add(linkTo(methodOn(this.getClass()).list(id)).withRel(RESTAction.PARENT));
    this.addPageLinks(linkTo(methodOn(this.getClass()).view(id, allInfo, pageable)), collection,
            pageable);
    return new ResponseEntity<>(collection, HttpStatus.OK);
  }

  @RequestMapping(value = "/{id}/" + RESTName.SAVE, method = RequestMethod.GET)
  public HttpEntity<Order> save(@PathVariable String id) throws DLCMException {
    Order item = this.itemRepository.findOne(id);
    if (item == null) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }
    if (item.getQuery().isEmpty()) {
      return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }
    boolean toUpdate = false;

    RESTCollection<OrderArchive> collection = null;
    Pageable pageable = new PageRequest(0, RESTPage.DEFAULT_SIZE_PAGE);
    do {
      collection = this.searchMgmt.search(item, true, pageable);
      List<OrderArchive> list = CollectionTool.toList(collection.get_data());
      for (OrderArchive archive : list) {
        if (archive.getArchive().getInfo().getStatus().getValue() != StatusValue.Completed) {
          continue;
        }
        if (this.aipRepository.findOne(archive.getArchive().getDlcmId()) == null) {
          archive.getArchive().reset();
          if (this.aipRepository.save(archive.getArchive()) == null) {
            return new ResponseEntity<>(HttpStatus.NOT_MODIFIED);
          }
          ContextHelper.getPublisher()
                  .publishEvent(new AipDownloadMessage(archive.getArchive().getDlcmId()));
        }
        if (item.addItem(archive.getArchive())) {
          toUpdate = true;
        }
      }
      pageable = pageable.next();
    } while (collection.get_page().hasNext());
    if (toUpdate) {
      item = itemRepository.save(item);
      if (item == null) {
        return new ResponseEntity<>(HttpStatus.NOT_MODIFIED);
      }
    } else {
      return new ResponseEntity<>(HttpStatus.NOT_MODIFIED);
    }
    item.add(linkTo(methodOn(this.getClass()).save(id)).withSelfRel());
    item.add(linkTo(methodOn(this.getClass()).list(id)).withRel(RESTAction.PARENT));
    return new ResponseEntity<>(item, HttpStatus.OK);
  }

  @RequestMapping(value = "/{id}/" + RESTName.DL, method = RequestMethod.GET)
  public HttpEntity<InputStreamResource> getFiles(@PathVariable String id) {
    Order item = ((OrderRepository) this.itemRepository).findOneWithDip(id);
    if (item == null) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }
    if (item.getStatus().getValue() != StatusValue.Ready) {
      return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }
    try {
      for (DisseminationInfoPackage dip : item.getDipPackages()) {
        // TODO download all Dips
      }

    } catch (UnsupportedOperationException e) {
      log.error("Download not supported for " + item.getClass().getName() + " (" + id + ")", e);
      return new ResponseEntity<>(HttpStatus.NOT_IMPLEMENTED);
    }
    return new ResponseEntity<>(HttpStatus.NOT_IMPLEMENTED);
  }

  @RequestMapping(value = "/" + RESTName.STATUS, method = RequestMethod.GET)
  public HttpEntity<StatusValue[]> listStatus() {
    return new ResponseEntity<>(Status.listStatusValues(StatusType.Order), HttpStatus.OK);
  }

  @RequestMapping(value = "/" + RESTName.QUERYTYPE, method = RequestMethod.GET)
  public HttpEntity<QueryType[]> listQueryType() {
    return new ResponseEntity<>(QueryType.values(), HttpStatus.OK);
  }

  @Override
  protected <W extends ResourceSupport> boolean addOthersLinks(W w) {
    w.add(linkTo(methodOn(this.getClass()).listStatus()).withRel(RESTAction.VALUES));
    w.add(linkTo(methodOn(this.getClass()).listQueryType()).withRel(RESTAction.VALUES));
    return true;
  }
}
