package ch.dlcm.service;

import java.net.URI;

import javax.annotation.PostConstruct;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jms.annotation.JmsListener;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import ch.dlcm.message.AipDownloadMessage;
import ch.dlcm.model.DataCategory;
import ch.dlcm.model.DataFile;
import ch.dlcm.model.Status.StatusValue;
import ch.dlcm.model.oais.AipDataFile;
import ch.dlcm.model.oais.ArchivalInfoPackage;
import ch.dlcm.repository.ArchivalInfoPackageRepository;
import ch.dlcm.rest.RESTName;

@Component
public class AipDownloadMgmt extends DLCMService {
  private static final Logger log = Logger.getLogger(AipDownloadMgmt.class);

  @Autowired
  ArchivalInfoPackageRepository repo;

  @PostConstruct
  @Override
  protected void loadConfig() {
  }

  @Transactional
  @JmsListener(destination = "${dlcm.queue.aip.download}")
  public void processAip(AipDownloadMessage aip) {
    log.info("Reading message " + aip.toString());
    ArchivalInfoPackage a = this.getOne(repo, aip.getDlcmId());
    if (a == null) {
      log.error("Cannot find AIP (" + aip.getDlcmId() + ")");
      return;
    }
    try {
      if (a.getInfo().getStatus().getValue() == StatusValue.InProgress) {
        if (a.getDataFiles().size() == 0) {
          AipDataFile aipPackage = new AipDataFile();
          DataFile df = new DataFile();
          df.setSourceData(URI.create(this.dlcmConfig.getArchivalStorageUrl() + "/" + RESTName.AIP + "/" + a.getDlcmId() + "/" + RESTName.DL));
          df.setDataCategory(DataCategory.Package);
          df.setDataType(DataCategory.InformationPackage);
          aipPackage.setDataFile(df);
          a.addItem(aipPackage);
          a.setPackageStatus(StatusValue.Downloading);
        }
      } else if (a.getInfo().getStatus().getValue() == StatusValue.Downloading) {
        if (a.isAllDataFileReady()) {
          a.setPackageStatus(StatusValue.Completed);
        }
      }
    } catch (Exception e) {
      log.error("Error in processing AIP Download", e);
      a.setErrorStatus(e.getMessage());
    }
    if (repo.save(a) == null) {
      log.error("AIP cannot be saved: dlcmId=" + a.getDlcmId() + " status=" + a.getInfo().getStatus().getValue().toString());
    }
    log.info("AIP download: dlcmId=" + a.getDlcmId() + " status=" + a.getInfo().getStatus().getValue().toString());
  }
}
