package ch.dlcm.service;

import java.net.URI;
import java.util.List;

import javax.annotation.PostConstruct;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jms.annotation.JmsListener;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import ch.dlcm.config.ContextHelper;
import ch.dlcm.message.DipMessage;
import ch.dlcm.message.OrderMessage;
import ch.dlcm.model.DataCategory;
import ch.dlcm.model.DataFile;
import ch.dlcm.model.Status.StatusValue;
import ch.dlcm.model.oais.ArchivalInfoPackage;
import ch.dlcm.model.oais.DipDataFile;
import ch.dlcm.model.oais.DisseminationInfoPackage;
import ch.dlcm.model.oais.RepresentationInfo;
import ch.dlcm.repository.DisseminationInfoPackageRepository;

@Component
public class DipMgmt extends DLCMService {
  private static final Logger log = Logger.getLogger(DipMgmt.class);

  @Autowired
  DisseminationInfoPackageRepository repo;

  @PostConstruct
  @Override
  protected void loadConfig() {
  }

  @Transactional
  @JmsListener(destination = "${dlcm.queue.dip}")
  public void processDip(DipMessage dipPayload) {
    log.info("Reading message " + dipPayload.toString());
    DisseminationInfoPackage dip = this.getOne(repo, dipPayload.getDlcmId());
    if (dip == null) {
      log.error("Cannot find DIP (" + dipPayload.getDlcmId() + ")");
      return;
    }
    while (dip.getInfo().getStatus().getValue() != StatusValue.InError
            && dip.getInfo().getStatus().getValue() != StatusValue.Completed) {
      try {
        switch (dip.getInfo().getStatus().getValue()) {
          case InPreparation:
            if (dip.isAllDataFileReady()) {
              dip.setPackageStatus(StatusValue.Completed);
            }
            break;
          case InProgress:
          case Ready:
          case Checked:
          default:
            break;
        }
      } catch (Exception e) {
        log.error("Error in  processing DIP (" + dipPayload.getDlcmId() + ")", e);
        inError(dip, e.getMessage());
      }
      if (repo.save(dip) == null) {
        log.error("DIP cannot be saved: dlcmId=" + dip.getDlcmId() + " status="
                + dip.getInfo().getStatus().getValue().toString());
      }
      if (dip.getInfo().getStatus().getValue() == StatusValue.Completed) {
        ContextHelper.getPublisher().publishEvent(new OrderMessage());
      }
    }
    log.info("DIP processed: dlcmId=" + dip.getDlcmId() + " status="
            + dip.getInfo().getStatus().getValue().toString());
  }

  public DisseminationInfoPackage createDIP(String id, String name, List<ArchivalInfoPackage> aips, URI orderUri) {

    try {
      // Create DIP
      DisseminationInfoPackage dip = new DisseminationInfoPackage();
      RepresentationInfo info = new RepresentationInfo();
      info.setName(messageService.get("access.dip.name", new Object[] { name }));
      info.setDescription(messageService.get("access.dip.description"));
      dip.setInfo(info);
      // AIP links
      for (ArchivalInfoPackage aip : aips) {
        dip.getAipIds().add(aip.getDlcmId());
      }
      // DIP data file
      DipDataFile dipPackage = new DipDataFile();
      DataFile df = new DataFile();
      df.setSourceData(orderUri);
      df.setDataCategory(DataCategory.Package);
      df.setDataType(DataCategory.InformationPackage);
      dipPackage.setDataFile(df);
      dip.getDataFiles().add(dipPackage);

      // Save DIP
      DisseminationInfoPackage resDip = repo.save(dip);
      if (resDip == null) {
        log.error("Cannot create DIP for order (" + id + ")");
      }
      return resDip;
    } catch (Exception e) {
      log.error("Error in creating DIP for order (" + id + ")", e);
    }
    return null;
  }

  private void inError(DisseminationInfoPackage dip, String msg) {
    dip.setErrorStatus(msg);
    log.error(msg + " for DIP " + dip.getDlcmId());
  }
}
