package ch.dlcm.service;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.PostConstruct;

import org.apache.log4j.Logger;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Component;

import ch.dlcm.DLCMConstants;
import ch.dlcm.exception.DLCMException;
import ch.dlcm.exception.DLCMResourceNotFoundException;
import ch.dlcm.model.ArchiveMetadata;
import ch.dlcm.model.Order;
import ch.dlcm.model.OrderArchive;
import ch.dlcm.model.oais.ArchivalInfoPackage;
import ch.dlcm.rest.RESTCollection;
import ch.dlcm.rest.RESTPage;
import ch.dlcm.tool.CollectionTool;

@Component
public class SearchMgmt extends DLCMService {
  private static final Logger log = Logger.getLogger(SearchMgmt.class);

  @PostConstruct
  @Override
  protected void loadConfig() {
  }

  public RESTCollection<OrderArchive> search(Order order, boolean allInfo, Pageable pageable)
          throws DLCMException {
    RESTCollection<OrderArchive> collection = null;
    try {
      switch (order.getQueryType()) {
        case ADVANCED:
          // TODO: ES advanced query
          break;
        case DIRECT:
          collection = new RESTCollection<>(this.getAIPs(order.getQuery(), allInfo));
          break;
        case IMAGE:
          // TODO: IIIF query
          break;
        case SIMPLE:
          collection = this.search(order.getQuery(), allInfo, pageable);
          break;
        default:
          break;
      }
    } catch (Exception e) {
      String message = "Query (" + order.getQueryType().getLabel() + "=" + order.getQuery()
              + ") error: " + e.getMessage();
      log.error(message, e);
      throw new DLCMException(message, e);
    }
    log.info("Query (" + order.getQueryType().getLabel() + "=" + order.getQuery()
            + ") submitted: result with " + collection.get_page().getCurrentItems() + " item(s) of "
            + collection.get_page().getTotalItems());
    return collection;
  }

  private RESTCollection<OrderArchive> search(String query, boolean aip, Pageable pageable)
          throws DLCMException {
    String jsonResult = this.getMetatadataList(query, pageable.getPageNumber(),
            pageable.getPageSize());
    List<OrderArchive> list = this
            .getMetadata(CollectionTool.getList(jsonResult, ArchiveMetadata.class), aip);
    RESTPage page = CollectionTool.getPage(jsonResult);
    RESTCollection<OrderArchive> collection = new RESTCollection<>(list, page);
    return collection;
  }

  private List<OrderArchive> getMetadata(List<ArchiveMetadata> mds, boolean aip)
          throws DLCMException {
    List<OrderArchive> list = new ArrayList<>();

    for (ArchiveMetadata md : mds) {
      if (aip) {
        ArchivalInfoPackage archive;
        try {
          archive = this.getAIP(md.getDlcmId());
        } catch (DLCMResourceNotFoundException e) {
          log.error("AIP not found ("+ md.getDlcmId()+")");
          continue;
        }
        list.add(new OrderArchive(archive, md));
      } else {
        list.add(new OrderArchive(md));
      }
    }
    return list;

  }

  private List<OrderArchive> getAIPs(String query, boolean metadata) throws DLCMException {
    List<OrderArchive> list = new ArrayList<>();

    for (String aip : query.split(DLCMConstants.SEPARATOR)) {
      // Find AIP
      ArchivalInfoPackage archive;
      try {
        archive = this.getAIP(aip);
      } catch (DLCMResourceNotFoundException e) {
        log.error("AIP not found ("+ aip+")");
        continue;
      }
      if (metadata) {
        // Find metadata
        ArchiveMetadata md;
        try {
          md = this.getMetatadata(aip);
        } catch (DLCMResourceNotFoundException e) {
          log.error("Archive Metadata not found ("+ aip+")");
          continue;
        }
        list.add(new OrderArchive(archive, md));
      } else {
        list.add(new OrderArchive(archive));
      }
    }
    return list;
  }

  @Deprecated
  public List<OrderArchive> searchAll(Order order, boolean allInfo) throws DLCMException {
    List<OrderArchive> list = new ArrayList<>();
    try {
      switch (order.getQueryType()) {
        case ADVANCED:
          // TODO: ES advanced query
          break;
        case DIRECT:
          list.addAll(this.getAIPs(order.getQuery(), allInfo));
          break;
        case IMAGE:
          // TODO: IIIF query
          break;
        case SIMPLE:
          list.addAll(
                  CollectionTool.toList(this.search(order.getQuery(), allInfo, null).get_data()));
          break;
        default:
          break;
      }
    } catch (Exception e) {
      String message = "Full Query (" + order.getQueryType().getLabel() + "=" + order.getQuery()
              + ") error: " + e.getMessage();
      log.error(message, e);
      throw new DLCMException(message, e);
    }
    log.info("Full Query (" + order.getQueryType().getLabel() + "=" + order.getQuery()
            + ") submitted: result with " + list.size() + " item(s)");
    return list;
  }
}
