package ch.dlcm.config.security;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.annotation.Order;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;

import static ch.dlcm.config.security.OAuthClients.DLCM_UI_CLIENT;
import static ch.dlcm.config.security.OAuthClients.DLCM_UI_APPROVAL;
import static ch.dlcm.config.security.OAuthClients.TEST_CLIENT_ID;
import static ch.dlcm.config.security.OAuthClients.TEST_CLIENT_ID_NO_ADMIN;

@Order(-1)
@Configuration
@EnableWebSecurity
public class AdminWebSecurityConfig extends WebSecurityConfigurerAdapter {

  @Autowired
  public void dlcmlUserDetails(final AuthenticationManagerBuilder auth) throws Exception {
    // @formatter:off
    auth.inMemoryAuthentication()
            // Users
            .withUser("john").password("111").roles("NOADMIN").and()
            .withUser("tom").password("111").roles("ADMIN").and()
            // Clients
            .withUser(DLCM_UI_APPROVAL.getClientId()).password(DLCM_UI_APPROVAL.getSecret()).roles("ADMIN").and()
            .withUser(DLCM_UI_CLIENT.getClientId()).password(DLCM_UI_CLIENT.getSecret()).roles("ADMIN").and()
            // Others clients
            .withUser(TEST_CLIENT_ID.getClientId()).password(TEST_CLIENT_ID.getSecret()).roles("ADMIN").and()
            .withUser(TEST_CLIENT_ID_NO_ADMIN.getClientId()).password(TEST_CLIENT_ID_NO_ADMIN.getSecret()).roles("NOADMIN");
    // @formatter:on
  }

  @Override
  protected void configure(final HttpSecurity http) throws Exception {
    // @formatter:off
    http.authorizeRequests()
          .antMatchers("/dashboard", "/health", "/metrics", "/info", "/logfile").permitAll()
          .antMatchers("/css/**", "/*.png").permitAll()
          .anyRequest().authenticated().and()
//            .formLogin().loginPage("/login_admin").and()
          .httpBasic().and()
          .csrf().disable();
    // @formatter:on
  }

  @Override
  @Bean
  public AuthenticationManager authenticationManagerBean() throws Exception {
    return super.authenticationManagerBean();
  }
}