package ch.dlcm.config.security;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.ClassPathResource;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.oauth2.config.annotation.configurers.ClientDetailsServiceConfigurer;
import org.springframework.security.oauth2.config.annotation.web.configuration.AuthorizationServerConfigurerAdapter;
import org.springframework.security.oauth2.config.annotation.web.configuration.EnableAuthorizationServer;
import org.springframework.security.oauth2.config.annotation.web.configurers.AuthorizationServerEndpointsConfigurer;
import org.springframework.security.oauth2.config.annotation.web.configurers.AuthorizationServerSecurityConfigurer;
import org.springframework.security.oauth2.provider.token.TokenStore;
import org.springframework.security.oauth2.provider.token.store.JwtAccessTokenConverter;
import org.springframework.security.oauth2.provider.token.store.JwtTokenStore;
import org.springframework.security.oauth2.provider.token.store.KeyStoreKeyFactory;

import static ch.dlcm.config.security.OAuthClients.DLCM_UI_CLIENT;
import static ch.dlcm.config.security.OAuthClients.DLCM_UI_APPROVAL;
import static ch.dlcm.config.security.OAuthClients.TEST_CLIENT_ID;
import static ch.dlcm.config.security.OAuthClients.TEST_CLIENT_ID_NO_ADMIN;
import static ch.dlcm.config.security.OAuthGrantType.AUTHORIZATION_CODE;

@Configuration
@EnableAuthorizationServer
public class OAuth2AuthorizationServerConfig extends AuthorizationServerConfigurerAdapter {

  @Autowired
  @Qualifier("authenticationManagerBean")
  private AuthenticationManager authenticationManager;

  @Override
  public void configure(AuthorizationServerEndpointsConfigurer endpoints) throws Exception {
    endpoints
            .tokenStore(tokenStore())
            .accessTokenConverter(accessTokenConverter())
            .authenticationManager(authenticationManager);
  }

  @Bean
  public TokenStore tokenStore() {
    return new JwtTokenStore(accessTokenConverter());
  }

  @Bean
  public JwtAccessTokenConverter accessTokenConverter() {
    JwtAccessTokenConverter converter = new JwtAccessTokenConverter();
    KeyStoreKeyFactory keyStoreKeyFactory =
            new KeyStoreKeyFactory(new ClassPathResource("mytest.jks"), "mypass".toCharArray());
    converter.setKeyPair(keyStoreKeyFactory.getKeyPair("mytest"));
    return converter;
  }

  @Override
  public void configure(final ClientDetailsServiceConfigurer clients) throws Exception {// @formatter:off

    clients
      .inMemory()
            .withClient(DLCM_UI_CLIENT.getClientId())
            .authorizedGrantTypes(AUTHORIZATION_CODE)
            .scopes("read", "write")
            .autoApprove(true)
            .accessTokenValiditySeconds(36000)
            .redirectUris("http://localhost:16106/dlcm/ingest")
       .and()
            .withClient(DLCM_UI_APPROVAL.getClientId())
            .authorizedGrantTypes(AUTHORIZATION_CODE)
            .scopes("read", "write")
            .accessTokenValiditySeconds(36000)
            .redirectUris("http://localhost:16106/dlcm/ingest")
       .and() // Clients for tests
            .withClient(TEST_CLIENT_ID.getClientId())
            .authorizedGrantTypes(AUTHORIZATION_CODE)
            .scopes("read", "write")
            .autoApprove(true)
            .accessTokenValiditySeconds(36000)
            .redirectUris("https://www.getpostman.com/oauth2/callback ")
        .and()
            .withClient(TEST_CLIENT_ID_NO_ADMIN.getClientId())
            .authorizedGrantTypes(AUTHORIZATION_CODE)
            .scopes("read", "write")
            .accessTokenValiditySeconds(36000)
            .redirectUris("https://www.getpostman.com/oauth2/callback ");
  }

  @Override
  public void configure(final AuthorizationServerSecurityConfigurer oauthServer) throws Exception {
    oauthServer
            .tokenKeyAccess("permitAll()")
            .checkTokenAccess("isAnonymous()");
  }

}
