package ch.dlcm.controller.admin;

import static ch.dlcm.rest.RESTName.IMPORTED;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.linkTo;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.methodOn;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.log4j.Logger;
import org.json.JSONObject;
import org.springframework.hateoas.ResourceSupport;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestPart;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.fasterxml.jackson.databind.ObjectMapper;

import ch.dlcm.controller.ResourceController;
import ch.dlcm.model.License;
import ch.dlcm.repository.LicenseRepository;
import ch.dlcm.rest.RESTDlcmRoutes;
import ch.dlcm.rest.RESTName;

@RestController
@RequestMapping(RESTDlcmRoutes.ADMIN_LICENSE_PATH)
public class LicenseController extends ResourceController<License> {
    private static final Logger log = Logger.getLogger(LicenseController.class);

    @RequestMapping(value = RESTDlcmRoutes.ADMIN_LICENSE_IMPORT_FILE_PATH, method = RequestMethod.POST)
    public HttpEntity<String> importJson(@RequestPart("file") MultipartFile file) {
        try {
            // Try import json array License format
            List<License> licenses = Arrays.asList(new ObjectMapper().readValue(file.getBytes(), License[].class));
            return createList(licenses);
        } catch (IOException e) {
            log.error("Error importing JSON", e);
        }
        return new HttpEntity(HttpStatus.INTERNAL_SERVER_ERROR);
    }

    @RequestMapping(value = RESTDlcmRoutes.ADMIN_OPEN_LICENSE_IMPORT_FILE_PATH, method = RequestMethod.POST)
    public HttpEntity<String> importOpenLicense(@RequestPart("file") MultipartFile file) {
        try {
            // Try all.json importation
            License.OpenLicense openLicense = new ObjectMapper().readValue(file.getBytes(), License.OpenLicense.class);
            List<License> licenses = new ArrayList(openLicense.getLicenses().values());
            return createList(licenses);
        } catch (Exception e) {
            log.error("Can not deserialize instance for file \"" + file.getOriginalFilename() + "\"", e);
        }

        try {
            // Try ckan.json format importation
            List<License> licenses = Arrays.asList(new ObjectMapper().readValue(file.getBytes(), License.OpenLicense[].class));
            return createList(licenses);
        } catch (IOException e) {
            log.error("Can not deserialize instance for file \"" + file.getOriginalFilename() + "\"", e);
        }
        return new HttpEntity(HttpStatus.INTERNAL_SERVER_ERROR);
    }


    @RequestMapping(value = RESTDlcmRoutes.ADMIN_LICENSE_IMPORT_LIST_PATH, method = RequestMethod.POST)
    public HttpEntity<String> createList(@RequestBody List<License> listPayload) {
        for (License payload : listPayload) {
            License license = ((LicenseRepository)itemRepository).findByOpenLicenseId(payload.getOpenLicenseId());
            if ( license != null && license.update(payload)){
                itemRepository.save(license);
            }else if (license == null){
                itemRepository.save(payload);
            }
        }
        return new ResponseEntity<>(new JSONObject().put(IMPORTED, listPayload.size()).toString(), HttpStatus.CREATED);
    }

    @Override
    protected <W extends ResourceSupport> boolean addOthersLinks(W licenseResource) {
        licenseResource.add(linkTo(methodOn(this.getClass()).importOpenLicense(null)).withRel(RESTName.IMPORT));
        licenseResource.add(linkTo(methodOn(this.getClass()).importJson(null)).withRel(RESTName.IMPORT));
        licenseResource.add(linkTo(methodOn(this.getClass()).createList(null)).withRel(RESTName.IMPORT));
        return true;
    }

}
