package ch.dlcm.controller.admin;

import ch.dlcm.controller.AssociationController;
import ch.dlcm.model.OrganizationalUnit;
import ch.dlcm.model.OrganizationalUnitPersonRole;
import ch.dlcm.model.Person;
import ch.dlcm.model.Role;
import ch.dlcm.model.display.PersonRole;
import ch.dlcm.model.display.RessourceRole;
import ch.dlcm.rest.*;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import static ch.dlcm.rest.RESTDlcmRoutes.ADMIN_UNIT_PERSON_PATH;

@RestController
@RequestMapping(ADMIN_UNIT_PERSON_PATH)
public class OrganizationalUnitPersonController extends AssociationController<OrganizationalUnit, Person> {

  private static final Logger log = Logger.getLogger(OrganizationalUnitPersonController.class);

  @Autowired
  protected RESTRepository<Role> roleRepository;

  @Override
  protected Class<Person> getSubResourceClass() {
    return Person.class;
  }

  @Override
  protected Class<OrganizationalUnit> getMainResourceClass() {
    return OrganizationalUnit.class;
  }

  @Override
  @RequestMapping(method = RequestMethod.GET)
  public HttpEntity<RESTCollection<Person>> listAll(@PathVariable String parentid, Pageable pageable) {
    if (pageable.getSort() != null)
      throw new UnsupportedOperationException(messageService.get("error.sort.unsupported",
              new Object[] { getMainResourceClass().toString(), getSubResourceClass().toString() }));
    OrganizationalUnit organizationalUnit = itemRepository.findOne(parentid);
    if (organizationalUnit == null)
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    Page<Person> list = itemRepository.findByParentId(parentid, getMainResourceClass(),
            getSubResourceClass(), pageable);
    List<RessourceRole> personRoles = new ArrayList<>();
    for (final Person person : list) {
      PersonRole personRole = new PersonRole(person, organizationalUnit);
      addLinks(parentid, personRole);
      personRoles.add(personRole);
      addRoleLinks(RoleController.class, personRole);
    }
    RESTCollection<Person> collection = processResourceRolesAndLinks(personRoles, parentid, pageable);
    return new ResponseEntity(collection, HttpStatus.OK);
  }

  @RequestMapping(value = "/{id}", method = RequestMethod.GET)
  public HttpEntity<Person> list(@PathVariable String parentid, @PathVariable String id) {
    OrganizationalUnit organizationalUnit = itemRepository.findOne(parentid);
    if (organizationalUnit == null) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }
    Person person = itemRepository.findByParentIdAndChildId(parentid, getMainResourceClass(), getSubResourceClass(), id);
    PersonRole personRole = new PersonRole(person, organizationalUnit);

    if (person != null) {
      addLinks(parentid, personRole);
      addRoleLinks(RoleController.class, personRole);
      return new ResponseEntity<>(personRole, HttpStatus.OK);
    }
    return new ResponseEntity<>(HttpStatus.NOT_FOUND);
  }


  @RequestMapping(method = RequestMethod.POST)
  @Override
  public HttpEntity<List<Person>> create(@PathVariable String parentid, @RequestBody String[] ids) {
    return create(parentid, ids, null);
  }

  //TODO change de "value" of this mapping later
  @RequestMapping(value = "/" + RESTName.ROLE, method = RequestMethod.POST)
  public HttpEntity<List<Person>> create(@PathVariable String parentid, @RequestBody Map<String, String[]> resourcesIds) {
    return create(parentid, resourcesIds.get(RESTName.PERSON), resourcesIds.get(RESTName.ROLE));
  }

  private HttpEntity<List<Person>> create(@NotNull String parentid, @NotNull String[] personIds, @NotNull String[] roleNames) {

    List<Role> roles = getRolesByName(roleNames);
    OrganizationalUnit organizationalUnit = itemRepository.findOne(parentid);
    if (organizationalUnit == null || roles.isEmpty()) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }

    List<Person> list = new ArrayList<>();
    for (String id : personIds) {
      Person childItem = childItemRepository.findOne(id);
      if (childItem == null) {
        return new ResponseEntity<>(HttpStatus.NOT_FOUND);
      }
      organizationalUnit.addPerson(childItem, roles);
      list.add(childItem);
      childItemRepository.save(childItem);
    }
    OrganizationalUnit item = itemRepository.save(organizationalUnit);
    if (item != null) {
      return new ResponseEntity<>(list, HttpStatus.CREATED);
    }
    return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
  }

  // TODO ducplicated method (PersonOrganizationalUnitController.getRolesByName()), should extract later
  private List<Role> getRolesByName(String[] roleNames) {
    Role defaultRole = new Role();
    List<Role> roles = new ArrayList<>();
    if (roleNames == null || roleNames.length == 0) {
      defaultRole.setName(Role.DEFAULT_ROLE);
      defaultRole = roleRepository.findOne(defaultRole.getSpecification());
      roles.add(defaultRole);
    } else {
      for (String roleName : roleNames) {
        Role role = new Role();
        role.setName(roleName);
        role = roleRepository.findOne(role.getSpecification());
        if (role == null) {
          log.error("Unknown role: " + roleName);
        }
        roles.add(role);
      }
    }
    return roles;
  }

  @RequestMapping(value = "/{personId}/" + RESTName.ROLE + "/{roleName}", method = RequestMethod.DELETE)
  public HttpStatus delete(@PathVariable("parentid") String projectId, @PathVariable String personId, @PathVariable String roleName) {
    Role role = new Role();
    role.setName(roleName);
    role = roleRepository.findOne(role.getSpecification());
    if (role == null) {
      return HttpStatus.NOT_FOUND;
    }
    OrganizationalUnitPersonRole organizationalUnitPersonRole = itemRepository.findByCompositeId(OrganizationalUnitPersonRole.class, projectId, personId, role.getDlcmId());
    if (organizationalUnitPersonRole == null) {
      return HttpStatus.NOT_FOUND;
    }
    return delete(organizationalUnitPersonRole);
  }

  private HttpStatus delete(OrganizationalUnitPersonRole organizationalUnitPersonRole) {
    OrganizationalUnit organizationalUnit = organizationalUnitPersonRole.getOrganizationalUnit();
    if (organizationalUnit.removeItem(organizationalUnitPersonRole)) {
      organizationalUnit = itemRepository.save(organizationalUnit);
      if (organizationalUnit != null) {
        return HttpStatus.OK;
      }
    }
    return HttpStatus.GONE;
  }

}
