package ch.dlcm.controller.admin;

import ch.dlcm.model.OrganizationalUnitPersonRole;
import ch.dlcm.model.Role;
import ch.dlcm.model.display.OrganizationalUnitRole;
import ch.dlcm.model.display.RessourceRole;
import ch.dlcm.rest.*;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import ch.dlcm.controller.AssociationController;
import ch.dlcm.model.OrganizationalUnit;
import ch.dlcm.model.Person;

import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import static ch.dlcm.rest.RESTDlcmRoutes.ADMIN_PERSON_UNIT_PATH;

@RestController
@RequestMapping(ADMIN_PERSON_UNIT_PATH)
public class PersonOrganizationalUnitController extends AssociationController<Person, OrganizationalUnit> {

  private static final Logger log = Logger.getLogger(PersonOrganizationalUnitController.class);

  @Autowired
  protected RESTRepository<Role> roleRepository;

  @Override
  protected Class<OrganizationalUnit> getSubResourceClass() {
    return OrganizationalUnit.class;
  }

  @Override
  protected Class<Person> getMainResourceClass() {
    return Person.class;
  }

  @Override
  @RequestMapping(method = RequestMethod.GET)
  public HttpEntity<RESTCollection<OrganizationalUnit>> listAll(@PathVariable String parentid, Pageable pageable) {
    if (pageable.getSort() != null)
      throw new UnsupportedOperationException(messageService.get("error.sort.unsupported",
              new Object[]{getMainResourceClass().toString(), getSubResourceClass().toString()}));
    Person person = itemRepository.findOne(parentid);
    if (person == null)
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    Page<OrganizationalUnit> list = itemRepository.findByParentId(parentid, getMainResourceClass(),
            getSubResourceClass(), pageable);
    List<RessourceRole> personRoles = new ArrayList<>();
    for (final OrganizationalUnit project : list) {
      OrganizationalUnitRole organizationalUnitRole = new OrganizationalUnitRole(person, project);
      addLinks(parentid, organizationalUnitRole);
      personRoles.add(organizationalUnitRole);
      addRoleLinks(RoleController.class, organizationalUnitRole);
    }
    RESTCollection<OrganizationalUnit> collection = processResourceRolesAndLinks(personRoles, parentid, pageable);
    return new ResponseEntity(collection, HttpStatus.OK);
  }

  @RequestMapping(value = "/{id}", method = RequestMethod.GET)
  public HttpEntity<OrganizationalUnit> list(@PathVariable String parentid, @PathVariable String id) {
    Person person = itemRepository.findOne(parentid);
    if (person == null) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }
    OrganizationalUnit organizationalUnit = itemRepository.findByParentIdAndChildId(parentid, getMainResourceClass(), getSubResourceClass(), id);
    OrganizationalUnitRole organizationalUnitRole = new OrganizationalUnitRole(person, organizationalUnit);

    if (organizationalUnit != null) {
      addLinks(parentid, organizationalUnitRole);
      addRoleLinks(RoleController.class, organizationalUnitRole);
      return new ResponseEntity<>(organizationalUnitRole, HttpStatus.OK);
    }
    return new ResponseEntity<>(HttpStatus.NOT_FOUND);
  }

  @RequestMapping(method = RequestMethod.POST)
  @Override
  public HttpEntity<List<OrganizationalUnit>> create(@PathVariable String parentid, @RequestBody String[] ids) {
    return create(parentid, ids, null);
  }

  //TODO change de "value" of this mapping later
  @RequestMapping(value = "/" + RESTName.ROLE, method = RequestMethod.POST)
  public HttpEntity<List<OrganizationalUnit>> create(@PathVariable String parentid, @RequestBody Map<String, String[]> resourcesIds) {
    return create(parentid, resourcesIds.get(RESTName.UNIT), resourcesIds.get(RESTName.ROLE));
  }

  private HttpEntity<List<OrganizationalUnit>> create(@NotNull String parentid, @NotNull String[] projectIds, @NotNull String[] roleNames) {
    List<Role> roles = getRoles(roleNames);
    Person person = itemRepository.findOne(parentid);
    if (person == null || roles.isEmpty()) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }
    List<OrganizationalUnit> list = new ArrayList<>();
    for (String id : projectIds) {
      OrganizationalUnit childItem = childItemRepository.findOne(id);
      if (childItem == null) {
        return new ResponseEntity<>(HttpStatus.NOT_FOUND);
      }
      person.addProject(childItem, roles);
      list.add(childItem);
      childItemRepository.save(childItem);
    }
    Person item = itemRepository.save(person);
    if (item != null) {
      return new ResponseEntity<>(list, HttpStatus.CREATED);
    }
    return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
  }

  private List<Role> getRoles(String[] roleNames) {
    Role defaultRole = new Role();
    List<Role> roles = new ArrayList<>();
    if (roleNames == null || roleNames.length == 0) {
      defaultRole.setName(Role.DEFAULT_ROLE);
      defaultRole = roleRepository.findOne(defaultRole.getSpecification());
      roles.add(defaultRole);
    } else {
      for (String roleName : roleNames) {
        Role role = new Role();
        role.setName(roleName);
        role = roleRepository.findOne(role.getSpecification());
        if (role == null) {
          log.error("Unknown role: " + roleName);
        }
        roles.add(role);
      }
    }
    return roles;
  }

  // TODO should be extract later
  @RequestMapping(value = "/{projectId}/" + RESTName.ROLE + "/{roleName}", method = RequestMethod.DELETE)
  public HttpStatus delete(@PathVariable("parentid") String personId, @PathVariable String projectId, @PathVariable String roleName) {
    Role role = new Role();
    role.setName(roleName);
    role = roleRepository.findOne(role.getSpecification());
    if (role == null) {
      return HttpStatus.NOT_FOUND;
    }
    OrganizationalUnitPersonRole organizationalUnitPersonRole = itemRepository.findByCompositeId(OrganizationalUnitPersonRole.class, projectId, personId, role.getDlcmId());
    if (organizationalUnitPersonRole == null) {
      return HttpStatus.NOT_FOUND;
    }
    return delete(organizationalUnitPersonRole);
  }

  private HttpStatus delete(OrganizationalUnitPersonRole organizationalUnitPersonRole) {
    Person person = organizationalUnitPersonRole.getPerson();
    if (person.removeItem(organizationalUnitPersonRole)) {
      person = itemRepository.save(person);
      if (person != null) {
        return HttpStatus.OK;
      }
    }
    return HttpStatus.GONE;
  }

}
