package ch.dlcm.test.doc;

import static ch.dlcm.test.executor.DLCMExecutorTestDoc.doDocumentationFieldsCycle;
import static ch.dlcm.test.executor.DLCMExecutorTestDoc.doDocumentationLinksCycle;
import static ch.dlcm.test.executor.DLCMExecutorTestDoc.doErrorUrlRestLink;
import static ch.dlcm.test.executor.DLCMExecutorTestDoc.doPerformPostRequestCycle;
import static ch.dlcm.test.executor.DLCMExecutorTestDoc.doPerformRequestCycle;
import static ch.dlcm.test.executor.DLCMExecutorTestDoc.doTestsExpectedCycle;
import static ch.dlcm.test.executor.DLCMTestDocHelper.APPLICATION;
import static ch.dlcm.test.executor.DLCMTestDocHelper.HAL_JSON_MEDIA_TYPE;
import static ch.dlcm.test.executor.DLCMTestDocHelper.HTML_MEDIA_TYPE;
import static org.springframework.restdocs.mockmvc.MockMvcRestDocumentation.documentationConfiguration;

import java.util.concurrent.ThreadLocalRandom;

import org.json.JSONObject;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.HttpStatus;
import org.springframework.restdocs.JUnitRestDocumentation;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.test.context.web.WebAppConfiguration;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.web.context.WebApplicationContext;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;

import ch.dlcm.rest.RESTAction;
import ch.dlcm.rest.RESTName;
import ch.dlcm.test.doc.matcher.administration.AdministrationDocMatcher;
import ch.dlcm.test.doc.matcher.ingestion.AdminDocMatcher;
import ch.dlcm.test.doc.matcher.ingestion.AdminInstitutionDocMatcher;
import ch.dlcm.test.executor.DLCMTestDocHelper;

@ActiveProfiles({"test"})
@RunWith(SpringJUnit4ClassRunner.class)
@SpringBootTest
@WebAppConfiguration
public class AdministrationRestTestsAndDocumentation {

    @Rule
    public final JUnitRestDocumentation restDocumentation = new JUnitRestDocumentation("target/generated-snippets");
    @Autowired
    private WebApplicationContext context;
    private MockMvc mockMvc;


    @Before
    public void setUp() {
        this.mockMvc = MockMvcBuilders.webAppContextSetup(this.context)
                .apply(documentationConfiguration(this.restDocumentation)).build();

    }

    @Test
    public void initAdministrationTestAndDocs() throws Exception {

        MvcResult result = doPerformRequestCycle(mockMvc, DLCMTestDocHelper.APPLICATION_ROOT, HAL_JSON_MEDIA_TYPE, HttpStatus.OK).andReturn();
        JsonNode links = new ObjectMapper().readTree(result.getResponse().getContentAsString()).get("_links");

        // Execute ingestion "_links"
        DLCMTestDocHelper.getAdministrationRootActions().forEach(action -> {
            try {
                if (links.has(action)) {
                    String url = links.get(action).get("href").asText();
                    switch (action) {
                        case RESTAction.SELF:
                            generateAdministrationDoc(DLCMTestDocHelper.APPLICATION_ROOT);
                            break;
                        case RESTName.ADMIN:
                            generateAdminDoc(url);
                            generateAdminInstitutionDoc(url + "/" + RESTName.INSTITUTION);
                            generateAdminInstitutionPostDoc(url + "/" + RESTName.INSTITUTION);
                            generateAdminPeopleDoc(url + "/" + RESTName.PERSON);
                            break;
                        case RESTAction.DASHBOARD:
                            generateDashboardDoc(url);
                            break;
                    }
                }
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        });
    }

    //	================================
    //	DASHBOARD
    //	================================

    private void generateDashboardDoc(String url) throws Exception {
        doPerformRequestCycle(mockMvc, url, HTML_MEDIA_TYPE, HttpStatus.OK);
    }

    //	================================
    //	ADMINISTRATION
    //	================================

    public MvcResult generateAdministrationDoc(String url) throws Exception {
        ResultActions request = doPerformRequestCycle(mockMvc, url, HAL_JSON_MEDIA_TYPE, HttpStatus.OK);
        doTestsExpectedCycle(request, AdministrationDocMatcher.getAdministrationRootExpectedMatchers());
        doDocumentationLinksCycle(request, APPLICATION, AdministrationDocMatcher.getAdministrationRootMapLinksDescription());
        return request.andReturn();
    }


    //	================================
    //	ADMIN
    //	================================

    private ResultActions createAnyPeople() throws Exception {
        JSONObject people = new JSONObject().put("firstName", "any first name").put("lastName", "any last name");
        return doPerformPostRequestCycle(mockMvc, DLCMTestDocHelper.APPLICATION_ROOT + RESTName.ADMIN + "/" + RESTName.PERSON, HAL_JSON_MEDIA_TYPE, HttpStatus.CREATED, people.toString());
    }

    private void generateAdminDoc(String url) throws Exception {
        ResultActions request = doPerformRequestCycle(mockMvc, url, HAL_JSON_MEDIA_TYPE, HttpStatus.OK);
        doTestsExpectedCycle(request, AdminDocMatcher.getAdminExpectedMatchers());
        doDocumentationFieldsCycle(request, RESTName.ADMIN, AdminDocMatcher.getAdminMapFieldsDescription());
        doDocumentationLinksCycle(request, RESTName.ADMIN, AdminDocMatcher.getAdminMapLinksDescription());
    }

    private void generateAdminInstitutionDoc(String url) throws Exception {
        ResultActions request = doPerformRequestCycle(mockMvc, url, HAL_JSON_MEDIA_TYPE, HttpStatus.OK);
        doTestsExpectedCycle(request, AdminInstitutionDocMatcher.getAdminInstitutionExpectedMatchers());
        doDocumentationFieldsCycle(request, RESTName.INSTITUTION, AdminInstitutionDocMatcher.getAdminInstitutionMapFieldsDescription());
        doDocumentationLinksCycle(request, RESTName.ADMIN, AdminInstitutionDocMatcher.getAdminInstitutionMapLinksDescription());
    }

    private void generateAdminInstitutionPostDoc(String url) throws Exception {
        // Docs for institutions with post return
        String institutionName = "mockmvc" + ThreadLocalRandom.current().nextInt();
        JSONObject institution = new JSONObject().put("name", institutionName).put("description","Description de " + institutionName);
        ResultActions request = doPerformPostRequestCycle(mockMvc, url, HAL_JSON_MEDIA_TYPE, HttpStatus.CREATED, institution.toString());
		doTestsExpectedCycle(request, AdminInstitutionDocMatcher.getAdminInstitutionGetORPostExpectedMatchers());
		doDocumentationFieldsCycle(request, RESTName.INSTITUTION, AdminInstitutionDocMatcher.getAdminInstitutionPostMapFieldsDescription());
		// Docs for institution getting id after post
		String institutionId = String.valueOf(new JSONObject(request.andReturn().getResponse().getContentAsString()).get("dlcmId"));
        request = doPerformRequestCycle(mockMvc, url + "/" + institutionId, HAL_JSON_MEDIA_TYPE, HttpStatus.OK);
        doTestsExpectedCycle(request, AdminInstitutionDocMatcher.getAdminInstitutionGetORPostExpectedMatchers());
        doDocumentationLinksCycle(request, RESTName.INSTITUTION, AdminInstitutionDocMatcher.getAdminInstitutionsPostMapLinksDescription());
    }

    private void generateAdminPeopleDoc(String url) {
        // TODO check it: this admin/people return a different hateos links than deposit/{depoId}/people/{peopleId}
    }

    @Test
    public void invalidAdministrationRootUrlTest() throws Exception {
        doErrorUrlRestLink(mockMvc, "/dlcm/thisdoesntexists");
    }

}
