package ch.dlcm.controller.archivalstorage;

import static org.springframework.hateoas.mvc.ControllerLinkBuilder.linkTo;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.methodOn;

import java.util.ArrayList;
import java.util.List;

import org.springframework.core.io.InputStreamResource;
import org.springframework.hateoas.ResourceSupport;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import ch.dlcm.config.ContextHelper;
import ch.dlcm.controller.ResourceController;
import ch.dlcm.exception.DLCMException;
import ch.dlcm.message.AipMessage;
import ch.dlcm.model.DataCategory;
import ch.dlcm.model.Status;
import ch.dlcm.model.Status.StatusType;
import ch.dlcm.model.Status.StatusValue;
import ch.dlcm.model.oais.AipDataFile;
import ch.dlcm.model.oais.ArchivalInfoPackage;
import ch.dlcm.rest.RESTAction;
import ch.dlcm.rest.RESTName;
import ch.dlcm.rest.RESTResult;
import ch.dlcm.rest.RESTResult.ActionStatus;

@RestController
@RequestMapping("/" + RESTName.ARCHIVALSTORAGE + "/" + RESTName.AIP)
public class AIPController extends ResourceController<ArchivalInfoPackage> {

	@RequestMapping(value = "/{id}/" + RESTName.DL, method = RequestMethod.GET)
	public HttpEntity<InputStreamResource> getFiles(@PathVariable String id) throws DLCMException {
		ArchivalInfoPackage item = itemRepository.findOne(id);
		if (item == null) {
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}
		return download(item.getClass().getName(), id, item.getDownloadUri(), id+".aip");
	}

	@RequestMapping(value = "/{id}/" + RESTName.RESUME, method = RequestMethod.GET)
	public HttpEntity<RESTResult> resume(@PathVariable String id) {
		RESTResult res = resumePackage(id, itemRepository);
		if (res == null) {
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}
		if (res.getStatus() == ActionStatus.Executed){
			ContextHelper.getPublisher().publishEvent(new AipMessage(id));
		}
		res.add(linkTo(this.getClass(), id).withSelfRel());
		res.add(linkTo(methodOn(this.getClass()).list(id)).withRel(RESTAction.PARENT));
		return new ResponseEntity<>(res, HttpStatus.OK);
	}
	
	@RequestMapping(value = "/{id}/" + RESTName.REINDEX, method = RequestMethod.GET)
	public HttpEntity<RESTResult> reindex(@PathVariable String id) {
		RESTResult res = reindexPackage(id, itemRepository);
		if (res == null) {
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}
		if (res.getStatus() == ActionStatus.Executed){
			ContextHelper.getPublisher().publishEvent(new AipMessage(id));
		}
		res.add(linkTo(methodOn(this.getClass()).resume(id)).withSelfRel());
		res.add(linkTo(methodOn(this.getClass()).list(id)).withRel(RESTAction.PARENT));
		return new ResponseEntity<>(res, HttpStatus.OK);
	}

	@RequestMapping(value = "/{id}/" + RESTName.UL, method = RequestMethod.POST)
	public HttpEntity<List<AipDataFile>> sendFiles(@PathVariable String id,
			@RequestParam("file") MultipartFile file) throws DLCMException{
		ArchivalInfoPackage item = itemRepository.findOne(id);
		if (item == null) {
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}
		if (item.getInfo().getStatus().getValue() == StatusValue.Completed 
			 || item.getInfo().getStatus().getValue() == StatusValue.Stored) {
			throw new DLCMException(messageService.get("archival.aip.completed", new Object[] { id }));
		}		
		AipDataFile df = new AipDataFile();
		List<MultipartFile> files = new ArrayList<>();
		List<String> dataCategories = new ArrayList<>();
		List<String> dataTypes = new ArrayList<>();
		files.add(file);
		dataCategories.add(DataCategory.Package.name());
		dataTypes.add(DataCategory.InformationPackage.name());
		return upload(id, df, files, dataCategories, dataTypes, null);
	}

	@RequestMapping(value = "/" + RESTName.STATUS, method = RequestMethod.GET)
	public HttpEntity<StatusValue[]> listStatus() {
		return new ResponseEntity<>(Status.listStatusValues(StatusType.Package), HttpStatus.OK);
	}

	@Override
	protected <W extends ResourceSupport> boolean addOthersLinks(W w) {
		w.add(linkTo(methodOn(this.getClass()).listStatus()).withRel(RESTAction.VALUES));
		return true;
	}

}