package ch.dlcm.controller.archivalstorage;

import static org.springframework.hateoas.mvc.ControllerLinkBuilder.linkTo;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.methodOn;

import java.util.List;

import org.springframework.hateoas.ResourceSupport;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import ch.dlcm.controller.CompositionController;
import ch.dlcm.model.DataCategory;
import ch.dlcm.model.Status;
import ch.dlcm.model.Status.StatusType;
import ch.dlcm.model.Status.StatusValue;
import ch.dlcm.model.oais.AipDataFile;
import ch.dlcm.model.oais.ArchivalInfoPackage;
import ch.dlcm.rest.RESTAction;
import ch.dlcm.rest.RESTName;
import ch.dlcm.rest.RESTResult;

@RestController
@RequestMapping("/" + RESTName.ARCHIVALSTORAGE + "/" + RESTName.AIP + "/{parentid}/" + RESTName.DATAFILE)
public class AipDataFileController extends CompositionController<ArchivalInfoPackage, AipDataFile> {

	@Override
	protected Class<AipDataFile> getSubResourceClass() {
		return AipDataFile.class;
	}
	
	@Override
    protected Class<ArchivalInfoPackage> getMainResourceClass() {
        return ArchivalInfoPackage.class;
    }
	
	@RequestMapping(value = "/{id}/" + RESTName.RESUME, method = RequestMethod.GET)
	public HttpEntity<RESTResult> resume(@PathVariable String parentid, @PathVariable String id) {
		RESTResult res = resumeDataFile(parentid, id, itemRepository, getMainResourceClass(), getSubResourceClass());
		if (res == null){
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}
		res.add(linkTo(methodOn(this.getClass()).resume(parentid, id)).withSelfRel());
		res.add(linkTo(methodOn(this.getClass()).list(parentid, id)).withRel(RESTAction.PARENT));
		return new ResponseEntity<>(res, HttpStatus.OK);
	}
	
	@RequestMapping(value = "/"+RESTName.STATUS, method = RequestMethod.GET)
	public HttpEntity<StatusValue[]> listStatus(@PathVariable String parentid) {
	    return new ResponseEntity<>(Status.listStatusValues(StatusType.DataFile), HttpStatus.OK);
	}
	
	@RequestMapping(value = "/"+RESTName.DATACATEGORY, method = RequestMethod.GET)
	public HttpEntity<List<DataCategory>> listDataCategory(@PathVariable String parentid) {
	    return new ResponseEntity<>(DataCategory.listCategories(), HttpStatus.OK);
	}
	
	@RequestMapping(value = "/" + RESTName.DATATYPE, method = RequestMethod.GET)
	public HttpEntity<List<DataCategory>> listDataType(@PathVariable String parentid, @RequestParam(defaultValue=RESTName.ALL) String category) {
		return new ResponseEntity<>(DataCategory.listSubCategories(category), HttpStatus.OK);
	}

	@Override
	protected<W extends ResourceSupport> boolean addOthersLinks(String parentid, W w) {
		w.add(linkTo(methodOn(this.getClass()).listStatus(parentid)).withRel(RESTAction.VALUES));
		w.add(linkTo(methodOn(this.getClass()).listDataCategory(parentid)).withRel(RESTAction.VALUES));
		w.add(linkTo(methodOn(this.getClass()).listDataType(parentid, RESTName.ALL)).withRel(RESTAction.VALUES));
		return true;
	}
}
