package ch.dlcm.storage.file;

import java.io.File;
import java.util.List;

import javax.annotation.PostConstruct;

import org.springframework.context.annotation.Profile;
import org.springframework.stereotype.Service;

import ch.dlcm.model.ArchiveMetadata;
import ch.dlcm.model.Status.StatusValue;
import ch.dlcm.service.DLCMService;
import ch.dlcm.storage.StorageService;
import ch.dlcm.tool.FileTool;

@Profile("arc-file")
@Service
public class FileStorageService extends DLCMService implements StorageService {

	private String aipPath;
	private String storagePath;

	@PostConstruct
	@Override
	protected void loadConfig() {
		this.aipPath = this.dlcmConfig.getArchivalLocation();
		this.storagePath = this.dlcmConfig.getArchivalStorage();
	}

	private String getAipLocation(String id) {
		return this.aipPath + "/" + id + "/aip";
	}

	private String getArchiveLocation(String id) {
		return this.storagePath + "/" + id;
	}

	@Override
	public boolean checkPackage(String id) {
		File aip = new File(this.getAipLocation(id));
		return aip.exists() && aip.isFile();
	}

	@Override
	public String submitPackage(String id, String name) {
		if (FileTool.copyFile(this.getAipLocation(id),this.getArchiveLocation(id))){
			return id;
		}
		return null;
	}

	@Override
	public StatusValue getStatus(String id) {
		File aip = new File(this.getAipLocation(id));
		File archive = new File(this.getArchiveLocation(id));
		if (!aip.exists()) {
			return StatusValue.InError;
		}
		if (archive.exists()) {
			if (aip.length() == archive.length()) {
				return StatusValue.Stored;
			}
			return StatusValue.InProgress;
		}
		return StatusValue.Ready;
	}

	@Override
	public String getPackageUri(String id) {
		return FileTool.getFileUri(this.getArchiveLocation(id));
	}

	@Override
	public String sendPackage(String id, String name) {
		File aip = new File(this.getAipLocation(id));
		File archive = new File(this.getArchiveLocation(id));
		if (aip.exists() && archive.exists()) {
			if (aip.length() == archive.length()) {
				return FileTool.getFileUri(archive);
			}
		}
		return null;
	}

	@Override
	public List<?> listPackages() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public boolean indexPackage(String id) {
		ArchiveMetadata md = this.getMetadata(System.getProperty("java.io.tmpdir"), this.dlcmConfig.getIndexName(), id, this.getArchiveLocation(id));
		if (md != null) {
			if (this.indexMetadata(id, md)) {
				return true;
			}
		}
		return false;
	}

	@Override
	public boolean checkIndexing(String id) {
		return this.checkMetadata(id);
	}

}
