package ch.dlcm.service.admin;


import static ch.dlcm.rest.RESTDlcmRoutes.ADMIN_LICENSE_IMPORT_FILE_PATH;
import static ch.dlcm.rest.RESTDlcmRoutes.ADMIN_LICENSE_IMPORT_LIST_PATH;

import java.util.List;

import ch.dlcm.tool.RestClientTool;
import org.apache.log4j.Logger;
import org.springframework.core.io.ClassPathResource;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.web.client.HttpStatusCodeException;

import ch.dlcm.model.License;
import ch.dlcm.rest.RESTDlcmRoutes;
import ch.dlcm.rest.RESTName;
import ch.dlcm.service.ResourceServiceREST;
import org.springframework.web.client.RestTemplate;

@Service
public class LicenseServiceREST extends ResourceServiceREST<License> implements LicenseService {
    private static final Logger log = Logger.getLogger(LicenseService.class);
    public LicenseServiceREST() {
        super("/" + RESTName.LICENSE, License.class, RESTName.ADMIN);
    }

    @Override
    public List<License> findAll() {
        return findAll(License.class);
    }

    public HttpEntity<String> createLicenses(List<License> licences) {
        final String url = getBaseUrl() + resourcePath + ADMIN_LICENSE_IMPORT_LIST_PATH;
        RestTemplate restTemplate = RestClientTool.getClient(getBaseUrl());
        String response = restTemplate.postForObject(url, licences, String.class);
        return new ResponseEntity(response, HttpStatus.CREATED);
    }

    public HttpEntity<String> importLicenseFile(String fileName) {
        final String url = getBaseUrl() + resourcePath + ADMIN_LICENSE_IMPORT_FILE_PATH;
        return importFile(url, fileName);
    }

    public HttpEntity<String> importOpenLicenseFile(String fileName) {
        final String url = getBaseUrl() + resourcePath + RESTDlcmRoutes.ADMIN_OPEN_LICENSE_IMPORT_FILE_PATH;
        return importFile(url, fileName);
    }

    private HttpEntity<String> importFile(String url, String filename) {
        String response;
        try {
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.MULTIPART_FORM_DATA);
            LinkedMultiValueMap<String, Object> multipartMap = new LinkedMultiValueMap<>();
            multipartMap.add("file", new ClassPathResource(filename));
            HttpEntity<LinkedMultiValueMap<String, Object>> requestEntity = new HttpEntity<>(multipartMap, headers);
            RestTemplate restTemplate = RestClientTool.getClient(getBaseUrl());
            response = restTemplate.postForObject(url, requestEntity, String.class);
            return new ResponseEntity(response, HttpStatus.CREATED);
        } catch (HttpStatusCodeException e) {
            log.error("Error importing file", e);
            response = "Error : " + e.getStatusCode() + "|" + e.getStatusText();
        }
        return new HttpEntity<>(response);
    }


}
