package ch.dlcm.service.admin;

import java.util.List;

import ch.dlcm.model.FundingAgency;
import ch.dlcm.model.Institution;
import ch.dlcm.model.OrganizationalUnit;
import ch.dlcm.model.Person;
import ch.dlcm.service.ResourceService;

public interface OrganizationalUnitService extends ResourceService<OrganizationalUnit> {

  /**
   * Add a new institution to the project
   */
  void addInstitution(String unitId, String institutionId);

  /**
   * Get list of institutions
   */
  List<Institution> getInstitutions(String unitId);

  /**
   * Add a new person to the project and five the default role.
   */
  void addPerson(String unitId, String personId);

  /**
   * Add a new person to the project with a list of roles
   */
  void addPerson(String personId, String projectId, String... roleNames);

  /**
   * Get list of people
   */
  List<Person> getPeople(String unitId);

  /**
   * Remove person from the project
   */
  void removePerson(String unitId, String personId);

  /**
   * Remove person with given role from the project
   */
  void removePerson(String unitId, String personId, String roleName);

  /**
   * Add a FundingAgency to the OrganizationalUnit
   */
  void addFundingAgency(String unitId, String fundingAgencyId);

  /**
   * Remove a FundingAgency from the OrganizationalUnit
   */
  void removeFundingAgency(String unitId, String fundingAgencyId);
  
  /**
   * Get the list of linked FundingAgencies
   */
  List<FundingAgency> getFundingAgencies(String unitId);
}

