package ch.dlcm.service.admin;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections.OrderedMap;
import org.apache.commons.collections.map.ListOrderedMap;
import org.springframework.stereotype.Service;

import ch.dlcm.model.FundingAgency;
import ch.dlcm.model.Institution;
import ch.dlcm.model.OrganizationalUnit;
import ch.dlcm.model.Person;
import ch.dlcm.rest.RESTName;
import ch.dlcm.service.ResourceServiceREST;

@Service
public class OrganizationalUnitServiceREST extends ResourceServiceREST<OrganizationalUnit> implements OrganizationalUnitService {

  public OrganizationalUnitServiceREST() {
    super("/" + RESTName.UNIT, OrganizationalUnit.class, RESTName.ADMIN);
  }

  @Override
  public List<OrganizationalUnit> findAll() {
    return findAll(OrganizationalUnit.class);
  }

  @Override
  public void addInstitution(String unitId, String institutionId) {
    this.addItem(unitId, RESTName.INSTITUTION, institutionId, Institution.class);
  }

  @Override
  public List<Institution> getInstitutions(String unitId) {
    return findAllLinkedResources(unitId, Institution.class);
  }

  @Override
  public void addPerson(String unitId, String personId) {
    this.addItem(unitId, RESTName.PERSON, personId, Person.class);
  }

  @Override
  public void addPerson(String projectId, String personIds, String... roleNames) {
    Map<String, String> params = new HashMap<>();
    params.put(DLCM_ID_LABEL, projectId);
    Map<String, String[]> resourceIds = new HashMap<>();
    resourceIds.put(RESTName.PERSON, new String[]{personIds});
    resourceIds.put(RESTName.ROLE, roleNames);
    this.addItem(params, RESTName.PERSON + "/" + RESTName.ROLE, Person.class, resourceIds);
  }

  @Override
  public List<Person> getPeople(String unitId) {
    return findAllLinkedResources(unitId, Person.class);
  }

  @Override
  public void removePerson(String unitId, String personId) {
    this.removeItem(unitId, RESTName.PERSON, personId);
  }

  @Override
  public void removePerson(String unitId, String personId, String roleName) {
    Map<String, String> params = new HashMap<>();
    params.put(DLCM_ID_LABEL, unitId);
    OrderedMap resourceIds = new ListOrderedMap();
    resourceIds.put(RESTName.PERSON, personId);
    resourceIds.put(RESTName.ROLE, roleName);

    this.removeItem(unitId, resourceIds);
  }

  @Override
  public void addFundingAgency(String unitId, String fundingAgencyId) {
    this.addItem(unitId, RESTName.FUNDING_AGENCY, fundingAgencyId, FundingAgency.class);
  }

  @Override
  public void removeFundingAgency(String unitId, String fundingAgencyId) {
    this.removeItem(unitId, RESTName.FUNDING_AGENCY, fundingAgencyId);
  }
  
  @Override
  public List<FundingAgency> getFundingAgencies(String unitId) {
    return findAllLinkedResources(unitId, FundingAgency.class);
  }
}