package ch.dlcm.service.admin;

import java.util.*;

import ch.dlcm.model.OrganizationalUnit;
import org.apache.commons.collections.OrderedMap;
import org.apache.commons.collections.map.ListOrderedMap;
import org.springframework.stereotype.Service;

import ch.dlcm.model.Institution;
import ch.dlcm.model.Person;
import ch.dlcm.rest.RESTName;
import ch.dlcm.service.ResourceServiceREST;

@Service
public class PersonServiceREST extends ResourceServiceREST<Person> implements PersonService{
    private static final String institutionPath = "/"+RESTName.PERSON;
    
    public PersonServiceREST(){
        super(institutionPath, Person.class, RESTName.ADMIN);
    }
    @Override
    public List<Person> findAll() {
       return findAll(Person.class);
    }
    @Override
    public void addInstitution(String personId, String institutionId){
        this.addItem(personId, "institutions", institutionId, Institution.class);
    }
    @Override
    public List<Institution> getInstitutions(String personId){
        return findAllLinkedResources(personId, Institution.class);
    }

    @Override
    public void addProject(String personId, String projectId) {
        this.addProject(personId, projectId, new String[]{});
    }

    @Override
    public void addProject(String personId, String projectId, String... roleNames) {
        Map<String, String> params = new HashMap<>();
        params.put(DLCM_ID_LABEL, personId);
        Map<String, String[]>  resourceIds = new HashMap<>();
        resourceIds.put(RESTName.UNIT, new String[]{projectId});
        resourceIds.put(RESTName.ROLE, roleNames);
        this.addItem(params, RESTName.UNIT + "/" + RESTName.ROLE, OrganizationalUnit.class, resourceIds);
    }

    @Override
    public List<OrganizationalUnit> getProjects(String personId) {
        return findAllLinkedResources(personId, OrganizationalUnit.class);
    }

    @Override
    public void removeProject(String personId, String projectId) {
        this.removeItem(personId, RESTName.UNIT, projectId);
    }

    // TODO should be extracted later
    @Override
    public void removeProject(String personId, String projectId, String roleName) {
        Map<String, String> params = new HashMap<>();
        params.put(DLCM_ID_LABEL, personId);
        OrderedMap resourceIds = new ListOrderedMap();
        resourceIds.put(RESTName.UNIT, projectId);
        resourceIds.put(RESTName.ROLE, roleName);
        this.removeItem(personId, resourceIds);
    }

}