package ch.dlcm.service.preingest;

import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.List;

import org.springframework.core.ParameterizedTypeReference;
import org.springframework.core.io.Resource;
import org.springframework.stereotype.Service;

import ch.dlcm.model.Deposit;
import ch.dlcm.model.DepositDataFile;
import ch.dlcm.model.Person;
import ch.dlcm.rest.RESTName;
import ch.dlcm.service.ResourceServiceREST;

@Service
public class DepositServiceREST extends ResourceServiceREST<Deposit> implements DepositService{
    
    private static final String depositPath = "/" + RESTName.DEPOSIT;
    
    public DepositServiceREST(){
        super(depositPath, Deposit.class, RESTName.PREINGEST);
    }
    @Override
    public void addAuthor(String depositId, String authorId) {
        this.addItem(depositId, RESTName.PERSON, authorId, Person.class);
    }
    @Override
    public List<Deposit> findAll() {
        return findAll(Deposit.class);
    }
    @Override
    public List<Person> getAuthors(String depositId){
        return findAllLinkedResources(depositId, Person.class);
    }
	@Override
	public void addDepositDataFile(String depositId, DepositDataFile df) {
		this.addItem(depositId, RESTName.DATAFILE, df, DepositDataFile.class);
	}
    @Override
    public void removeDepositDataFile(String depositId, String dataFileId) {
        this.removeItem(depositId, RESTName.DATAFILE, dataFileId);
    }
    @Override
	public List<DepositDataFile> getDepositDataFile(String depositId) {
		return findAllLinkedResources(depositId, DepositDataFile.class);
	}
	@Override
	public DepositDataFile upload(String depositId, Resource resource) {
		List<Resource> resList = new ArrayList<>();
		resList.add(resource);
		return uploadFiles(depositId, new ParameterizedTypeReference<DepositDataFile[]>() {}, resList)[0];
	}
	@Override
    public ByteArrayOutputStream downloadDataFile(String depositId, String dataFileId){
        return this.download(depositId, dataFileId);
    }
}
