package ch.dlcm.test.admin;

import java.util.List;

import org.junit.After;
import org.springframework.beans.factory.annotation.Autowired;

import ch.dlcm.model.FundingAgency;
import ch.dlcm.model.OrganizationalUnit;
import ch.dlcm.service.admin.FundingAgencyService;
import ch.dlcm.service.admin.OrganizationalUnitService;
import ch.dlcm.service.admin.PersonService;
import ch.dlcm.test.DLCMTestConstants;

public abstract class DLCMAdminTest {

  @Autowired
  protected FundingAgencyService fundingAgencyService;
  
  @Autowired
  protected OrganizationalUnitService organizationalUnitService;
  
  /***********************************/
  
  @After
  public void tearDown()
  {
    this.deleteFixtures();
  }
  
  /***********************************/
  
  /**
   * Delete fixtures created by tests
   */
  protected abstract void deleteFixtures();
  
  protected void clearFundingAgencyFixtures()
  {
    List<FundingAgency> agencies = fundingAgencyService.findAll();
    
    for (FundingAgency a : agencies) {
      
      if (a.getName().startsWith(DLCMTestConstants.PREFIX)) {
        
        /*
         * Remove eventual links with OrganizationalUnits first
         */
        List<OrganizationalUnit> child_units = fundingAgencyService.getOrganizationalUnits(a.getDlcmId());
        if (child_units != null) {
          for (OrganizationalUnit u : child_units) {
            fundingAgencyService.removeOrganizationalUnit(a.getDlcmId(), u.getDlcmId());
          }
        }
        
        fundingAgencyService.delete(a.getDlcmId());
      }
    }
  }
  
  protected void clearOrganizationalUnitFixtures()
  {
    List<OrganizationalUnit> units = organizationalUnitService.findAll();
    
    for (OrganizationalUnit u : units) {
      
      if (u.getName().startsWith(DLCMTestConstants.PREFIX)) {
        organizationalUnitService.delete(u.getDlcmId());
      }
      
    }
  }
  
}
