package ch.dlcm.test.admin;


import static org.junit.Assert.assertEquals;

import java.util.List;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import ch.dlcm.model.Institution;
import ch.dlcm.model.Person;
import ch.dlcm.service.admin.InstitutionService;
import ch.dlcm.service.admin.PersonService;
import ch.dlcm.test.DLCMTestConstants;

@RunWith(SpringJUnit4ClassRunner.class)
@SpringBootTest
public class InstitutionTest {

    @Autowired private InstitutionService institutionService;
    @Autowired private PersonService personService;

    @Test
    public void creationTest() {
        // Create an institution
        Institution institution1 = new Institution();
        institution1.setName(DLCMTestConstants.getLabel("Institut Du Du "));
        institution1.setDescription("C'est très joli");
        Institution institution2 = institutionService.create(institution1);

        // Test the creation
        assertEquals(institution1.getName(),        institution2.getName());
        assertEquals(institution1.getDescription(), institution2.getDescription());
        assertEquals(institution1.getCreationTime(),institution2.getCreationTime());
    }
    @Test
    public void updateTest(){
        // Create an institution
        Institution institution1 = new Institution();
        institution1.setName(DLCMTestConstants.getLabel("A"));
        institution1.setDescription("A");
        institution1 = institutionService.create(institution1);

        // Update the institution
        institution1.setName(DLCMTestConstants.getLabel("B"));
        institution1.setDescription("B");
        institutionService.update(institution1.getDlcmId(), institution1);
        Institution institution2 = institutionService.findOne(institution1.getDlcmId());

        // Test the update
        assertEquals(institution1.getName(),        institution2.getName());
        assertEquals(institution1.getDescription(), institution2.getDescription());
        assertEquals(institution1.getCreationTime(),institution2.getCreationTime());
    }
    @Test
    public void personTest(){
        // Create an institution
        Institution institution = new Institution();
        institution.setName(DLCMTestConstants.getLabel("Institut Du Du "));
        institution = institutionService.create(institution);

        // Create a person
        Person person = new Person();
        person.setFirstName(DLCMTestConstants.getLabel("Jojo "));
        person.setLastName(DLCMTestConstants.getLabel("Tutu "));
        person = personService.create(person);

        // Add a person to the institution
        institutionService.addPerson(institution.getDlcmId(), person.getDlcmId());
        List<Person> listPersons = institutionService.getPersons(institution.getDlcmId());
        Person person3 = listPersons.get(0);

        // Test that the person is added to the institution
        assertEquals(person.getDlcmId(),    person3.getDlcmId());
        assertEquals(person.getFirstName(), person3.getFirstName());
        assertEquals(person.getLastName(),  person3.getLastName());
    }
}
