package ch.dlcm.test.admin;

import static org.junit.Assert.assertEquals;

import java.util.List;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import ch.dlcm.model.Institution;
import ch.dlcm.model.OrganizationalUnit;
import ch.dlcm.model.Person;
import ch.dlcm.service.admin.InstitutionService;
import ch.dlcm.service.admin.OrganizationalUnitService;
import ch.dlcm.service.admin.PersonService;
import ch.dlcm.test.DLCMTestConstants;

@RunWith(SpringJUnit4ClassRunner.class)
@SpringBootTest
public class PersonTest {

    @Autowired private PersonService personService;
    @Autowired private InstitutionService institutionService;
    @Autowired private OrganizationalUnitService organizationalUnitService;

    @Test
    public void creationTest() {
        // Create a person
        Person person1 = new Person();
        person1.setFirstName(DLCMTestConstants.getLabel("Jojo"));
        person1.setLastName(DLCMTestConstants.getLabel("Tutu"));
        person1.setOrcid("orcid_C");
        person1 = personService.create(person1);
        Person person2 = personService.findOne(person1.getDlcmId());

        // Test the creation
        assertEquals(person1.getFirstName(),   person2.getFirstName());
        assertEquals(person1.getLastName(),    person2.getLastName());
        assertEquals(person1.getCreationTime(),person2.getCreationTime());
        assertEquals(person1.getOrcid(),person2.getOrcid());
    }
    @Test
    public void updateTest(){
        // Create a person
        Person person1 = new Person();
        person1.setFirstName(DLCMTestConstants.getLabel("A firtName"));
        person1.setLastName(DLCMTestConstants.getLabel("A lastName"));
        person1 = personService.create(person1);

        // Update the person
        Person person2 = new Person();
        person2.setFirstName(DLCMTestConstants.getLabel("B firstname"));
        person2.setLastName(DLCMTestConstants.getLabel("B lastName"));
        person2.setOrcid("orcid_U");
        Person person3 = personService.update(person1.getDlcmId(), person2);

        // Test the update
        assertEquals(person2.getFirstName(), person3.getFirstName());
        assertEquals(person2.getLastName(),  person3.getLastName());
        assertEquals(person2.getOrcid(),person3.getOrcid());
    }
    @Test
    public void institutionTest(){
        // Create a person
        Person person = new Person();
        person.setFirstName(DLCMTestConstants.getLabel("Jojo"));
        person.setLastName(DLCMTestConstants.getLabel("Tutu"));
        person = personService.create(person);

        // Create an institution
        Institution institution1 = new Institution();
        institution1.setName(DLCMTestConstants.getLabel("institution"));
        institution1.setDescription(DLCMTestConstants.getLabel("description"));
        institution1 = institutionService.create(institution1);

        // Add the person to the institution
        personService.addInstitution(person.getDlcmId(), institution1.getDlcmId());
        List<Institution> listInstitutions = personService.getInstitutions(person.getDlcmId());
        Institution institution2 = listInstitutions.get(0);

        // Test the institution
        assertEquals(institution1.getDlcmId(),      institution2.getDlcmId());
        assertEquals(institution1.getName(),        institution2.getName());
        assertEquals(institution1.getDescription(), institution2.getDescription());
    }

    @Test
    public void addRemoveProjectFromPersonTest(){

        // Create a person
        Person person = new Person();
        person.setFirstName(DLCMTestConstants.getLabel("Superman"));
        person.setLastName(DLCMTestConstants.getLabel("SuperHero"));
        person = personService.create(person);

        // Create a project
        OrganizationalUnit project = new OrganizationalUnit();
        project.setName(DLCMTestConstants.getLabel("DLCM project"));
        project.setDescription("Swiss Research Data Preservation #PEOPLE");
        project = organizationalUnitService.create(project);

//         Add the person to the project with defaut role
        personService.addProject(person.getDlcmId(), project.getDlcmId());
        List<OrganizationalUnit> projects = personService.getProjects(person.getDlcmId());

        assertEquals(1, projects.size());

        // Add the person to the project with others roles
        personService.addProject(person.getDlcmId(), project.getDlcmId(), "Approver", "Creator");
        projects = personService.getProjects(person.getDlcmId());

        assertEquals(1, projects.size());
        OrganizationalUnit expectedProject = projects.get(0);

        // Test added person on the project
        assertEquals(project.getDlcmId(),      expectedProject.getDlcmId());
        assertEquals(project.getDescription(), expectedProject.getDescription());

        // Test remove a role from the person in project
        personService.removeProject(person.getDlcmId(), expectedProject.getDlcmId(), "Approver");
        assertEquals(1, projects.size());

        // Test remove project from the person
        personService.removeProject(person.getDlcmId(), expectedProject.getDlcmId());
        projects = personService.getProjects(person.getDlcmId());
        assertEquals(true, projects.isEmpty());

    }

}
