package ch.dlcm.test.admin;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.List;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import ch.dlcm.model.Role;
import ch.dlcm.service.admin.RoleService;
import ch.dlcm.test.DLCMTestConstants;

@RunWith(SpringJUnit4ClassRunner.class)
@SpringBootTest
public class RoleTest {


    @Autowired private RoleService roleService;

    @Test
    public void creationTest() {
       //  Create a role
        Role expectedRole = new Role();
        String stewart = DLCMTestConstants.getLabel("stewart");
        expectedRole.setName(stewart);
        expectedRole = roleService.create(expectedRole);
        Role actualRole = roleService.findOne(expectedRole.getDlcmId());

        // Test the creation
        assertNotNull(expectedRole.getDlcmId());
        assertsRole(expectedRole, actualRole);

        // Test unique name
        Role sameRole = new Role();
        expectedRole.setName(stewart);

        try {
            roleService.create(sameRole);
            fail("Should raise exception: Unique constraint name" );
        }catch(Exception e){
            assertEquals("500", e.getMessage().trim());
        }
    }

    @Test
    public void updateTest() throws Exception {
        //Create a role
        Role expectedRole = new Role();
        expectedRole.setName(DLCMTestConstants.getLabel("stewart"));
        expectedRole = roleService.create(expectedRole);

        // Update the licence
        String  roleName = DLCMTestConstants.getLabel("stewart2");
        expectedRole.setName(roleName);
        Role actualRole = roleService.update(expectedRole.getDlcmId(), expectedRole);

        // Test the update
        assertNotNull(actualRole);
        assertsRole(expectedRole, actualRole);

        // Test unique name
        Role sameRole = new Role();
        sameRole.setName(DLCMTestConstants.getLabel("C"));
        sameRole = roleService.create(sameRole);
        sameRole.setName(roleName);
        try {
            roleService.update(sameRole.getDlcmId(), sameRole);
            fail("Should raise exception: Unique constraint role name" );
        }catch(Exception e){
            assertEquals("500", e.getMessage().trim());
        }
    }

    private void assertsRole(Role expectedRole, Role actualRole){
        assertEquals(expectedRole.getName(), actualRole.getName());
        assertEquals(expectedRole.getCreationTime(), actualRole.getCreationTime());
    }

    @Test
    public void deleteTest(){
        // Create a new role
        Role expectedRole = new Role();
        expectedRole.setName(DLCMTestConstants.getLabel("Role"));
        expectedRole = roleService.create(expectedRole);

        // Delete
        String dlcmId = expectedRole.getDlcmId();
        roleService.delete(dlcmId);
        Role actualRole = roleService.findOne(dlcmId);

        //Test deletion
        assertNull(actualRole);
    }

    @Test
    public void findAllTest(){
        // Create a new licence
        Role expectedRole = new Role();
        expectedRole.setName(DLCMTestConstants.getLabel("Role"));

        roleService.create(expectedRole);

        // Find all
        List<Role> actualLicenses = roleService.findAll();

        //Test at least
        assertFalse(actualLicenses.isEmpty());
        assertTrue(actualLicenses.size() >= 1);
    }
}
