package ch.dlcm.test.admin;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.util.List;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import ch.dlcm.model.SubmissionAgreement;
import ch.dlcm.service.admin.SubmissionAgreementService;
import ch.dlcm.test.DLCMTestConstants;

@RunWith(SpringJUnit4ClassRunner.class)
@SpringBootTest
public class SubmissionAgreementTest {

    @Autowired private SubmissionAgreementService submissionAgreementService;

    @Test
    public void creationTest() {
       //  Create a submission agreement
        SubmissionAgreement expectedSubmissionAgreement = new SubmissionAgreement();
        expectedSubmissionAgreement.setTitle(DLCMTestConstants.getLabel("DLCM project"));
        expectedSubmissionAgreement.setDescription("Swiss Research Data Preservation #CREATE");
        expectedSubmissionAgreement = submissionAgreementService.create(expectedSubmissionAgreement);
        SubmissionAgreement actualSubmissionAgreement = submissionAgreementService.findOne(expectedSubmissionAgreement.getDlcmId());

        // Test the creation
        assertNotNull(expectedSubmissionAgreement.getDlcmId());
        assertEquals(expectedSubmissionAgreement.getTitle(),   		actualSubmissionAgreement.getTitle());
        assertEquals(expectedSubmissionAgreement.getDescription(), actualSubmissionAgreement.getDescription());
        assertEquals(expectedSubmissionAgreement.getCreationTime(),actualSubmissionAgreement.getCreationTime());
    }

    @Test
    public void updateTest(){
        //Create a submission agreement
        SubmissionAgreement expectedSubmissionAgreement = new SubmissionAgreement();
        expectedSubmissionAgreement.setTitle(DLCMTestConstants.getLabel("A"));
        expectedSubmissionAgreement.setVersion(DLCMTestConstants.getLabel("A"));
        expectedSubmissionAgreement = submissionAgreementService.create(expectedSubmissionAgreement);

        // Update the submission agreement
        expectedSubmissionAgreement.setTitle(DLCMTestConstants.getLabel("B"));
        expectedSubmissionAgreement.setVersion(DLCMTestConstants.getLabel("B"));
        SubmissionAgreement actualSubmissionAgreement = submissionAgreementService.update(expectedSubmissionAgreement.getDlcmId(), expectedSubmissionAgreement);

        // Test the update
        assertNotNull(actualSubmissionAgreement);
        assertEquals(expectedSubmissionAgreement.getTitle(),        actualSubmissionAgreement.getTitle());
        assertEquals(expectedSubmissionAgreement.getCreationTime(),actualSubmissionAgreement.getCreationTime());
        assertEquals(expectedSubmissionAgreement.getVersion(),actualSubmissionAgreement.getVersion());

    }

    @Test
    public void deleteTest(){
        // Create a new submission agreement
        SubmissionAgreement expectedSubmissionAgreement = new SubmissionAgreement();
        expectedSubmissionAgreement.setTitle(DLCMTestConstants.getLabel("Deposit "));
        expectedSubmissionAgreement = submissionAgreementService.create(expectedSubmissionAgreement);

        // Delete
        String dlcmId = expectedSubmissionAgreement.getDlcmId();
        submissionAgreementService.delete(dlcmId);
        SubmissionAgreement actualSubmissionAgreement = submissionAgreementService.findOne(dlcmId);

        //Test deletion
        assertNull(actualSubmissionAgreement);
    }

    @Test
    public void findAllTest(){
        // Create a new submission agreement
        SubmissionAgreement expectedSubmissionAgreement = new SubmissionAgreement();
        expectedSubmissionAgreement.setTitle(DLCMTestConstants.getLabel("Deposit "));
        submissionAgreementService.create(expectedSubmissionAgreement);

        // Find all
        List<SubmissionAgreement> actualSubmissionAgreements = submissionAgreementService.findAll();

        //Test at least
        assertFalse(actualSubmissionAgreements.isEmpty());
        assertTrue(actualSubmissionAgreements.size() >= 1);
    }

}
