package ch.dlcm.test.ingest;

import static org.junit.Assert.assertEquals;

import java.net.URI;
import java.util.Collections;
import java.util.List;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import ch.dlcm.model.DataCategory;
import ch.dlcm.model.DataFile;
import ch.dlcm.model.Status.StatusValue;
import ch.dlcm.model.oais.SipDataFile;
import ch.dlcm.model.oais.SubmissionInfoPackage;
import ch.dlcm.service.ingest.SIPService;
import ch.dlcm.test.DLCMTestConstants;

@RunWith(SpringJUnit4ClassRunner.class)
@SpringBootTest
public class SubmissionInfoPackageTest {
  private final static String REFERENCE_FILE = "http://www.unige.ch/rectorat/static/budget-2013.pdf";

  @Autowired private SIPService sipService;

  @Test public void creationTest() {
   // Create SIP
    createRemoteSipWithDataFile();
  }


  @Test
  public void deleteTest() {
    // Create SIP
    SubmissionInfoPackage remoteSip = createRemoteSipWithDataFile();

    // Test if data file is present
    int depositListSize = sipService.getSipDataFile(remoteSip.getDlcmId()).size();
    assertEquals(depositListSize, 1);

    // Remove the data file
    String sipDataFileId = remoteSip.getDataFiles().get(0).getDlcmId();
    sipService.removeSipDataFile(remoteSip.getDlcmId(), sipDataFileId);

    // Test if data file is absent
    assertEquals(sipService.getSipDataFile(remoteSip.getDlcmId()).size(), 0);

  }

  @Test
  public void deleteWhenReadyTest() {
    // Create SIP
    SubmissionInfoPackage remoteSip = createRemoteSipWithDataFile();

    // Wait until data file downloaded
    waitDataFileIsReady(remoteSip.getDlcmId(), 1);

    // Remove the data file
    String sipDataFileId = remoteSip.getDataFiles().get(0).getDlcmId();
    sipService.removeSipDataFile(remoteSip.getDlcmId(), sipDataFileId);

    // Test if data file is absent
    int depositListSize = sipService.getSipDataFile(remoteSip.getDlcmId()).size();
    assertEquals(depositListSize, 0);

  }

  private SubmissionInfoPackage createRemoteSipWithDataFile(){
    // Create a SIP
    SubmissionInfoPackage localSip = new SubmissionInfoPackage();
    localSip.getInfo().setName(DLCMTestConstants.getLabel("SIP "));
    SubmissionInfoPackage remoteSip = sipService.create(localSip);
    String sipId = remoteSip.getDlcmId();

    // Create a data file object
    DataFile dataFile = new DataFile();
    dataFile.setSourceData(URI.create(REFERENCE_FILE));
    dataFile.setDataCategory(DataCategory.Secondary);
    dataFile.setDataType(DataCategory.Documentation);

    // Create a SIP data file object
    SipDataFile localSipDataFile = new SipDataFile();
    localSipDataFile.setDataFile(dataFile);
    sipService.addSipDataFile(sipId, localSipDataFile);
    SipDataFile remoteSipDataFile = sipService.getSipDataFile(sipId).get(0);

    // Set the retrieved data file to the SIP object
    remoteSip.setDataFiles(Collections.singletonList(remoteSipDataFile));

    // Test the creation
    assertEquals(localSip.getDlcmId(), remoteSip.getDlcmId());
    assertEquals(localSipDataFile.getDlcmId(), remoteSipDataFile.getDlcmId());
    assertEquals(localSipDataFile.getPackageId(), remoteSipDataFile.getPackageId());

    return remoteSip;

  }

  private void waitDataFileIsReady(String sipId, int dataFileNumber) {
    List<SipDataFile> dataFileList = sipService.getSipDataFile(sipId);

    // Test if data file is present
    assertEquals(dataFileList.size(), dataFileNumber);

    int count = 0;
    while (!checkDataFileListIsCompleted(dataFileList)) {
      DLCMTestConstants.checkTries(count);
      DLCMTestConstants.waitInSeconds();
      dataFileList = sipService.getSipDataFile(sipId);
      count++;
    }
  }

  private boolean checkDataFileListIsCompleted(List<SipDataFile> dataFileList) {
    for (SipDataFile df : dataFileList) {
      if (df.getDataFile().getStatus().getValue() != StatusValue.Ready) {
        return false;
      }
    }
    return true;
  }

}
