package ch.dlcm.test.preingest;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

import java.util.List;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import ch.dlcm.model.Deposit;
import ch.dlcm.model.Person;
import ch.dlcm.model.Status;
import ch.dlcm.service.admin.PersonService;
import ch.dlcm.service.preingest.DepositService;
import ch.dlcm.test.DLCMTestConstants;

@RunWith(SpringJUnit4ClassRunner.class)
@SpringBootTest
public class DepositTest {

    @Autowired private DepositService depositService;
    @Autowired private PersonService personService;

    @Test
    public void creationTest() {
        // Create a deposit
        Deposit deposit1 = new Deposit();
        deposit1.setTitle(DLCMTestConstants.getLabel("Deposit "));
        Deposit deposit2 = depositService.create(deposit1);

        // Test the creation
        assertEquals(deposit1.getTitle(), deposit2.getTitle());
        assertEquals(deposit1.getCreationTime(),deposit2.getCreationTime());

        // Check if pending ( InProgress, inValidation, inError )
        assertEquals(Status.StatusValue.InProgress, deposit2.getStatus().getValue());

    }

    @Test
    public void updateTest(){
        //Create a deposit
        Deposit deposit = new Deposit();
        deposit.setTitle(DLCMTestConstants.getLabel("A"));
        deposit = depositService.create(deposit);

        // Update the deposit
        deposit.setTitle(DLCMTestConstants.getLabel("B"));
        Deposit actualDeposit = depositService.update(deposit.getDlcmId(), deposit);
        Deposit deposit4 = depositService.findOne(deposit.getDlcmId());

        // Test the update
        assertEquals(deposit.getTitle(),        deposit4.getTitle());
        assertEquals(deposit.getCreationTime(),deposit4.getCreationTime());

        Status status = new Status();
        status.setValue(Status.StatusValue.InValidation);
        deposit.setStatus(status);
        deposit4 = depositService.update(deposit4.getDlcmId(), deposit);
        assertEquals(Status.StatusValue.InValidation, deposit4.getStatus().getValue());
        assertNotNull(actualDeposit);

    }
    @Test
    public void deleteTest(){
        // Create a new deposit
        Deposit deposit = new Deposit();
        deposit.setTitle(DLCMTestConstants.getLabel("Deposit "));
        deposit = depositService.create(deposit);

        // Delete the deposit
        String dlcmId = deposit.getDlcmId();
        depositService.delete(dlcmId);
        deposit = depositService.findOne(dlcmId);

        //Test deletion
        assertNull(deposit);
    }
    @Test
    public void authorTest(){
        // Create a new deposit
        Deposit deposit = new Deposit();
        deposit.setTitle(DLCMTestConstants.getLabel("Deposit "));
        deposit = depositService.create(deposit);

        // Create a new Person
        Person person1 = new Person();
        person1.setFirstName(DLCMTestConstants.getLabel("firstName "));
        person1.setLastName(DLCMTestConstants.getLabel("lastName "));
        Person person2 = personService.create(person1);

        // Add the person as author
        depositService.addAuthor(deposit.getDlcmId(), person2.getDlcmId());
        List<Person> listAuthors = depositService.getAuthors(deposit.getDlcmId());
        Person person3 = listAuthors.get(0);

        // Test the author
        assertEquals(person2.getDlcmId(),    person3.getDlcmId());
        assertEquals(person2.getFirstName(), person3.getFirstName());
        assertEquals(person2.getLastName(),  person3.getLastName());
    }
    
}