package ch.dlcm.config;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

import javax.annotation.PostConstruct;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

@Component
public class DLCMConfig {
  private static final Logger log = Logger.getLogger(DLCMConfig.class);

  // Constants
  private enum Directory {
    DEPOSIT("deposits"), SIP("sip"), AIP("aip"), DIP("dip"), STORAGE("storage"), ORDER("order");
    private String d;

    private Directory(String s) {
      d = s;
    }

    private String getName() {
      return d;
    }
  }

  // General
  @Value("${dlcm.home}")
  private String home;
  @Value("${dlcm.tries.max}")
  private int maxtries;
  @Value("${dlcm.wait.milliseconds}")
  private long waitTime;
  @Value("${dlcm.checksums}")
  private String[] checksums;
  @Value("${dlcm.queue.history}")
  private String historyQueue;
  @Value("${dlcm.queue.datafile}")
  private String dataFileQueue;
  

  // PreIngest Parameters
  @Value("${dlcm.module.preingest}")
  private String preingestUrl;
  @Value("${dlcm.queue.deposit}")
  private String depositQueue;
  @Value("${dlcm.preingest.preparation.id}")
  private String preingestPreparationId;
  @Value("${dlcm.preingest.preparation.url}")
  private String preingestPreparationUrl;
  @Value("${dlcm.preingest.preparation.user}")
  private String preingestPreparationUser;
  @Value("${dlcm.preingest.preparation.apikey}")
  private String preingestPreparationApiKey;

  @Value("${dlcm.module.ingest}")
  private String ingestUrl;
  @Value("${dlcm.queue.sip}")
  private String sipQueue;

  // Archival Storage Parameters
  @Value("${dlcm.module.archival-storage}")
  private String archivalStorageUrl;
  @Value("${dlcm.queue.aip}")
  private String aipQueue;

  // Data Mgmt Parameters
  @Value("${dlcm.module.data-mgmt}")
  private String dataMgmtUrl;
  @Value("${dlcm.module.image-mgmt}")
  private String imageMgmtUrl;
  @Value("${dlcm.datamgmt.indexing.host}")
  private String dataMgmtIndexingHost;
  @Value("${dlcm.datamgmt.indexing.port}")
  private int dataMgmtIndexingPort;
  @Value("${dlcm.datamgmt.index.name}")
  private String dataMgmtIndexName;
  

  // Access Parameters
  @Value("${dlcm.module.access}")
  private String accessUrl;
  @Value("${dlcm.queue.dip}")
  private String dipQueue;
  @Value("${dlcm.queue.aip.download}")
  private String aipDownloadQueue;
  @Value("${dlcm.queue.order}")
  private String orderQueue;

  // Access Parameters

  public String getOrderQueue() {
    return orderQueue;
  }

  public String[] getChecksums() {
    return checksums;
  }

  public String getPreingestUrl() {
    return preingestUrl;
  }

  public String getPreingestPreparationLocation() {
    return this.home + "/" + Directory.DEPOSIT.getName();
  }

  public String getPreingestPreparationId() {
    return preingestPreparationId;
  }

  public String getPreingestPreparationUrl() {
    return preingestPreparationUrl;
  }

  public String getPreingestPreparationUser() {
    return preingestPreparationUser;
  }

  public String getPreingestPreparationApiKey() {
    return preingestPreparationApiKey;
  }

  public String getIngestUrl() {
    return ingestUrl;
  }

  public String getIngestLocation() {
    return this.home + "/" + Directory.SIP.getName();
  }

  public String getArchivalStorageUrl() {
    return archivalStorageUrl;
  }

  public String getArchivalStorage() {
    return this.home + "/" + Directory.STORAGE.getName();
  }

  public String getArchivalLocation() {
    return this.home + "/" + Directory.AIP.getName();
  }

  public String getDataMgmtUrl() {
    return dataMgmtUrl;
  }

  public String getAccessUrl() {
    return accessUrl;
  }

  public String getAccessLocation() {
    return this.home + "/" + Directory.DIP.getName();
  }

  public String getOrderLocation() {
    return this.home + "/" + Directory.ORDER.getName();
  }

  public String getHistoryQueue() {
    return historyQueue;
  }

  public String getDataFileQueue() {
    return dataFileQueue;
  }

  public String getDepositQueue() {
    return depositQueue;
  }

  public String getSipQueue() {
    return sipQueue;
  }

  public String getAipQueue() {
    return aipQueue;
  }

  public String getAipDownloadQueue() {
    return aipDownloadQueue;
  }

  public String getDipQueue() {
    return dipQueue;
  }

  public String getIndexingHost() {
    return dataMgmtIndexingHost;
  }

  public int getIndexingPort() {
    return dataMgmtIndexingPort;
  }

  public String getIndexName() {
    return dataMgmtIndexName;
  }
  @PostConstruct
  void checkConfiguration() {
    // Check Home
    Path p = Paths.get(this.home);
    if (!p.toFile().exists()) {
      log.fatal("The directory " + this.home + " does not exist");
    }

    for (Directory d : Directory.values()) {
      p = Paths.get(this.home, d.getName());
      if (!p.toFile().exists()) {
        try {
          Files.createDirectories(p);
        } catch (IOException e) {
          log.fatal("The directory " + p.toString() + " cannot be created", e);
        }
      }
    }
  }

  public String getImageMgmtUrl() {
    return imageMgmtUrl;
  }

  public int getMaxTries() {
    return maxtries;
  }

  public long getWaitTime() {
    return waitTime; //milliseconds
  }
}
