package ch.dlcm.config.security;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Primary;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.oauth2.config.annotation.web.configuration.ResourceServerConfigurerAdapter;
import org.springframework.security.oauth2.config.annotation.web.configurers.ResourceServerSecurityConfigurer;
import org.springframework.security.oauth2.provider.token.DefaultTokenServices;
import org.springframework.security.oauth2.provider.token.RemoteTokenServices;
import org.springframework.security.oauth2.provider.token.TokenStore;
import org.springframework.security.oauth2.provider.token.store.JwtAccessTokenConverter;
import org.springframework.security.oauth2.provider.token.store.JwtTokenStore;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.stream.Collectors;

public class DLCMCommonResourceServerConfig extends ResourceServerConfigurerAdapter {


  @Value("${dlcm.module.auth}")
  private String authenticationBaseUrl;

  @Override
  public void configure(final HttpSecurity http) throws Exception {
    http.sessionManagement().sessionCreationPolicy(SessionCreationPolicy.IF_REQUIRED)
            .and().authorizeRequests()
              .antMatchers("/dashboard", "/health", "/metrics", "/info", "/logfile").permitAll()
              .antMatchers("/css/**", "/*.png").permitAll()
            .and().authorizeRequests()
              .anyRequest().authenticated()
            .and().csrf().disable();

  }

  @Override
  public void configure(final ResourceServerSecurityConfigurer config) {
    config.tokenServices(tokenService());
  }

  @Bean
  public TokenStore tokenStore() {
    return new JwtTokenStore(accessTokenConverter());
  }

  @Bean
  public JwtAccessTokenConverter accessTokenConverter() {
    final JwtAccessTokenConverter converter = new JwtAccessTokenConverter();
    final Resource resource = new ClassPathResource("public.txt");
    String publicKey = null;
    try {
      publicKey = new BufferedReader(new InputStreamReader(resource.getInputStream())).lines().collect(Collectors.joining("\n"));
    } catch (IOException e) {
      e.printStackTrace();
    }
    converter.setVerifierKey(publicKey);
    return converter;
  }


  @Primary
  @Bean
  public RemoteTokenServices tokenService() {
    final RemoteTokenServices tokenService = new RemoteTokenServices();
    tokenService.setCheckTokenEndpointUrl( authenticationBaseUrl + OAuthURLs.OAUTH_CHECK_TOKEN_URL);
    tokenService.setClientId(OAuthClients.DLCM_UI_CLIENT.getClientId());
    tokenService.setClientSecret(OAuthClients.DLCM_UI_CLIENT.getSecret());
    return tokenService;
  }



  // TODO remove me after remote service works
  @Bean
  public DefaultTokenServices tokenServicess() {
    final DefaultTokenServices defaultTokenServices = new DefaultTokenServices();
    defaultTokenServices.setTokenStore(tokenStore());
    return defaultTokenServices;
  }

}

