package ch.dlcm.controller;

import static org.springframework.hateoas.mvc.ControllerLinkBuilder.linkTo;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.methodOn;

import java.util.ArrayList;
import java.util.List;

import ch.dlcm.model.Role;
import ch.dlcm.model.display.RessourceRole;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import ch.dlcm.rest.RESTRepository;
import ch.dlcm.rest.RESTResource;
import ch.dlcm.rest.RESTSubResourceInterface;


public abstract class AssociationController<T extends RESTSubResourceInterface, V extends RESTResource<V>>
		extends SubResourceController<T, V> {
    //private static final Logger log = Logger.getLogger(AssociationController.class);

	@Autowired
	protected RESTRepository<V> childItemRepository;

	@RequestMapping(method = RequestMethod.POST)
	public HttpEntity<List<V>> create(@PathVariable String parentid, @RequestBody String[] ids) {
		T parentitem = itemRepository.findOne(parentid);
		if (parentitem == null){
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}
		List<V> list = new ArrayList<>();
		for (String id : ids) {
			V childItem = childItemRepository.findOne(id);
			if (childItem == null){
				return new ResponseEntity<>(HttpStatus.NOT_FOUND);
			}
			parentitem.addItem(childItem);
			list.add(childItem);
			childItemRepository.save(childItem);
		}
		T item = itemRepository.save(parentitem);
		if (item != null){
			return new ResponseEntity<>(list, HttpStatus.CREATED);
		}
		return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
	}

	@RequestMapping(value = "/{id}", method = { RequestMethod.PUT, RequestMethod.PATCH })
	public HttpEntity<V> update(@PathVariable String parentid, @PathVariable String id, @RequestBody T t2) {
		return new ResponseEntity<>(HttpStatus.NOT_IMPLEMENTED);
	}

	@RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
	public HttpStatus delete(@PathVariable String parentid, @PathVariable String id) {
		T parentItem = itemRepository.findOne(parentid);
		if (parentItem == null){
			return HttpStatus.NOT_FOUND;
		}
		V item = itemRepository.findByParentIdAndChildId(parentid,getMainResourceClass(), getSubResourceClass(), id);
		if (item == null){
			return HttpStatus.NOT_FOUND;
		}
		if (parentItem.removeItem(item)) {
			T t = itemRepository.save(parentItem);
			if (t != null){
				return HttpStatus.OK;
			}
		}
		return HttpStatus.GONE;
	}

	@RequestMapping(method = RequestMethod.DELETE)
	public HttpStatus deleteAll(@PathVariable String parentid) {
		T parentItem = itemRepository.findOne(parentid);
		if (parentItem == null){
			return HttpStatus.NOT_FOUND;
		}
		Page<V> list = itemRepository.findByParentId(parentid,getMainResourceClass(), getSubResourceClass(), null);
		if (list.getTotalElements() > 0) {
			for (V v : list){
				parentItem.removeItem(v); // TODO to optimize removeall
			}
			T t = itemRepository.save(parentItem);
			if (t != null){
				return HttpStatus.OK;
			}
		}
		return HttpStatus.GONE;
	}

	@Override
	protected boolean addLinks(String parentid, V v) {
		v.removeLinks();
		v.addLinks(linkTo(methodOn(this.getClass()).listAll(parentid, null)), false, false);
		return true;
	}

	protected void addRoleLinks(Class controllerClz, RessourceRole personRole) {
		for (Role role : personRole.getRoles()) {
			if (!role.hasLinks()) {
				role.addLinks(linkTo(controllerClz), true, true);
			}
		}
	}

}
