package ch.dlcm.controller;

import static org.springframework.hateoas.mvc.ControllerLinkBuilder.linkTo;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.methodOn;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import ch.dlcm.rest.RESTRepository;
import ch.dlcm.rest.RESTResource;
import ch.dlcm.rest.RESTSubResourceInterface;

public abstract class CompositionController<T extends RESTSubResourceInterface, V extends RESTResource<V>>
        extends SubResourceController<T, V> {

    @Autowired
    private RESTRepository<V> childItemRepository;

    @RequestMapping(method = RequestMethod.POST)
    public HttpEntity<V> create(@PathVariable final String parentid, final @RequestBody V childItem) {
        T parentItem = itemRepository.findOne(parentid);
        if (parentItem == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        if (parentItem.addItem(childItem)) {
            T item = itemRepository.save(parentItem);
            if (item != null) {
                return new ResponseEntity<>(childItem, HttpStatus.CREATED);
            }
        }
        return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
    }

    @RequestMapping(value = "/{id}", method = { RequestMethod.PUT, RequestMethod.PATCH })
    public HttpEntity<V> update(@PathVariable final String parentid, @PathVariable final String id, @RequestBody final V newChildItem) {
        T oldParentItem = itemRepository.findOne(parentid);
        if (oldParentItem == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        V oldChildItem = itemRepository.findByParentIdAndChildId(parentid, getMainResourceClass(), getSubResourceClass(), id);
        if (oldChildItem == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        T newParentItem = null;
        if (oldChildItem.update(newChildItem)) {
            newParentItem = itemRepository.save(oldParentItem);
        }
        if (newParentItem != null) {
            return new ResponseEntity<>(oldChildItem, HttpStatus.OK);
        }
        return new ResponseEntity<>(HttpStatus.NOT_MODIFIED);
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public HttpStatus delete(@PathVariable final String parentid, @PathVariable final String id) {
        T oldParentItem = itemRepository.findOne(parentid);
        if (oldParentItem == null) {
            return HttpStatus.NOT_FOUND;
        }
        V childItem = childItemRepository.findOne(id);
        if (childItem == null) {
            return HttpStatus.NOT_FOUND;
        }
        if (oldParentItem.removeItem(childItem)) {
            T newParentItem = itemRepository.save(oldParentItem);
            if (newParentItem != null) {
                return HttpStatus.OK;
            }
        }
        return HttpStatus.GONE;
    }

    @Override
    protected boolean addLinks(final String parentId, final V childItem) {
        childItem.removeLinks();
        childItem.addLinks(linkTo(methodOn(this.getClass()).listAll(parentId, null)), false, true);
        return true;
    }
}
