package ch.dlcm.controller;

import static org.springframework.hateoas.mvc.ControllerLinkBuilder.linkTo;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.methodOn;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestMapping;

import ch.dlcm.model.Module;
import ch.dlcm.rest.RESTAction;
import ch.dlcm.service.MessageService;

public abstract class ModuleController {

	@Autowired
	MessageService messageService;
	
	private String name;
	protected String root;
	protected List<ResourceControllerInterface> resourceList;
	
	protected ModuleController(String name) {
		this.name = name;
		root = name.toLowerCase();
		resourceList = new ArrayList<>();
	}

	public String getName() {
		return name;
	}

	public String getRoot() {
		return root;
	}

	@RequestMapping
	public HttpEntity<Module> home() {
		Module mod = new Module(getName());
		mod.add(linkTo(methodOn(this.getClass()).home()).withSelfRel());
		for(ResourceControllerInterface r:resourceList){
			mod.add(linkTo(methodOn(r.getClass()).home()).withRel(RESTAction.RESOURCES));
		}
		return new ResponseEntity<>(mod, HttpStatus.OK);
	}

	public void addResource(ResourceControllerInterface r) {
		resourceList.add(r);
	}
	
	// Function to initialze data by module
	public abstract void init() throws Exception;
}
