package ch.dlcm.controller;

import static org.springframework.hateoas.mvc.ControllerLinkBuilder.linkTo;

import java.io.IOException;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.hateoas.ResourceSupport;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import ch.dlcm.exception.DLCMException;
import ch.dlcm.rest.RESTAction;
import ch.dlcm.rest.RESTCollection;
import ch.dlcm.rest.RESTRepositoryNoSql;
import ch.dlcm.rest.RESTResourceNoSql;
import ch.dlcm.rest.RESTTool;

public abstract class ResourceNoSqlController<T extends RESTResourceNoSql<T>> extends DLCMController
		implements ResourceControllerInterface {
	// private static final Logger log = Logger.getLogger(ResourceNoSqlController.class);

	@Autowired
	protected RESTRepositoryNoSql<T> itemRepository;

	public HttpEntity<?> home() {
		return this.listAll(null);
	}

	@RequestMapping(method = RequestMethod.GET)
	public HttpEntity<RESTCollection<T>> listAll(Pageable pageable) {
		final Page<T> listItem = itemRepository.findAll(pageable);
		for (final T t : listItem) {
			addLinks(t);
		}
		RESTCollection<T> collection = new RESTCollection<>(listItem, pageable);
		collection.add(linkTo(this.getClass(), pageable).withSelfRel());
		collection.add(RESTTool.parentLink((linkTo(this.getClass(), pageable)).toUriComponentsBuilder()).withRel(RESTAction.MODULE));
		this.addSortLinks(linkTo(this.getClass(), pageable), collection);
		this.addPageLinks(linkTo(this.getClass(), pageable), collection, pageable);
		this.addOthersLinks(collection);
		return new ResponseEntity<>(collection, HttpStatus.OK);
	}


	@RequestMapping(method = RequestMethod.POST)
	public HttpEntity<T> create(@RequestBody T t) throws DLCMException,IOException {
		T item = itemRepository.save(t);
		if (item != null)
			return new ResponseEntity<>(item, HttpStatus.CREATED);
		return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
	}

	@RequestMapping(value = "/{id}", method = RequestMethod.GET)
	public HttpEntity<T> list(@PathVariable String id) {
		T t = itemRepository.findOne(id);
		if (t != null) {
			addLinks(t);
			return new ResponseEntity<>(t, HttpStatus.OK);
		}
		return new ResponseEntity<>(HttpStatus.NOT_FOUND);
	}

	@RequestMapping(value = "/{id}", method = { RequestMethod.PUT, RequestMethod.PATCH })
	public HttpEntity<T> update(@PathVariable String id, @RequestBody T t2) throws DLCMException, IOException {
		T t1 = itemRepository.findOne(id);
		if (t1 == null){
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}
		T item = null;
		if (t1.update(t2)){
			item = itemRepository.save(t1);
		}
		if (item != null){
			return new ResponseEntity<>(item, HttpStatus.OK);
		}
		return new ResponseEntity<>(HttpStatus.NOT_MODIFIED);
	}

	@RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
	public HttpStatus delete(@PathVariable String id) throws DLCMException {
		T t = itemRepository.findOne(id);
		if (t == null){ 
			return HttpStatus.NOT_FOUND;
		}
		if (!itemRepository.delete(t)){
			return HttpStatus.GONE;
		}
		return HttpStatus.OK;
	}

	protected boolean addLinks(T t) {
		t.removeLinks();
		t.addLinks(linkTo(this.getClass()));
		return true;
	}

	protected <W extends ResourceSupport> boolean addOthersLinks(W w) {
		return false;
	}

}
