package ch.dlcm.controller;

import static org.springframework.hateoas.mvc.ControllerLinkBuilder.linkTo;


import ch.dlcm.model.display.RessourceRole;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.InputStreamResource;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.hateoas.ResourceSupport;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import ch.dlcm.exception.DLCMException;
import ch.dlcm.model.StatusHistory;
import ch.dlcm.repository.HistoryRepository;
import ch.dlcm.rest.RESTAction;
import ch.dlcm.rest.RESTCollection;
import ch.dlcm.rest.RESTName;
import ch.dlcm.rest.RESTRepository;
import ch.dlcm.rest.RESTResource;
import ch.dlcm.rest.RESTSubResourceInterface;
import ch.dlcm.rest.RESTTool;

import java.util.List;

public abstract class SubResourceController<T extends RESTSubResourceInterface, V extends RESTResource<V>>
		extends DLCMController {

	@Autowired
	protected RESTRepository<T> itemRepository;
	
	@Autowired
	HistoryRepository historyRepository;

	@RequestMapping(method = RequestMethod.GET)
	public HttpEntity<RESTCollection<V>> listAll(@PathVariable String parentid, Pageable pageable) {
		if (pageable.getSort() != null)
			throw new UnsupportedOperationException(messageService.get("error.sort.unsupported",
					new Object[] { getMainResourceClass().toString(), getSubResourceClass().toString() }));
		T mainResourceItem = itemRepository.findOne(parentid);
		if (mainResourceItem == null)
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		Page<V> list = itemRepository.findByParentId(parentid, getMainResourceClass(),
				getSubResourceClass(), pageable);
		for (final V v : list) {
			addLinks(parentid, v);
		}
		RESTCollection<V> collection = processPageAndLinks(list, parentid, pageable);
		return new ResponseEntity<>(collection, HttpStatus.OK);
	}

	protected RESTCollection<V> processPageAndLinks(Page<V> list, String parentid, Pageable pageable) {
		RESTCollection<V> collection = new RESTCollection(list, pageable);
		collection.add(linkTo(this.getClass(), parentid).withSelfRel());
		collection.add((RESTTool.parentLink((linkTo(this.getClass(), parentid)).toUriComponentsBuilder())).withRel(RESTAction.PARENT));
		//TODO if sort working: this.addSortLinks(linkTo(this.getClass(),parentid, pageable), collection);
		this.addPageLinks(linkTo(this.getClass(), parentid), collection, pageable);
		this.addOthersLinks(parentid, collection);
		return collection;
	}

	protected RESTCollection<V> processResourceRolesAndLinks(List<RessourceRole> ressourceRoles, String parentid, Pageable pageable) {
		Page<V> resourceRolePage = new PageImpl(ressourceRoles);
		return  processPageAndLinks(resourceRolePage, parentid, pageable);
	}

	@RequestMapping(value = "/{id}", method = RequestMethod.GET)
	public HttpEntity<V> list(@PathVariable String parentid, @PathVariable String id) {
		T parentitem = itemRepository.findOne(parentid);
		if (parentitem == null) {
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}
		V item = itemRepository.findByParentIdAndChildId(parentid, getMainResourceClass(), getSubResourceClass(), id);
		if (item != null) {
			addLinks(parentid, item);
			return new ResponseEntity<>(item, HttpStatus.OK);
		}
		return new ResponseEntity<>(HttpStatus.NOT_FOUND);
	}

	@RequestMapping(value = "/{id}/"+ RESTName.DL , method = RequestMethod.GET)
	public HttpEntity<InputStreamResource> download(@PathVariable String parentid, @PathVariable String id) throws DLCMException {
		T parentitem = itemRepository.findOne(parentid);
		if (parentitem == null) {
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}
		V item = itemRepository.findByParentIdAndChildId(parentid, getMainResourceClass(), getSubResourceClass(), id);
		if (item == null) {
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}
		return download(item.getClass().getName(), id,item.getDownloadUri(),null);
	}
	
	@RequestMapping(value = "/{id}/" + RESTName.HISTORY, method = RequestMethod.GET)
	public HttpEntity<RESTCollection<StatusHistory>> history(@PathVariable String parentid, @PathVariable String id, Pageable pageable) {
		T parentitem = itemRepository.findOne(parentid);
		if (parentitem == null) {
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}
		V item = itemRepository.findByParentIdAndChildId(parentid, getMainResourceClass(), getSubResourceClass(), id);
		if (item == null) {
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}
		final Page<StatusHistory> listItem = historyRepository.findByDlcmId(id, pageable); 	// TODO check if it's the "id" or "parentid" instead in parameter. It's should never found because it is a history for deposit status
		RESTCollection<StatusHistory> collection = new RESTCollection<>(listItem, pageable);
		collection.add(linkTo(this.getClass(), parentid, id, pageable).withSelfRel());
		collection.add(RESTTool.parentLink((linkTo(this.getClass(),parentid, id, pageable)).toUriComponentsBuilder()).withRel(RESTAction.PARENT));
		this.addPageLinks(linkTo(this.getClass(), parentid, id, pageable), collection, pageable);
		return new ResponseEntity<>(collection,HttpStatus.OK);
	}
	
	protected abstract Class<V> getSubResourceClass();

	protected abstract Class<?> getMainResourceClass();

	protected abstract boolean addLinks(String parentid, V v);

	protected <W extends ResourceSupport> boolean addOthersLinks(String parentid, W w) {
		return false;
	}
}