package ch.dlcm.repository;

import ch.dlcm.model.*;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import ch.dlcm.model.Status.StatusValue;
import ch.dlcm.rest.RESTRepository;

@Repository
public interface DepositRepository extends RESTRepository<Deposit>  {
	@Query("SELECT p FROM Deposit d JOIN d.contributors p WHERE d.dlcmId = :parentId")
	public Page<Person> findContributorsByParentId(@Param("parentId") String parentId, Pageable pageable);
	
	@Query("SELECT p FROM Deposit d JOIN d.contributors p WHERE d.dlcmId = :parentId AND p.dlcmId=:id")
	public Person findContributorsByParentId(@Param("parentId") String parentId, @Param("id") String id);
	
	@Query("SELECT df FROM Deposit d JOIN d.dataFiles df  WHERE d.dlcmId = :parentId")
	public Page<DepositDataFile> findDepositDataFileByParentId(@Param("parentId") String parentId, Pageable pageable);
	
	@Query("SELECT df FROM Deposit d JOIN d.dataFiles df  WHERE d.dlcmId = :parentId AND df.dlcmId=:id")
	public DepositDataFile findDepositDataFileByParentId(@Param("parentId") String parentId, @Param("id") String id);

	@Query("SELECT sa FROM Deposit d JOIN d.submissionAgreement sa WHERE d.dlcmId = :parentId")
	Page<SubmissionAgreement> findAgreededDepositByParentId(@Param("parentId") String parentId, Pageable pageable);

	@Query("SELECT sa FROM Deposit d JOIN d.submissionAgreement sa WHERE d.dlcmId = :parentId AND sa.dlcmId=:id")
	SubmissionAgreement findAgreededDeposit(@Param("parentId") String parentId, @Param("id") String id);

	@Query("SELECT sa FROM Deposit d JOIN d.license sa WHERE d.dlcmId = :parentId")
	Page<License> findLicenseByParentId(@Param("parentId") String parentId, Pageable pageable);

	@Query("SELECT sa FROM Deposit d JOIN d.license sa WHERE d.dlcmId = :parentId AND sa.dlcmId=:id")
    License findLicenseDeposit(@Param("parentId") String parentId, @Param("id") String id);

	@Query("SELECT dl FROM Deposit d JOIN d.language dl WHERE d.dlcmId = :parentId")
	Page<Language> findLanguageByParentId(@Param("parentId") String parentId, Pageable pageable);

	@Query("SELECT dl FROM Deposit d JOIN d.language dl WHERE d.dlcmId = :parentId AND dl.dlcmId=:id")
	License findLanguageByParentId(@Param("parentId") String parentId, @Param("id") String id);

	public Page<Deposit> findByStatusValue(StatusValue depositstatus, Pageable pageable);
}