package ch.dlcm.repository;

import org.apache.log4j.Logger;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import ch.dlcm.exception.DLCMInitialDataException;
import ch.dlcm.model.FundingAgency;
import ch.dlcm.model.OrganizationalUnit;
import ch.dlcm.rest.RESTRepository;

@Repository
public interface FundingAgencyRepository extends RESTRepository<FundingAgency> {
  static final Logger log = Logger.getLogger(FundingAgencyRepository.class);

  @Query("SELECT i FROM FundingAgency u JOIN u.organizationalUnits i WHERE u.dlcmId = :parentId AND i.dlcmId=:id")
  OrganizationalUnit findOrganizationalUnitByParentId(@Param("parentId") String parentId,
          @Param("id") String id);

  @Query("SELECT i FROM FundingAgency u JOIN u.organizationalUnits i WHERE u.dlcmId = :parentId")
  Page<OrganizationalUnit> findOrganizationalUnitsByParentId(@Param("parentId") String parentId,
          Pageable pageable);

  FundingAgency findByAcronym(String acronym);

  public default void init() throws Exception {
    createFundingAgency("SNF", "Swiss National Science Foundation", "http://www.snf.ch");
    createFundingAgency("CTI", "Commission for Technology and Innovation",
            "https://www.kti.admin.ch/");
    createFundingAgency("H2020", "Horizon 2020", "https://ec.europa.eu/programmes/horizon2020/");
  }

  public default void createFundingAgency(String acronym, String name, String url)
          throws Exception {
    if (findByAcronym(acronym) == null) {
      FundingAgency fa = new FundingAgency();
      fa.setAcronym(acronym);
      fa.setName(name);
      fa.setUrl(url);
      if (save(fa) == null) {
        try {
          String message = "Cannot create '" + acronym + "' funding agency";
          log.error(message);
          throw new DLCMInitialDataException(message);
        } catch (DLCMInitialDataException e) {
          throw new RuntimeException(e);
        }

      }
      log.info("Funding agency '" + acronym + "' created");
    }
    return;
  }
}
