package ch.dlcm.repository;

import org.apache.log4j.Logger;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import ch.dlcm.exception.DLCMInitialDataException;
import ch.dlcm.model.Institution;
import ch.dlcm.model.OrganizationalUnit;
import ch.dlcm.model.Person;
import ch.dlcm.rest.RESTRepository;

@Repository
public interface InstitutionRepository extends RESTRepository<Institution> {
  static final Logger log = Logger.getLogger(InstitutionRepository  .class);
  
  @Query("SELECT p FROM Institution i JOIN i.people p WHERE i.dlcmId = :parentId")
  public Page<Person> findPeopleByParentId(@Param("parentId") String parentId, Pageable pageable);

  @Query("SELECT p FROM Institution i JOIN i.people p WHERE i.dlcmId = :parentId AND p.dlcmId=:id")
  public Person findPeopleByParentId(@Param("parentId") String parentId, @Param("id") String id);

  @Query("SELECT p FROM Institution i JOIN i.projects p WHERE i.dlcmId = :parentId")
  public Page<OrganizationalUnit> findProjectsByParentId(@Param("parentId") String parentId,
          Pageable pageable);

  @Query("SELECT p FROM Institution i JOIN i.projects p WHERE i.dlcmId = :parentId AND p.dlcmId=:id")
  public OrganizationalUnit findProjectsByParentId(@Param("parentId") String parentId,
          @Param("id") String id);
  
  Institution findByName(String name);

  public default void init() throws Exception {
    createInstitution("UNIGE", "Université de Genève");
    createInstitution("EPFL", "Ecole Polytechnique Fédérale de Lausanne");
    createInstitution("ETH", "ETH Zürich");
    createInstitution("UZH", "University of Zurich");
  }
  
  public default void createInstitution(String acronym, String name)
          throws Exception {
    if (findByName(acronym) == null) {
      Institution i = new Institution();
      i.setName(acronym);
      i.setDescription(name);
      if (save(i) == null) {
        try {
          String message = "Cannot create '" + acronym + "' institution";
          log.error(message);
          throw new DLCMInitialDataException(message);
        } catch (DLCMInitialDataException e) {
          throw new RuntimeException(e);
        }

      }
      log.info("Institution '" + acronym + "' created");
    }
    return;
  }
}