package ch.dlcm.repository;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.springframework.stereotype.Repository;

import ch.dlcm.exception.DLCMInitialDataException;
import ch.dlcm.model.Language;
import ch.dlcm.rest.RESTRepository;

@Repository
public interface LanguageRepository extends RESTRepository<Language> {

  static final Logger log = Logger.getLogger(LanguageRepository.class);
  
  Language findByCode(String code);

  public default void init() {
    List<String> languages = new ArrayList<>();
    languages.add("en");
    languages.add("de");
    languages.add("fr");

    languages.forEach(code -> {
      if (findByCode(code) == null) {
        Language l = new Language();
        l.setDlcmId(code.toUpperCase());
        l.setCode(code);
        if (save(l) == null) {
          try {
            String message = "Cannot create '" + code + "' language";
            log.error(message);
            throw new DLCMInitialDataException(message);
          } catch (DLCMInitialDataException e) {
            throw new RuntimeException(e);
          }
        }
        log.info("Language '" + code + "' created");
      }
    });
  }
}
