package ch.dlcm.repository;

import java.net.URI;
import java.net.URISyntaxException;

import org.apache.log4j.Logger;
import org.springframework.stereotype.Repository;

import ch.dlcm.exception.DLCMInitialDataException;
import ch.dlcm.model.License;
import ch.dlcm.model.License.ConformanceStatus;
import ch.dlcm.model.License.LicenseStatus;
import ch.dlcm.rest.RESTRepository;

@Repository
public interface LicenseRepository extends RESTRepository<License> {
  static final Logger log = Logger.getLogger(LicenseRepository.class);

  License findByOpenLicenseId(String openLicenseid);

  public default void init() throws URISyntaxException {
    String defaultLicense = "CC-BY-4.0";
    if (findByOpenLicenseId(defaultLicense) == null) {
      License l = new License();
      l.setDomainContent(true);
      l.setDomainData(true);
      l.setDomainSoftware(false);
      l.setOpenLicenseId(defaultLicense);
      l.setMaintainer("Creative Commons");
      l.setTitle("Creative Commons Attribution 4.0");
      l.setUrl(new URI("https://creativecommons.org/licenses/by/4.0/"));
      l.setOdConformance(ConformanceStatus.APPROVED);
      l.setOsdConformance(ConformanceStatus.NOT_REVIEWED);
      l.setStatus(LicenseStatus.ACTIVE);
      if (save(l) == null) {
        try {
          String message = "Cannot create '" + defaultLicense + "' license";
          log.error(message);
          throw new DLCMInitialDataException(message);
        } catch (DLCMInitialDataException e) {
          throw new RuntimeException(e);
        }
      }
      log.info("License '" + defaultLicense + "' created");
    }
  }
}
