package ch.dlcm.repository;

import java.util.List;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import ch.dlcm.model.Order;
import ch.dlcm.model.Status.StatusValue;
import ch.dlcm.model.oais.ArchivalInfoPackage;
import ch.dlcm.model.oais.DisseminationInfoPackage;
import ch.dlcm.rest.RESTRepository;

@Repository
public interface OrderRepository extends RESTRepository<Order> {
  @Query("SELECT dip FROM Order o JOIN o.dipPackages dip WHERE o.dlcmId = :parentId")
  public Page<DisseminationInfoPackage> findDipPackagesByParentId(@Param("parentId") String parentId, Pageable pageable);

  @Query("SELECT dip FROM Order o JOIN o.dipPackages dip WHERE o.dlcmId = :parentId AND dip.dlcmId=:id")
  public DisseminationInfoPackage findDipPackagesByParentId(@Param("parentId") String parentId, @Param("id") String id);

  @Query("SELECT aip FROM Order o JOIN o.aipPackages aip  WHERE o.dlcmId = :parentId")
  public Page<ArchivalInfoPackage> findAipPackagesByParentId(@Param("parentId") String parentId, Pageable pageable);

  @Query("SELECT aip FROM Order o JOIN o.aipPackages aip  WHERE o.dlcmId = :parentId AND aip.dlcmId=:id")
  public ArchivalInfoPackage findAipPackagesByParentId(@Param("parentId") String parentId, @Param("id") String id);
 
  @Query("SELECT o FROM Order o LEFT JOIN FETCH o.dipPackages where o.dlcmId=:id")
  public Order findOneWithDip(@Param("id")String id);
  
  @Query("SELECT o FROM Order o LEFT JOIN FETCH o.aipPackages where o.dlcmId=:id")
  public Order findOneWithAip(@Param("id")String id);

  public List<Order> findByStatusValue(List<StatusValue> statuses);
}