package ch.dlcm.repository;

import ch.dlcm.model.FundingAgency;
import ch.dlcm.model.Institution;
import ch.dlcm.model.OrganizationalUnit;
import ch.dlcm.model.Person;
import ch.dlcm.rest.RESTRepository;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

@Repository
public interface OrganizationalUnitRepository extends RESTRepository<OrganizationalUnit>, OrganizationalUnitPersonRoleRepository {

  @Query("SELECT i FROM OrganizationalUnit u JOIN u.institutions i WHERE u.dlcmId = :parentId")
  Page<Institution> findInstitutionsByParentId(@Param("parentId") String parentId, Pageable pageable);

  @Query("SELECT i FROM OrganizationalUnit u JOIN u.institutions i WHERE u.dlcmId = :parentId AND i.dlcmId=:id")
  Institution findInstitutionsByParentId(@Param("parentId") String parentId, @Param("id") String id);

  @Query("SELECT distinct oupr.dlcmCompositeId.person FROM OrganizationalUnit u JOIN u.organizationalUnitPersonRoles oupr WHERE oupr.dlcmCompositeId.organizationalUnit.dlcmId = :parentId")
  Page<Person> findPeopleByParentId(@Param("parentId") String parentId, Pageable pageable);

  @Query("SELECT oupr.dlcmCompositeId.person FROM OrganizationalUnit u JOIN u.organizationalUnitPersonRoles oupr WHERE oupr.dlcmCompositeId.organizationalUnit.dlcmId = :parentId AND oupr.dlcmCompositeId.person.dlcmId=:id")
  Person findPeopleByParentId(@Param("parentId") String parentId, @Param("id") String id);

  @Query("SELECT i FROM OrganizationalUnit u JOIN u.fundingAgencies i WHERE u.dlcmId = :parentId AND i.dlcmId=:id")
  FundingAgency findFundingAgencyByParentId(@Param("parentId") String parentId, @Param("id") String id);

  @Query("SELECT i FROM OrganizationalUnit u JOIN u.fundingAgencies i WHERE u.dlcmId = :parentId")
  Page<FundingAgency> findFundingAgenciesByParentId(@Param("parentId") String parentId, Pageable pageable);
  
}