package ch.dlcm.repository;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.springframework.stereotype.Repository;

import ch.dlcm.exception.DLCMInitialDataException;
import ch.dlcm.model.Role;
import ch.dlcm.rest.RESTRepository;

@Repository
public interface RoleRepository
        extends RESTRepository<Role>, OrganizationalUnitPersonRoleRepository {

  static final Logger log = Logger.getLogger(RoleRepository.class);

  Role findByName(String name);

  public default void init() {
    List<String> roles = new ArrayList<>();
    roles.add("Admin");
    roles.add("Stewart");
    roles.add("Approver");
    roles.add("Creator");
    roles.add("Visitor");

    roles.forEach(role -> {
      if (findByName(role) == null) {
        Role r = new Role();
        r.setDlcmId(role.toUpperCase());
        r.setName(role);
        if (save(r) == null) {
          try {
            String message = "Cannot create '" + role + "' role";
            log.error(message);
            throw new DLCMInitialDataException(message);
          } catch (DLCMInitialDataException e) {
            throw new RuntimeException(e);
          }
        }
        log.info("Role '" + role + "' created");
      }
    });
  }

}
