package ch.dlcm.rest;

import ch.dlcm.repository.*;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.repository.NoRepositoryBean;

import ch.dlcm.model.Deposit;
import ch.dlcm.model.DepositDataFile;
import ch.dlcm.model.FundingAgency;
import ch.dlcm.model.Institution;
import ch.dlcm.model.Language;
import ch.dlcm.model.License;
import ch.dlcm.model.Order;
import ch.dlcm.model.OrganizationalUnit;
import ch.dlcm.model.Person;
import ch.dlcm.model.SubmissionAgreement;
import ch.dlcm.model.OrganizationalUnitPersonRole;
import ch.dlcm.model.oais.AipDataFile;
import ch.dlcm.model.oais.ArchivalInfoPackage;
import ch.dlcm.model.oais.DipDataFile;
import ch.dlcm.model.oais.DisseminationInfoPackage;
import ch.dlcm.model.oais.SipDataFile;
import ch.dlcm.model.oais.SubmissionInfoPackage;
import org.springframework.data.repository.PagingAndSortingRepository;

import java.util.Arrays;
import java.util.List;

@NoRepositoryBean
public interface RESTRepository<T> extends PagingAndSortingRepository<T, String>, JpaSpecificationExecutor<T> {

	@SuppressWarnings("unchecked")
	public default <V> Page<V> findByParentId(String parentId, Class<?> parentClass, Class<V> childClass,
																						Pageable pageable) {
		// Deposit<->Person
		if (parentClass.equals(Deposit.class) && childClass.equals(Person.class)) {
			return (Page<V>) ((DepositRepository) this).findContributorsByParentId(parentId, pageable);
		} // Deposit<->DepositDataFile
		else if (parentClass.equals(Deposit.class) && childClass.equals(DepositDataFile.class)) {
			return (Page<V>) ((DepositRepository) this).findDepositDataFileByParentId(parentId, pageable);
		} // Deposit<->SubmissionAgreement
		if (parentClass.equals(Deposit.class) && childClass.equals(SubmissionAgreement.class)) {
			return (Page<V>) ((DepositRepository) this).findAgreededDepositByParentId(parentId, pageable);
		} // Deposit<->License
		if (parentClass.equals(Deposit.class) && childClass.equals(License.class)) {
			return (Page<V>) ((DepositRepository) this).findLicenseByParentId(parentId, pageable);
		} // Deposit<->Language
		if (parentClass.equals(Deposit.class) && childClass.equals(Language.class)) {
			return (Page<V>) ((DepositRepository) this).findLanguageByParentId(parentId, pageable);
		} // Institution<-> Unit
		else if (parentClass.equals(Institution.class) && childClass.equals(OrganizationalUnit.class)) {
			return (Page<V>) ((InstitutionRepository) this).findProjectsByParentId(parentId, pageable);
		} // Institution<->Person
		else if (parentClass.equals(Institution.class) && childClass.equals(Person.class)) {
			return (Page<V>) ((InstitutionRepository) this).findPeopleByParentId(parentId, pageable);
		} // Person<->Institution
		else if (parentClass.equals(Person.class) && childClass.equals(Institution.class)) {
			return (Page<V>) ((PersonRepository) this).findInstitutionsByParentId(parentId, pageable);
		} // Person<-> Unit
		else if (parentClass.equals(Person.class) && childClass.equals(OrganizationalUnit.class)) {
			return (Page<V>) ((PersonRepository) this).findProjectsByParentId(parentId, pageable);
		} // SIP<->SipDataFile
		else if (parentClass.equals(SubmissionInfoPackage.class) && childClass.equals(SipDataFile.class)) {
			return (Page<V>) ((SubmissionInfoPackageRepository) this).findSipDataFileByParentId(parentId, pageable);
		} // AIP<->AipDataFile
		else if (parentClass.equals(ArchivalInfoPackage.class) && childClass.equals(AipDataFile.class)) {
			return (Page<V>) ((ArchivalInfoPackageRepository) this).findAipDataFileByParentId(parentId, pageable);
		} // DIP<->DipDataFile
		else if (parentClass.equals(DisseminationInfoPackage.class) && childClass.equals(DipDataFile.class)) {
			return (Page<V>) ((DisseminationInfoPackageRepository) this).findDipDataFileByParentId(parentId, pageable);
		} // Order<->OrderDataFile
		else if (parentClass.equals(Order.class) && childClass.equals(ArchivalInfoPackage.class)) {
			return (Page<V>) ((OrderRepository) this).findAipPackagesByParentId(parentId, pageable);
		} // Order<->DIP
		else if (parentClass.equals(Order.class) && childClass.equals(DisseminationInfoPackage.class)) {
			return (Page<V>) ((OrderRepository) this).findDipPackagesByParentId(parentId, pageable);
		} // Unit<->Institution
		else if (parentClass.equals(OrganizationalUnit.class) && childClass.equals(Institution.class)) {
			return (Page<V>) ((OrganizationalUnitRepository) this).findInstitutionsByParentId(parentId, pageable);
		} // Unit<->Person
		else if (parentClass.equals(OrganizationalUnit.class) && childClass.equals(Person.class)) {
			return (Page<V>) ((OrganizationalUnitRepository) this).findPeopleByParentId(parentId, pageable);
		} // Unit<->FundingAgency 
		else if (parentClass.equals(OrganizationalUnit.class) && childClass.equals(FundingAgency.class)) {
			return (Page<V>) ((OrganizationalUnitRepository) this).findFundingAgenciesByParentId(parentId, pageable);
		} // FundingAgency<->Unit
		else if (parentClass.equals(FundingAgency.class) && childClass.equals(OrganizationalUnit.class)) {
			return (Page<V>) ((FundingAgencyRepository) this).findOrganizationalUnitsByParentId(parentId, pageable);
		}
		else {
			throw new UnsupportedOperationException("Missing implementation 'findByParentId' function");
		}
	}

	@SuppressWarnings("unchecked")
	public default <V> V findByParentIdAndChildId(String parentId, Class<?> parentClass, Class<V> childClass,
																								String id) {
		// Deposit<->Person
		if (parentClass.equals(Deposit.class) && childClass.equals(Person.class)) {
			return (V) ((DepositRepository) this).findContributorsByParentId(parentId, id);
		}
		// Deposit<->DataFile
		else if (parentClass.equals(Deposit.class) && childClass.equals(DepositDataFile.class)) {
			return (V) ((DepositRepository) this).findDepositDataFileByParentId(parentId, id);
		}
		// Deposit<->SubmissionAgreement
		else if (parentClass.equals(Deposit.class) && childClass.equals(SubmissionAgreement.class)) {
			return (V) ((DepositRepository) this).findAgreededDeposit(parentId, id);
		}
		// Deposit<-> License
		else if (parentClass.equals(Deposit.class) && childClass.equals(License.class)) {
			return (V) ((DepositRepository) this).findLicenseDeposit(parentId, id);
		}
		// Deposit<-> Language
		else if (parentClass.equals(Deposit.class) && childClass.equals(Language.class)) {
			return (V) ((DepositRepository) this).findLanguageByParentId(parentId, id);
		}
		// Institution<->Person
		else if (parentClass.equals(Institution.class) && childClass.equals(Person.class)) {
			return (V) ((InstitutionRepository) this).findPeopleByParentId(parentId, id);
		} // Institution<-> Unit
		else if (parentClass.equals(Institution.class) && childClass.equals(OrganizationalUnit.class)) {
			return (V) ((InstitutionRepository) this).findProjectsByParentId(parentId, id);
		} // Person<->Institution
		else if (parentClass.equals(Person.class) && childClass.equals(Institution.class)) {
			return (V) ((PersonRepository) this).findInstitutionsByParentId(parentId, id);
		}// Person<-> Unit
		else if (parentClass.equals(Person.class) && childClass.equals(OrganizationalUnit.class)) {
			return (V) ((PersonRepository) this).findProjectsByParentId(parentId, id);
		} // SIP<->SipDataFile
		else if (parentClass.equals(SubmissionInfoPackage.class) && childClass.equals(SipDataFile.class)) {
			return (V) ((SubmissionInfoPackageRepository) this).findSipDataFileByParentId(parentId, id);
		} // AIP<->AipDataFile
		else if (parentClass.equals(ArchivalInfoPackage.class) && childClass.equals(AipDataFile.class)) {
			return (V) ((ArchivalInfoPackageRepository) this).findAipDataFileByParentId(parentId, id);
		} // DIP<->DAipDataFile
		else if (parentClass.equals(DisseminationInfoPackage.class) && childClass.equals(DipDataFile.class)) {
			return (V) ((DisseminationInfoPackageRepository) this).findDipDataFileByParentId(parentId, id);
		} // Order<->AIP
		else if (parentClass.equals(Order.class) && childClass.equals(ArchivalInfoPackage.class)) {
			return (V) ((OrderRepository) this).findAipPackagesByParentId(parentId, id);
		} // Order<->DIP
		else if (parentClass.equals(Order.class) && childClass.equals(DisseminationInfoPackage.class)) {
			return (V) ((OrderRepository) this).findDipPackagesByParentId(parentId, id);
		} // Unit<->Institution
		else if (parentClass.equals(OrganizationalUnit.class) && childClass.equals(Institution.class)) {
			return (V) ((OrganizationalUnitRepository) this).findInstitutionsByParentId(parentId, id);
		} // Unit<->Person
		else if (parentClass.equals(OrganizationalUnit.class) && childClass.equals(Person.class)) {
			return (V) ((OrganizationalUnitRepository) this).findPeopleByParentId(parentId, id);
		} // Unit<->FundingAgency
		else if (parentClass.equals(OrganizationalUnit.class) && childClass.equals(FundingAgency.class)) {
			return (V) ((OrganizationalUnitRepository) this).findFundingAgencyByParentId(parentId, id);
		} // FundingAgency<->Unit
		else if (parentClass.equals(FundingAgency.class) && childClass.equals(OrganizationalUnit.class)) {
			return (V) ((FundingAgencyRepository) this).findOrganizationalUnitByParentId(parentId, id);
		}
		else {
			throw new UnsupportedOperationException("Missing implementation 'findByParentIdAndChildId' function");
		}
	}

	/**
	 * This figure out by a composite id entity and query using the id's ordered by the query implementation.
	 */
	default <V> V findByCompositeId(Class<?> compositeClass, String... ids) {
		if (compositeClass.equals(OrganizationalUnitPersonRole.class)) {
			List<String> listIds = Arrays.asList(ids);
			return (V) ((OrganizationalUnitPersonRoleRepository) this).findByCompositeId(listIds.get(0), listIds.get(1), listIds.get(2));
		}
		throw new UnsupportedOperationException("Missing 'findByCompositeId' implementation.");
	}
}
