package ch.dlcm.rest;

import java.util.List;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.data.repository.NoRepositoryBean;

import ch.dlcm.exception.DLCMException;

@NoRepositoryBean
public interface RESTRepositoryNoSql<T> {

  // CRUD functions
  public List<T> findAll();

  public T save(T t) throws DLCMException;

  public boolean delete(T t);

  public T findOne(String id);

  // Default Functions
  public default Page<T> search(String query, Pageable pageable) {
    return findAll(pageable);
  }

  public default Page<T> findAll(Pageable pageable) {
    List<T> list = findAll();
    int start = pageable.getOffset();
    int end = (start + pageable.getPageSize()) > list.size() ? list.size()
            : (start + pageable.getPageSize());
    return new PageImpl<>(list.subList(start, end), pageable, list.size());
  }
}
