package ch.dlcm.service;

import java.util.List;

import javax.annotation.PostConstruct;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jms.annotation.JmsListener;
import org.springframework.stereotype.Component;

import ch.dlcm.model.StatusHistory;
import ch.dlcm.repository.HistoryRepository;

@Component
public class HistoryService extends DLCMService {
    private static final Logger log = Logger.getLogger(HistoryService.class);

	@Autowired
	HistoryRepository historyRepository;

	@Override
	@PostConstruct
	protected void loadConfig() {
	}

	@JmsListener(destination = "${dlcm.queue.history}")
	public void saveEvent(StatusHistory event) {
		log.info("Reading message " + event.toString());
		// Read last event
		List<StatusHistory> last = historyRepository.findLast(event.getDlcmId());
		if (last.isEmpty() || !last.get(0).getStatus().equals(event.getStatus())) {
			// Add new event
			checkEvent(event);
			StatusHistory res = historyRepository.save(event);
			if (res == null) {
				log.error("cannot create event history");
			}
		}else{
			log.warn("No new event in history");
		}
	}

	private void checkEvent(StatusHistory event) {
		if (event.getDescription() == null){
			String msg = "status.history";
			if ("Deposit".equals(event.getType())){
				msg = "preingest.deposit.status."+event.getStatus().toLowerCase();
			}else if (event.getType().endsWith("Order")){
				msg = "access.order.status."+event.getStatus().toLowerCase();
			}else if (event.getType().endsWith("DataFile")){
				msg = "datafile.status."+event.getStatus().toLowerCase();
			}else if (event.getType().endsWith("InfoPackage")){
				msg = "package.status."+event.getStatus().toLowerCase();
			}
			event.setDescription(messageService.get(msg));
		}
	}

}
