package ch.dlcm.service;

import java.io.IOException;
import java.net.URL;
import java.net.UnknownHostException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

import javax.annotation.PostConstruct;

import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Component;

import ch.dlcm.model.ComplianceLevel;
import ch.dlcm.model.DataFileInterface;
import ch.dlcm.model.oais.AipDataFile;
import ch.dlcm.model.oais.DipDataFile;
import ch.dlcm.tool.SchemaResourceResolver;
import ch.dlcm.tool.XMLTool;

@Component
public class MetadataService extends DLCMService {

  private String sipXmlSchema;
  private String aipXmlSchema;
  private String dipXmlSchema;

  @PostConstruct
  @Override
  protected void loadConfig() throws UnknownHostException {
    // TODO this.dlcmConfig....
    sipXmlSchema = SchemaResourceResolver.SCHEMA_HOME + "/dlcm_sip.xsd";
    aipXmlSchema = SchemaResourceResolver.SCHEMA_HOME + "/dlcm_aip.xsd";
    dipXmlSchema = SchemaResourceResolver.SCHEMA_HOME + "/dlcm_dip.xsd";
  }

  public boolean checkDlcmMetadata(DataFileInterface df) throws IOException {
    Path mdFile = Paths.get(df.getDataFile().getFinalData());
    if (XMLTool.wellformed(mdFile)) {
      df.getDataFile().setComplianceLevel(ComplianceLevel.AVERAGE_COMPLIANCE);
      if (XMLTool.validate(this.getDlcmSchema(df), mdFile)) {
        df.getDataFile().setComplianceLevel(ComplianceLevel.FULL_COMPLIANCE);
      } else {
        return false;
      }
    }
    return true;
  }

  private List<URL> getDlcmSchema(DataFileInterface df) throws IOException {
		List<URL> list = new ArrayList<>();
		ClassPathResource xsd;
		if (df instanceof DipDataFile){
			xsd = new ClassPathResource(dipXmlSchema);
		}else if (df instanceof AipDataFile){
			xsd = new ClassPathResource(aipXmlSchema);
		}else{
			xsd = new ClassPathResource(sipXmlSchema);
		}
		list.add(xsd.getURL());
		return list;
	}

  public boolean checkExternalMetadata(DataFileInterface df) {
    Path mdFile = Paths.get(df.getDataFile().getFinalData());
    if (XMLTool.wellformed(mdFile)) {
      df.getDataFile().setComplianceLevel(ComplianceLevel.AVERAGE_COMPLIANCE);
      return true;
    }
    return false;
  }

  public boolean checkMetadata(String sipFileUri) {
    // TODO Auto-generated method stub
    return true;
  }

}
