package ch.dlcm.service;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.stereotype.Component;

import ch.dlcm.controller.ModuleController;
import ch.dlcm.controller.ResourceControllerInterface;
import ch.dlcm.rest.RESTResource;

@Component
public class ModuleService<T extends RESTResource<T>> implements ApplicationContextAware {
  private static final Logger log = Logger.getLogger(ModuleService.class);
  private List<ModuleController> moduleControllerList = new ArrayList<>();

  @Value("${dlcm.encrypt.status}")
  private String encryptStatus;

  public List<ModuleController> getModuleControllerList() {
    return moduleControllerList;
  }

  @Override
  public void setApplicationContext(ApplicationContext applicationContext) {
    String[] modList = applicationContext.getBeanNamesForType(ModuleController.class);
    log.info("Modules found : " + modList.length);
    for (String b : modList) {
      log.info("Finding module: " + b);
      moduleControllerList.add((ModuleController) applicationContext.getBean(b));
    }
    String[] resList = applicationContext.getBeanNamesForType(ResourceControllerInterface.class);
    for (String r : resList) {
      log.info("Finding resource: " + r);
      if (!addResource((ResourceControllerInterface) applicationContext.getBean(r)))
        log.error("Resource (" + r + ") unassigned to a module");
    }
    //Display config server encryption status
    log.info("Config server encryption status : " + encryptStatus);

  }

  private boolean addResource(ResourceControllerInterface r) {
    String name = r.getClass().getName();
    String module = name.substring(0, name.lastIndexOf('.')).toLowerCase();
    for (ModuleController mod : moduleControllerList) {
      if (mod.getClass().getCanonicalName().toLowerCase().startsWith(module)) {
        mod.addResource(r);
        return true;
      }
    }
    return false;
  }

}