package ch.dlcm.ingestion.zip;

import java.nio.file.Path;
import java.util.List;

import javax.annotation.PostConstruct;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Profile;
import org.springframework.stereotype.Service;

import ch.dlcm.ingestion.IngestionService;
import ch.dlcm.model.Package;
import ch.dlcm.model.Status.StatusValue;
import ch.dlcm.service.DLCMService;
import ch.dlcm.service.MetadataService;
import ch.dlcm.tool.FileTool;
import ch.dlcm.tool.ZipTool;

@Profile("ing-zip")
@Service
public class ZipIngestionService extends DLCMService implements IngestionService {

	@Autowired
	protected MetadataService metadataService;

	private String ingestionPath;

	@PostConstruct
	@Override
	protected void loadConfig(){
		this.ingestionPath = this.dlcmConfig.getIngestLocation();
	}

	@Override
	public boolean checkPackage(String id) {
		List<Path> list = FileTool.readDirectory(this.getPackageFolder(id));
		if (list.size() != 1){
			return false;
		}
		String sipFileUri = this.getPackageUri(id);
		ZipTool zipTool = new ZipTool(sipFileUri);
		if (zipTool.checkZip()){
			String destination = System.getProperty("java.io.tmpdir") + "/" + id;
			if (zipTool.unzipFiles(destination, Package.getMetadataStructure())){ 
				return metadataService.checkMetadata(destination);
			}
		}
		return false;
	}

	@Override
	public String submitPackage(String id, String name) {
		return id;
	}

	@Override
	public StatusValue getStatus(String id) {
		return StatusValue.Completed;
	}

	@Override
	public String getPackageUri(String id) {
		return FileTool.readDirectory(this.getPackageFolder(id)).get(0).toString();
	}

	private String getPackageFolder(String id) {
		return this.ingestionPath + "/" + id;
	}

	@Override
	public String sendPackage(String id, String name) {
		return this.sendAIP(id, name);
	}

	@Override
	public List<?> listPackages() {
		// TODO Auto-generated method stub
		return null;
	}

}
