package ch.dlcm.service;

import javax.annotation.PostConstruct;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jms.annotation.JmsListener;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import ch.dlcm.ingestion.IngestionService;
import ch.dlcm.message.SipMessage;
import ch.dlcm.model.Status.StatusValue;
import ch.dlcm.model.oais.SubmissionInfoPackage;
import ch.dlcm.repository.SubmissionInfoPackageRepository;

@Component
public class SipMgmt extends DLCMService {
  private static final Logger log = Logger.getLogger(SipMgmt.class);

  @Autowired
  SubmissionInfoPackageRepository repo;

  @Autowired
  IngestionService sipPackageBuilder;

  @PostConstruct
  @Override
  protected void loadConfig() {
  }

  @Transactional
  @JmsListener(destination = "${dlcm.queue.sip}")
  public void processSip(SipMessage sipPayload) {
    log.info("Reading message " + sipPayload.toString());
    SubmissionInfoPackage sip = this.getOne(repo, sipPayload.getDlcmId());
    if (sip == null) {
      log.error("Cannot find SIP (" + sipPayload.getDlcmId() + ")");
      return;
    }
    while (sip.getInfo().getStatus().getValue() != StatusValue.InError && sip.getInfo().getStatus().getValue() != StatusValue.Completed) {
      try {
        switch (sip.getInfo().getStatus().getValue()) {
          case InPreparation:
            if (sip.isAllDataFileReady()) {
              sip.setPackageStatus(StatusValue.Ready);
            }
            break;
          case InProgress:
          case Ready:
            if (sipPackageBuilder.checkPackage(sip.getDlcmId())) {
              sip.setPackageStatus(StatusValue.Checked);
            } else {
              inError(sip, messageService.get("ingest.sip.error.check"));
            }
            break;
          case Checked:
            String aipId;
            if (sipPackageBuilder.getStatus(sip.getDlcmId()) == StatusValue.Completed) {
              sip.setAip(sipPackageBuilder.getPackageUri(sip.getDlcmId()));
              if ((aipId = sipPackageBuilder.sendPackage(sip.getDlcmId(), sip.getInfo().getName())) != null) {
                sip.setAipId(aipId);
                sip.setPackageStatus(StatusValue.Completed);
              } else {
                inError(sip, messageService.get("ingest.sip.error.sending"));
              }
            }
            break;
          default:
            break;
        }
      } catch (Exception e) {
        log.error("Error processing SIP (" + sipPayload.getDlcmId() + ")", e);
        inError(sip, e.getMessage());
      }
      if (repo.save(sip)==null) {
        log.error("SIP cannot be saved: dlcmId=" + sip.getDlcmId() + " status=" + sip.getInfo().getStatus().getValue().toString());
      }
    }
    log.info("SIP processed: dlcmId=" + sip.getDlcmId() + " status=" + sip.getInfo().getStatus().getValue().toString());
  }

  private void inError(SubmissionInfoPackage sip, String msg) {
    sip.setErrorStatus(msg);
    log.error(msg + " for SIP " + sip.getDlcmId());
  }
}
