package ch.dlcm.test.doc;

import static ch.dlcm.test.executor.DLCMExecutorTestDoc.doDocumentationFieldsCycle;
import static ch.dlcm.test.executor.DLCMExecutorTestDoc.doDocumentationLinksCycle;
import static ch.dlcm.test.executor.DLCMExecutorTestDoc.doErrorUrlRestLink;
import static ch.dlcm.test.executor.DLCMExecutorTestDoc.doPerformPostRequestCycle;
import static ch.dlcm.test.executor.DLCMExecutorTestDoc.doPerformRequestCycle;
import static ch.dlcm.test.executor.DLCMExecutorTestDoc.doTestsExpectedCycle;
import static ch.dlcm.test.executor.DLCMTestDocHelper.APPLICATION;
import static ch.dlcm.test.executor.DLCMTestDocHelper.HAL_JSON_MEDIA_TYPE;
import static ch.dlcm.test.executor.DLCMTestDocHelper.HTML_MEDIA_TYPE;
import static ch.dlcm.test.executor.DLCMTestDocHelper.JSON_MEDIA_TYPE;
import static org.springframework.restdocs.mockmvc.MockMvcRestDocumentation.documentationConfiguration;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ThreadLocalRandom;

import org.json.JSONArray;
import org.json.JSONObject;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.HttpStatus;
import org.springframework.restdocs.JUnitRestDocumentation;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.test.context.web.WebAppConfiguration;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.web.context.WebApplicationContext;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;

import ch.dlcm.model.Access;
import ch.dlcm.rest.RESTAction;
import ch.dlcm.rest.RESTName;
import ch.dlcm.test.doc.matcher.ingestion.AdminDocMatcher;
import ch.dlcm.test.doc.matcher.ingestion.AdminInstitutionDocMatcher;
import ch.dlcm.test.doc.matcher.ingestion.IngestDocMatcher;
import ch.dlcm.test.doc.matcher.ingestion.IngestionDocMatcher;
import ch.dlcm.test.doc.matcher.ingestion.PreIngestDocMatcher;
import ch.dlcm.test.executor.DLCMTestDocHelper;

@ActiveProfiles({"test", "pre-zip", "ing-zip"})
@RunWith(SpringJUnit4ClassRunner.class)
@SpringBootTest
@WebAppConfiguration
public class IngestionRestTestsAndDocumentation {

    @Rule
    public final JUnitRestDocumentation restDocumentation = new JUnitRestDocumentation("target/generated-snippets");
    @Autowired
    private WebApplicationContext context;
    private MockMvc mockMvc;


    @Before
    public void setUp() {
        this.mockMvc = MockMvcBuilders.webAppContextSetup(this.context)
                .apply(documentationConfiguration(this.restDocumentation)).build();

    }

    @Test
    public void initIngestionTestAndDocs() throws Exception {

        MvcResult result = doPerformRequestCycle(mockMvc, DLCMTestDocHelper.APPLICATION_ROOT, HAL_JSON_MEDIA_TYPE, HttpStatus.OK).andReturn();
        JsonNode links = new ObjectMapper().readTree(result.getResponse().getContentAsString()).get("_links");

        // Execute ingestion "_links"
        DLCMTestDocHelper.getIngestionRootActions().forEach(action -> {
            try {
                if (links.has(action)) {
                    String url = links.get(action).get("href").asText();
                    switch (action) {
                        case RESTAction.SELF:
                            generateIngestionDoc(DLCMTestDocHelper.APPLICATION_ROOT);
                            break;
                        case RESTName.ADMIN:
                            generateAdminDoc(url);
                            generateAdminInstitutionDoc(url + "/" + RESTName.INSTITUTION);
                            generateAdminInstitutionPostDoc(url + "/" + RESTName.INSTITUTION);
                            generateAdminPeopleDoc(url + "/" + RESTName.PERSON);
                            break;
                        case RESTName.PREINGEST:
                            generatePreingestDoc(url);
                            generatePreingestDepositsAddPeopleDoc(url + "/" + RESTName.DEPOSIT);
                            break;
                        case RESTName.INGEST:
                            generateingestSipPostDoc(url + "/" + RESTName.SIP);
                            generateIngestSipResume(url + "/" + RESTName.SIP);
                            generateIngestSipDataResume(url + "/" + RESTName.SIP);
                            break;
                        case RESTAction.DASHBOARD:
                            generateDashboardDoc(url);
                            break;
                    }
                }
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        });
    }

    //	================================
    //	DASHBOARD
    //	================================

    private void generateDashboardDoc(String url) throws Exception {
        doPerformRequestCycle(mockMvc, url, HTML_MEDIA_TYPE, HttpStatus.OK);
    }

    //	================================
    //	INGESTION
    //	================================

    public MvcResult generateIngestionDoc(String url) throws Exception {
        ResultActions request = doPerformRequestCycle(mockMvc, url, HAL_JSON_MEDIA_TYPE, HttpStatus.OK);
        doTestsExpectedCycle(request, IngestionDocMatcher.getPreIngestRootExpectedMatchers());
        doDocumentationLinksCycle(request, APPLICATION, IngestionDocMatcher.getPreIngestRootMapLinksDescription());
        return request.andReturn();
    }

    //	================================
    //	PRE-INGEST
    //	================================

    private void generatePreingestDoc(String url) throws Exception {
        ResultActions request = doPerformRequestCycle(mockMvc, url, HAL_JSON_MEDIA_TYPE, HttpStatus.OK);
        doTestsExpectedCycle(request, PreIngestDocMatcher.getPreIngestExpectedMatchers());
        doDocumentationFieldsCycle(request, RESTName.PREINGEST, PreIngestDocMatcher.getPreIngestMapFieldsDescription());
        doDocumentationLinksCycle(request, RESTName.PREINGEST, PreIngestDocMatcher.getPreIngestMapLinksDescription());
    }

    private void generatePreingestDepositsAddPeopleDoc(String url) throws Exception {
        // Create people and deposit
        Map<String, String> depoPeople = createDepositsAndPeople(url);
        String peopleId = depoPeople.get(RESTName.PERSON);
        String depositsId = depoPeople.get(RESTName.DEPOSIT);

        // Add people to deposits and check post links
        doPerformPostRequestCycle(mockMvc, url + "/"+ depositsId +"/" + RESTName.PERSON , JSON_MEDIA_TYPE, HttpStatus.CREATED, new JSONArray().put(peopleId).toString() );

        // Check get links after post
        ResultActions request = doPerformRequestCycle(mockMvc, url + "/"+ depositsId +"/" + RESTName.PERSON , HAL_JSON_MEDIA_TYPE, HttpStatus.OK);
        doTestsExpectedCycle(request, PreIngestDocMatcher.getPreIngestDepositsPeopleExpectedMatchers(depositsId));

        // Deep: Check people id after
        request = doPerformRequestCycle(mockMvc, url + "/"+ depositsId +"/" + RESTName.PERSON  + "/" + peopleId, HAL_JSON_MEDIA_TYPE, HttpStatus.OK);
        doTestsExpectedCycle(request, PreIngestDocMatcher.getPreIngestAdminPeopleIDExpectedMatchers(peopleId));

    }

    private Map<String, String> createDepositsAndPeople(String url) throws Exception {
        Map<String, String> depoPeople = new HashMap<>();
        ResultActions request = createAnyPeople();
        depoPeople.put(RESTName.PERSON ,String.valueOf(new JSONObject(request.andReturn().getResponse().getContentAsString()).get("dlcmId")));
        request = createAnyDeposits();
        depoPeople.put(RESTName.DEPOSIT, String.valueOf(new JSONObject(request.andReturn().getResponse().getContentAsString()).get("dlcmId")));
        return depoPeople;
    }


    private ResultActions createAnyDeposits() throws Exception{
        JSONObject deposits = new JSONObject().put("title", "any deposity name").put("access", Access.Embargoed);
        return doPerformPostRequestCycle(mockMvc, DLCMTestDocHelper.APPLICATION_ROOT + RESTName.PREINGEST + "/" + RESTName.DEPOSIT, HAL_JSON_MEDIA_TYPE, HttpStatus.CREATED, deposits.toString());
    }

    //	================================
    //	ADMIN
    //	================================

    private ResultActions createAnyPeople() throws Exception {
        JSONObject people = new JSONObject().put("firstName", "any first name").put("lastName", "any last name");
        return doPerformPostRequestCycle(mockMvc, DLCMTestDocHelper.APPLICATION_ROOT + RESTName.ADMIN + "/" + RESTName.PERSON, HAL_JSON_MEDIA_TYPE, HttpStatus.CREATED, people.toString());
    }

    private void generateAdminDoc(String url) throws Exception {
        ResultActions request = doPerformRequestCycle(mockMvc, url, HAL_JSON_MEDIA_TYPE, HttpStatus.OK);
        doTestsExpectedCycle(request, AdminDocMatcher.getAdminExpectedMatchers());
        doDocumentationFieldsCycle(request, RESTName.ADMIN, AdminDocMatcher.getAdminMapFieldsDescription());
        doDocumentationLinksCycle(request, RESTName.ADMIN, AdminDocMatcher.getAdminMapLinksDescription());
    }

    private void generateAdminInstitutionDoc(String url) throws Exception {
        ResultActions request = doPerformRequestCycle(mockMvc, url, HAL_JSON_MEDIA_TYPE, HttpStatus.OK);
        doTestsExpectedCycle(request, AdminInstitutionDocMatcher.getAdminInstitutionExpectedMatchers());
        doDocumentationFieldsCycle(request, RESTName.INSTITUTION, AdminInstitutionDocMatcher.getAdminInstitutionMapFieldsDescription());
        doDocumentationLinksCycle(request, RESTName.ADMIN, AdminInstitutionDocMatcher.getAdminInstitutionMapLinksDescription());
    }

    private void generateAdminInstitutionPostDoc(String url) throws Exception {
        // Docs for institutions with post return
        String institutionName = "mockmvc" + ThreadLocalRandom.current().nextInt();
        JSONObject institution = new JSONObject().put("name", institutionName).put("description","Description de " + institutionName);
        ResultActions request = doPerformPostRequestCycle(mockMvc, url, HAL_JSON_MEDIA_TYPE, HttpStatus.CREATED, institution.toString());
		doTestsExpectedCycle(request, AdminInstitutionDocMatcher.getAdminInstitutionGetORPostExpectedMatchers());
		doDocumentationFieldsCycle(request, RESTName.INSTITUTION, AdminInstitutionDocMatcher.getAdminInstitutionPostMapFieldsDescription());
		// Docs for institution getting id after post
		String institutionId = String.valueOf(new JSONObject(request.andReturn().getResponse().getContentAsString()).get("dlcmId"));
        request = doPerformRequestCycle(mockMvc, url + "/" + institutionId, HAL_JSON_MEDIA_TYPE, HttpStatus.OK);
        doTestsExpectedCycle(request, AdminInstitutionDocMatcher.getAdminInstitutionGetORPostExpectedMatchers());
        doDocumentationLinksCycle(request, RESTName.INSTITUTION, AdminInstitutionDocMatcher.getAdminInstitutionsPostMapLinksDescription());
    }

    private void generateAdminPeopleDoc(String url) {
        // TODO check it: this admin/people return a different hateos links than deposit/{depoId}/people/{peopleId}
    }

    //	================================
    //	INGEST
    //	================================

    private ResultActions createAnySip() throws Exception{
        Object obj = null;
        JSONObject sip = new JSONObject().put("depositId", obj);
        return doPerformPostRequestCycle(mockMvc, DLCMTestDocHelper.APPLICATION_ROOT + RESTName.INGEST + "/" + RESTName.SIP, HAL_JSON_MEDIA_TYPE, HttpStatus.CREATED, sip.toString());
    }

    private ResultActions createAnySipData(String sipId) throws Exception{
        JSONObject sipData = new JSONObject().put("sourceData", "any-deposit-id/sip"); // required with a / because filename DataFile.getFileName() construction
        return doPerformPostRequestCycle(mockMvc, DLCMTestDocHelper.APPLICATION_ROOT + RESTName.INGEST + "/" + RESTName.SIP + "/" + sipId + "/"+ RESTName.DATAFILE, HAL_JSON_MEDIA_TYPE, HttpStatus.CREATED, sipData.toString());
    }

    private void generateingestSipPostDoc(String url) throws Exception {
        ResultActions request = createAnySip();
        doTestsExpectedCycle(request, IngestDocMatcher.getIngestSipExpectedMatchers());
    }

    private void generateIngestSipResume(String url) throws Exception {
        // create sip
        ResultActions request = createAnySip();
        doTestsExpectedCycle(request, IngestDocMatcher.getIngestSipExpectedMatchers());
        String sipId = String.valueOf(new JSONObject(request.andReturn().getResponse().getContentAsString()).get("dlcmId"));
        // do sip resume
        request = doPerformRequestCycle(mockMvc, url + "/" + sipId + "/" + RESTName.RESUME, HAL_JSON_MEDIA_TYPE, HttpStatus.OK);
        doTestsExpectedCycle(request, IngestDocMatcher.getIngestSipResumeExpectedMatchers(sipId));
    }

    private void generateIngestSipDataResume(String url) throws Exception {
        // create sip
        ResultActions request = createAnySip();
        doTestsExpectedCycle(request, IngestDocMatcher.getIngestSipExpectedMatchers());
        String sipId = String.valueOf(new JSONObject(request.andReturn().getResponse().getContentAsString()).get("dlcmId"));
        // do sip data file and resume
        request = createAnySipData(sipId);
        String sipDataId = String.valueOf(new JSONObject(request.andReturn().getResponse().getContentAsString()).get("dlcmId"));
        request = doPerformRequestCycle(mockMvc, url + "/" + sipId + "/" + RESTName.DATAFILE + "/" + sipDataId + "/" + RESTName.RESUME, HAL_JSON_MEDIA_TYPE, HttpStatus.OK);
        // sipdata resume shared with sip/resume
        doTestsExpectedCycle(request, IngestDocMatcher.getIngestSipResumeExpectedMatchers(sipDataId));
    }

    @Test
    public void invalidStoragionRootUrlTest() throws Exception {
        doErrorUrlRestLink(mockMvc, "/dlcm/thisdoesntexists");
    }

}
