package ch.dlcm.converter;

import javax.persistence.AttributeConverter;
import javax.persistence.Converter;
import java.util.Arrays;
import java.util.List;

import static java.util.Collections.emptyList;
import static java.util.Collections.sort;

@Converter
public class KeywordsConverter implements AttributeConverter<List<String>, String>{

    @Override
    public String convertToDatabaseColumn(List<String> keywords) {
        if (keywords == null){
            return null;
        }
        StringBuilder keywordBuilder = new StringBuilder();
        for (String keyword : keywords) {
            keywordBuilder.append(keyword);
            keywordBuilder.append(" ");
        }
        return keywordBuilder.toString();
    }

    @Override
    public List<String> convertToEntityAttribute(String keywords) {
        if (keywords == null){
            return emptyList();
        }
        List<String> keywordsList = Arrays.asList(keywords.split(" "));
        sort(keywordsList);
        return keywordsList;
    }
}
