package ch.dlcm.criteria;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import org.springframework.data.jpa.domain.Specification;

import ch.dlcm.model.Order;

public class OrderSpecification implements Specification<Order>{
    
    private final Order criteria;
    
    public OrderSpecification(Order criteria) {
        this.criteria=criteria;
    }

    @Override
    public Predicate toPredicate(Root<Order> root, CriteriaQuery<?> query, CriteriaBuilder builder) {
        List<Predicate> listPredicate = new ArrayList<>();
        if(criteria.getName() != null){
            listPredicate.add(builder.like(root.get("name"), "%" + criteria.getName() + "%"));
        }
        if(criteria.getStatus().getValue() != null){
        	listPredicate.add(builder.equal(root.get("status").get("value"), criteria.getStatus().getValue()));        	
        }
        if(criteria.getStatus().getInfo() != null){
        	listPredicate.add(builder.equal(root.get("status").get("info"), criteria.getStatus().getInfo()));        	
        }
        if(criteria.getQuery() != null){
            listPredicate.add(builder.like(root.get("query"), "%" + criteria.getQuery() + "%"));
        }
        if(criteria.getQueryType() != null){
          listPredicate.add(builder.equal(root.get("queryType"), criteria.getQueryType()));
        }
        return builder.and(listPredicate.toArray(new Predicate[listPredicate.size()])); 
    }

}
