package ch.dlcm.model;

import java.time.OffsetDateTime;
import java.time.ZoneOffset;

import javax.persistence.Column;
import javax.persistence.Embeddable;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.validation.constraints.NotNull;

import com.fasterxml.jackson.annotation.JsonFormat;

import ch.dlcm.tool.StringTool;

@Embeddable
public class DataFileChecksum implements JPAEmbeddableEntityInterface<DataFileChecksum> {
	@NotNull
	@Enumerated(EnumType.STRING)
	private ChecksumAlgorithm checksumAlgo;

	@NotNull
	@Enumerated(EnumType.STRING)
	private ChecksumType checksumType;
	
	@NotNull
	@Enumerated(EnumType.STRING)
	private ChecksumOrigin checksumOrigin;
	
	// Embedded classes
	public enum ChecksumAlgorithm {
		MD5, SHA1, SHA256
	}

	public enum ChecksumType {
		Complete, Partial
	}
	
	public enum ChecksumOrigin {
		Provided, DLCM
	}

	@NotNull
	private String checksum;
	
	@NotNull
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = StringTool.DATE_FORMAT)
    @Column(length=3)   //Used by MySQL5ExtendedDialect
    private OffsetDateTime creationTime;
		
	public DataFileChecksum(){
		this.checksumAlgo = ChecksumAlgorithm.MD5;
		this.checksumOrigin = ChecksumOrigin.DLCM;
		this.checksumType = ChecksumType.Complete;
		this.creationTime = OffsetDateTime.now(ZoneOffset.UTC);
	}
	
	public DataFileChecksum(ChecksumAlgorithm algo, ChecksumType type, ChecksumOrigin origin){
		this.checksumAlgo = algo;
		this.checksumOrigin = origin;
		this.checksumType = type;
		this.creationTime = OffsetDateTime.now(ZoneOffset.UTC);
	}
	
	public DataFileChecksum(ChecksumAlgorithm algo, ChecksumType type, ChecksumOrigin origin, String checksum){
		this.checksumAlgo = algo;
		this.checksumOrigin = origin;
		this.checksumType = type;
		this.checksum = checksum;
		this.creationTime = OffsetDateTime.now(ZoneOffset.UTC);
	}

	public void init() {
		if (this.creationTime == null){
			this.creationTime = OffsetDateTime.now(ZoneOffset.UTC);
		}
	}
	
	public String getChecksum() {
		return checksum;
	}

	public void setChecksum(String checksum) {
		this.checksum = checksum;
	}


	public OffsetDateTime getCreationTime() {
		return creationTime;
	}

	public void setCreationTime(OffsetDateTime creationTime) {
		this.creationTime = creationTime;
	}

	
	public ChecksumAlgorithm getChecksumAlgo() {
		return checksumAlgo;
	}

	public void setChecksumAlgo(ChecksumAlgorithm checksumAlgo) {
		this.checksumAlgo = checksumAlgo;
	}

	public ChecksumType getChecksumType() {
		return checksumType;
	}

	public void setChecksumType(ChecksumType checksumType) {
		this.checksumType = checksumType;
	}

	public ChecksumOrigin getChecksumOrigin() {
		return checksumOrigin;
	}

	public void setChecksumOrigin(ChecksumOrigin checksumOrigin) {
		this.checksumOrigin = checksumOrigin;
	}

	@Override
	public boolean update(DataFileChecksum df) {
		boolean changed = false;
		if (StringTool.toUpdate(this.getChecksum(), df.getChecksum())) {
			this.setChecksum(df.getChecksum());
			changed = true;
		}
		return changed;
	}

	 @Override
   public boolean equals(Object o) {
       if (this == o) return true;
       if (!(o instanceof DataFileChecksum)) return false;
       if (!super.equals(o)) return false;

       DataFileChecksum that = (DataFileChecksum) o;

       if (getChecksumAlgo() != null ? !getChecksumAlgo().equals(that.getChecksumAlgo()) : that.getChecksumAlgo() != null)
           return false;
       if (getChecksumOrigin() != null ? !getChecksumOrigin().equals(that.getChecksumOrigin()) : that.getChecksumOrigin() != null)
         return false;
       if (getChecksumType() != null ? !getChecksumType().equals(that.getChecksumType()) : that.getChecksumType() != null)
         return false;
       if (getCreationTime() != null ? !getCreationTime().equals(that.getCreationTime()) : that.getCreationTime() != null)
         return false;
       return getChecksum() != null ? getChecksum().equals(that.getChecksum()) : that.getChecksum() == null;
   }

   @Override
   public int hashCode() {
       int result = super.hashCode();
       result = 31 * result + (getChecksumAlgo() != null ? getChecksumAlgo().hashCode() : 0);
       result = 31 * result + (getChecksumOrigin() != null ? getChecksumOrigin().hashCode() : 0);
       result = 31 * result + (getChecksumType() != null ? getChecksumType().hashCode() : 0);
       result = 31 * result + (getCreationTime() != null ? getCreationTime().hashCode() : 0);
       result = 31 * result + (getChecksum() != null ? getChecksum().hashCode() : 0);
       return result;
   }
}