package ch.dlcm.model;

import java.beans.Transient;
import java.time.OffsetDateTime;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.ManyToMany;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import org.springframework.data.jpa.domain.Specification;
import org.springframework.hateoas.mvc.ControllerLinkBuilder;

import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.dlcm.criteria.FundingAgencySpecification;
import ch.dlcm.rest.RESTName;
import ch.dlcm.rest.RESTResource;
import ch.dlcm.rest.RESTSubResourceInterface;
import ch.dlcm.tool.StringTool;

@Entity
public class FundingAgency extends RESTResource<FundingAgency> implements RESTSubResourceInterface {
  
  @NotNull
  @Column(unique = true)
  @Size(min = 1, max = 256)
  private String name;

  @NotNull
  @Column(unique = true)
  @Size(min = 1, max = 100)
  private String acronym;
  
  @Size(max = 1024)
  private String description;

  @Size(max = 256)
  private String url;
  
  @ManyToMany(mappedBy = "fundingAgencies")
  @JsonIgnore
  private List<OrganizationalUnit> organizationalUnits;
  
  /***************************************************/
  
  @Override
  public void init() {
  }
  
  @Override
  public void addLinks(ControllerLinkBuilder clb, boolean mainRes, boolean subResOnly) {
    
    super.addLinks(clb, mainRes, subResOnly);
    
      if (mainRes) {
        this.add(clb.slash(getDlcmId()).slash(RESTName.UNIT).withRel(RESTName.UNIT));
      }
  }

  /***************************************************/

  public String getName() {
    return this.name;
  }

  public void setName(String name) {
    this.name = name;
    updateTime();
  }

  public String getAcronym() {
    return this.acronym;
  }

  public void setAcronym(String acronym) {
    this.acronym = acronym;
    updateTime();
  }
  
  public String getUrl() {
    return this.url;
  }

  public void setUrl(String url) {
    this.url = url;
    updateTime();
  }

  public String getDescription() {
    return description;
  }

  public void setDescription(String description) {
    this.description = description;
    updateTime();
  }

  /***************************************************/
  
  @Override
  public <T> boolean addItem(T t) {
    if(t instanceof OrganizationalUnit) {
      return this.addOrganizationalUnit((OrganizationalUnit) t);
    }
    return false;
  }
  
  @Override
  public <T> boolean removeItem(T t) {
    if(t instanceof OrganizationalUnit) {
      return this.removeOrganizationalUnit((OrganizationalUnit) t);
    }
    return false;
  }
  
  public List<OrganizationalUnit> getOrganizationalUnits() {
    return organizationalUnits;
  }
  
  public boolean addOrganizationalUnit(OrganizationalUnit organizationalUnit) {
    
    this.updateTime();
    
    boolean result = this.organizationalUnits.add(organizationalUnit);
    
    if (result) {
      if (!organizationalUnit.getFundingAgencies().contains(this)) {
        result = organizationalUnit.addFundingAgency(this);
      }
    }
    
    return result;
  }
  
  public boolean removeOrganizationalUnit(OrganizationalUnit organizationalUnit) {
    
    boolean result = true;
    
    if(this.organizationalUnits.contains(organizationalUnit)) {
      
      this.updateTime();
      
      result = this.organizationalUnits.remove(organizationalUnit);
      
      if (result) {
        if (organizationalUnit.getFundingAgencies().contains(this)) {
          result = organizationalUnit.removeFundingAgency(this);
        }
      }
    }
    
    return result;
  }
  
  /***************************************************/

  @Override
  public boolean update(RESTResource<FundingAgency> item) {
    
    FundingAgency t = (FundingAgency)item;
    
    OffsetDateTime lastUpdate = this.getUpdateTime();
    
    if (StringTool.toUpdate(this.getName(), t.getName())) {
      this.setName(t.getName());
    }
    
    if (StringTool.toUpdate(this.getAcronym(), t.getAcronym())) {
      this.setAcronym(t.getAcronym());
    }
    
    if (StringTool.toUpdate(this.getDescription(), t.getDescription())) {
      this.setDescription(t.getDescription());
    }
    
    if (StringTool.toUpdate(this.getUrl(), t.getUrl())) {
      this.setUrl(t.getUrl());
    }
    
    if (lastUpdate == this.getUpdateTime()) {
      return false;
    } else {
      return true;
    }
  }
  
  @Override
  @Transient
  @JsonIgnore
  public Specification<FundingAgency> getSpecification() {
    return new FundingAgencySpecification(this);
  }
  
  @Override
  public String managedBy() {
    return RESTName.ADMIN;
  }
}
