package ch.dlcm.model;

import java.beans.Transient;
import java.time.OffsetDateTime;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.ManyToMany;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import org.springframework.data.jpa.domain.Specification;
import org.springframework.hateoas.mvc.ControllerLinkBuilder;

import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.dlcm.criteria.InstitutionSpecification;
import ch.dlcm.rest.RESTName;
import ch.dlcm.rest.RESTResource;
import ch.dlcm.rest.RESTSubResourceInterface;
import ch.dlcm.tool.StringTool;

@Entity
public class Institution extends RESTResource<Institution> implements RESTSubResourceInterface {
	@NotNull
	@Column(unique = true)
	@Size(min = 1, max = 256)
	private String name;

	@Size(max = 1024)
	private String description;

	@ManyToMany(mappedBy = "institutions")
	@JsonIgnore
	private List<Person> people;

	@ManyToMany(mappedBy = "institutions")
	@JsonIgnore
	private List<OrganizationalUnit> projects;

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
		updateTime();
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
		updateTime();
	}

	public List<Person> getPeople() {
		return people;
	}

	@Override
	public void init() {
	}

	@Override
	public boolean update(RESTResource<Institution> item) {
		Institution t = (Institution)item;
		OffsetDateTime lastUpdate = this.getUpdateTime();
		if (StringTool.toUpdate(this.getName(), t.getName()))
			this.setName(t.getName());
		if (StringTool.toUpdate(this.getDescription(), t.getDescription()))
			this.setDescription(t.getDescription());
		if (lastUpdate == this.getUpdateTime())
			return false;
		else
			return true;
	}

	@Override
	@Transient
	@JsonIgnore
	public Specification<Institution> getSpecification() {
		return new InstitutionSpecification(this);
	}

	@Override
	public <T> boolean addItem(T t) {
		if (t instanceof Person){
			return this.addPerson((Person) t);
		}
		if (t instanceof OrganizationalUnit){
			return this.addProject((OrganizationalUnit) t);
		}
		return false;
	}

	public boolean addPerson(Person p) {
		updateTime();
		boolean result = people.add(p);
		if(!p.getInstitutions().contains(this)){
	        p.addInstitution(this);
		}
		return result;
	}
	
	public boolean addProject(OrganizationalUnit project) {
		updateTime();
		boolean result = projects.add(project);
		if(!project.getInstitutions().contains(this)){
			project.addInstitution(this);
		}
		return result;
	}

	public List<OrganizationalUnit> getProjects() {
		return projects;
	}

	@Override
	public <T> boolean removeItem(T t) {
		if (t instanceof Person){
			return this.removePerson((Person) t);
		}
		if (t instanceof OrganizationalUnit){
			return this.removeProject((OrganizationalUnit) t);
		}
		return false;
	}

	public boolean removePerson(Person p) {
		updateTime();
		boolean result = people.remove(p);
		if(p.getInstitutions().contains(this)){
		    p.removeInstitution(this);
		}
		return result;
	}
	
	public boolean removeProject(OrganizationalUnit project) {
		updateTime();
		boolean result = projects.remove(project);
		if(project.getInstitutions().contains(this)){
			project.removeInstitution(this);
		}
		return result;
	}

	@Override
	public void addLinks(ControllerLinkBuilder clb, boolean mainRes, boolean subResOnly) {
		super.addLinks(clb, mainRes, subResOnly);
		if (mainRes){
			this.add(clb.slash(getDlcmId()).slash(RESTName.PERSON).withRel(RESTName.PERSON));
			this.add(clb.slash(getDlcmId()).slash(RESTName.UNIT).withRel(RESTName.UNIT));
		}
	}

	@Override
	public String managedBy() {
		return RESTName.ADMIN;
	}
}
