package ch.dlcm.model;

import ch.dlcm.criteria.LanguageSpecification;
import ch.dlcm.rest.RESTName;
import ch.dlcm.rest.RESTResource;
import ch.dlcm.tool.StringTool;
import org.springframework.data.jpa.domain.Specification;

import javax.persistence.Column;
import javax.persistence.Entity;
import java.time.OffsetDateTime;

/**
 * Language representation names based in ISO 639-1 (2002)
 */
@Entity
public class Language extends RESTResource<Language> {


    @Column(unique = true, nullable = false)
    private String code;

    public String getCode() {
        return code;
    }

    public void setCode(String code) {
        this.code = code;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        Language language = (Language) o;

        if (code != null ? !code.equals(language.code) : language.code != null) return false;

        return true;
    }

    @Override
    public int hashCode() {
        return code != null ? code.hashCode() : 0;
    }

    @Override
    public Specification<Language> getSpecification() {
        return new LanguageSpecification(this);
    }

    @Override
    public void init() {}

    @Override
    public boolean update(RESTResource<Language> resource) {
        Language language = (Language) resource;
        OffsetDateTime lastUpdate = this.getUpdateTime();

        if (StringTool.toUpdate(this.getCode(), language.getCode())) {
            this.setCode(language.getCode());
        }

        if (lastUpdate == this.getUpdateTime()) {
            return false;
        } else {
            return true;
        }
    }

    @Override
    public String managedBy() {
        return RESTName.ADMIN;
    }
}
