package ch.dlcm.model;

import ch.dlcm.criteria.RoleSpecification;
import ch.dlcm.rest.RESTName;
import ch.dlcm.rest.RESTResource;
import ch.dlcm.tool.StringTool;
import com.fasterxml.jackson.annotation.JsonIgnore;
import org.springframework.data.jpa.domain.Specification;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import java.time.OffsetDateTime;
import java.util.ArrayList;
import java.util.List;

@Entity
public class Role extends RESTResource<Role> {

    public static final String DEFAULT_ROLE = "Visitor";

    @NotNull
    @Column(unique = true)
    private String name;

    @JsonIgnore
    @OneToMany(mappedBy = "dlcmCompositeId.role", cascade = CascadeType.ALL, fetch = FetchType.LAZY)
    private List<OrganizationalUnitPersonRole> organizationalUnitPersonRoles = new ArrayList<>();

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
        updateTime();
    }

    @Override
    public Specification<Role> getSpecification() {
        return new RoleSpecification(this);
    }

    @Override
    public void init() {}

    @Override
    public boolean update(RESTResource<Role> resource) {
        Role role = (Role) resource;
        OffsetDateTime lastUpdate = this.getUpdateTime();

        // Name
        if (StringTool.toUpdate(this.getName(), role.getName())) {
            this.setName(role.getName());
        }

        if (lastUpdate == this.getUpdateTime()) {
            return false;
        } else {
            return true;
        }    }

    @Override
    public String managedBy() {
        return RESTName.ADMIN;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof Role)) return false;
        if (!super.equals(o)) return false;

        Role role = (Role) o;

        if (getName() != null ? !getName().equals(role.getName()) : role.getName() != null) return false;
        return true;
    }

    @Override
    public int hashCode() {
        int result = super.hashCode();
        result = 31 * result + (getName() != null ? getName().hashCode() : 0);
        return result;
    }

    public List<OrganizationalUnitPersonRole> getOrganizationalUnitPersonRoles() {
        return organizationalUnitPersonRoles;
    }
}
