package ch.dlcm.model;

import java.io.Serializable;
import java.time.OffsetDateTime;
import java.time.ZoneOffset;
import java.util.Objects;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.validation.constraints.NotNull;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.dlcm.tool.StringTool;

@Entity
public class StatusHistory implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = 481362136969468794L;

	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	@JsonIgnore
	private long seq;
	
	@NotNull
	@JsonIgnore
	private String dlcmId;
	
	@NotNull
	@JsonIgnore
	private String type;
	
	@NotNull
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = StringTool.DATE_FORMAT)
    @Column(length=3)   //Used by MySQL5ExtendedDialect
    private OffsetDateTime changeTime;

	@NotNull
	private String status;

	private String description;
	
	@Override
	public boolean equals(final Object other) {
		if (!(other instanceof StatusHistory)) {
			return false;
		}
		StatusHistory castOther = (StatusHistory) other;
		return Objects.equals(seq, castOther.seq) && Objects.equals(dlcmId, castOther.dlcmId)
				&& Objects.equals(type, castOther.type) && Objects.equals(changeTime, castOther.changeTime)
				&& Objects.equals(status, castOther.status) && Objects.equals(description, castOther.description);
	}

	@Override
	public int hashCode() {
		return Objects.hash(seq, dlcmId, type, changeTime, status, description);
	}

	public StatusHistory(){
	}
	
	public StatusHistory(String type, String id, String sts){
		this.type = type;
		this.dlcmId = id;
		this.status = sts;
		this.changeTime = OffsetDateTime.now(ZoneOffset.UTC);
	}
	
	public StatusHistory(String type, String id, OffsetDateTime datetime){
		this.type = type;
		this.dlcmId = id;
		this.status = "Created";
		this.changeTime = datetime;
	}
	
	public StatusHistory(String type, String id, OffsetDateTime datetime, String sts, String desc){
		this.type = type;
		this.dlcmId = id;
		this.status = sts;
		this.changeTime = datetime;
		this.description = desc;
	}

	public long getSeq() {
		return seq;
	}

	public String getType() {
		return type;
	}
	
	public String getDlcmId() {
		return dlcmId;
	}
	
	public String getStatus() {
		return status;
	}

	public OffsetDateTime getChangeTime() {
		return changeTime;
	}

	public String getDescription() {
		return description;
	}

	public static long getSerialversionuid() {
		return serialVersionUID;
	}	

	public void setDescription(String description) {
		this.description = description;
	}

	@Override
	public String toString() {
		return "StatusHistory [dlcmId=" + dlcmId + ", type=" + type + ", changeTime=" + changeTime
				+ ", status=" + status + ", description=" + description + "]";
	}
	
}