package ch.dlcm.model;

import ch.dlcm.criteria.SubmissionAgreementSpecification;
import ch.dlcm.rest.RESTName;
import ch.dlcm.rest.RESTResource;
import ch.dlcm.tool.StringTool;
import org.springframework.data.jpa.domain.Specification;

import javax.persistence.Entity;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.time.OffsetDateTime;

@Entity
public class SubmissionAgreement extends RESTResource<SubmissionAgreement> {


    @NotNull
    private String title;

    @Size(max = 1024)
    private String description;
    private String version;


    @Override
    public Specification<SubmissionAgreement> getSpecification() {
        return new SubmissionAgreementSpecification(this);
    }

    @Override
    public void init() {
    }

    @Override
    public boolean update(RESTResource<SubmissionAgreement> resource) {
        SubmissionAgreement submissionAgreement = (SubmissionAgreement) resource;
        OffsetDateTime lastUpdate = this.getUpdateTime();
        // Title
        if (StringTool.toUpdate(this.getTitle(), submissionAgreement.getTitle())){
            this.setTitle(submissionAgreement.getTitle());
        }
        // Description
        if(StringTool.toUpdate(this.getDescription(), submissionAgreement.getDescription())){
            this.setDescription(submissionAgreement.getDescription());
        }
        // Version
        if(StringTool.toUpdate(this.getVersion(), submissionAgreement.getVersion())){
            this.setVersion(submissionAgreement.getVersion());
        }

        if (lastUpdate == this.getUpdateTime()){
            return false;
        }
        else{
            return true;
        }
    }

    @Override
    public String managedBy() {
        return RESTName.ADMIN;
    }

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
        updateTime();
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
        updateTime();
    }

    public String getVersion() {
        return version;
    }

    public void setVersion(String version) {
        this.version = version;
        updateTime();
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof SubmissionAgreement)) return false;
        if (!super.equals(o)) return false;

        SubmissionAgreement submissionAgreement = (SubmissionAgreement) o;

        if (getTitle() != null ? !getTitle().equals(submissionAgreement.getTitle()) : submissionAgreement.getTitle() != null) return false;
        if (getVersion() != null ? !getVersion().equals(submissionAgreement.getVersion()) : submissionAgreement.getVersion() != null) return false;
        if (getDescription() != null ? !getDescription().equals(submissionAgreement.getDescription()) : submissionAgreement.getDescription() != null) return false;
        return true;
    }

    @Override
    public int hashCode() {
        int result = super.hashCode();
        result = 31 * result + (getTitle() != null ? getTitle().hashCode() : 0);
        result = 31 * result + (getVersion() != null ? getVersion().hashCode() : 0);
        result = 31 * result + (getDescription() != null ? getDescription().hashCode() : 0);
        return result;
    }
}
