package ch.dlcm.model.display.mapper;

import ch.dlcm.model.OrganizationalUnit;
import ch.dlcm.model.OrganizationalUnitPersonRole;
import ch.dlcm.model.Person;
import ch.dlcm.model.Role;
import ch.dlcm.model.display.RessourceRole;
import ch.dlcm.rest.RESTResource;
import com.fasterxml.jackson.annotation.JsonIgnore;
import org.apache.log4j.Logger;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Predicate;

public class ResourceRoleMapper<Resource> {

  private static final Logger log = Logger.getLogger(ResourceRoleMapper.class);

  public Resource getResourceRole() {
    return (Resource) resourceRole;
  }

  @JsonIgnore
  protected RESTResource<Resource> resourceRole;

  private ResourceRoleMapper(){}

  public ResourceRoleMapper(RESTResource resourceRole, Resource resource) {
    this.resourceRole = resourceRole;
    this.mapResourceToResourceRole(resource);
  }

  public void doRoleFilter(Person person, OrganizationalUnit organizationalUnit){
    person.getProjects().stream().forEach(organizationalUnitConsumer -> {
      List<Role> roles = new ArrayList<>();
      final Predicate<OrganizationalUnitPersonRole> projectCriteria = listPrecicate -> organizationalUnitConsumer.getDlcmId().equals(organizationalUnit.getDlcmId());
      person.getOrganizationalUnitPersonRoles().stream().filter(projectCriteria).forEach(organizationalUnitPersonRole1 ->
              roles.add(organizationalUnitPersonRole1.getRole())
      );
      ((RessourceRole) this.resourceRole).getRoles().addAll(roles);
    });
  }

  protected void mapResourceToResourceRole(Resource resource) {
    Class[] classes = {resource.getClass(), resource.getClass().getSuperclass(), resource.getClass().getSuperclass().getSuperclass()};
    for (Class clz : classes) {
      Field[] fields = clz.getDeclaredFields();
      for (Field projectField : fields) {
        projectField.setAccessible(true);
        try {
          Object fieldValue = projectField.get(resource);
          projectField.set(this.resourceRole, fieldValue);
        } catch (IllegalAccessException e) {
          log.error("Could not convert " + resource.getClass().getName() + " to " + this.resourceRole.getClass().getName(), e);
        }
        projectField.setAccessible(false);
      }
    }
  }

}
