package ch.dlcm.model.oais;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import javax.persistence.CascadeType;
import javax.persistence.CollectionTable;
import javax.persistence.Column;
import javax.persistence.ElementCollection;
import javax.persistence.Entity;
import javax.persistence.EntityListeners;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.PrePersist;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;

import org.springframework.data.jpa.domain.Specification;
import org.springframework.hateoas.mvc.ControllerLinkBuilder;

import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.dlcm.criteria.DisseminationInfoPackageSpecification;
import ch.dlcm.exception.DLCMException;
import ch.dlcm.listener.HistoryListener;
import ch.dlcm.listener.PackageListener;
import ch.dlcm.model.Status.StatusValue;
import ch.dlcm.rest.RESTName;
import ch.dlcm.rest.RESTResource;
import ch.dlcm.rest.RESTSubResourceInterface;

@Entity
@Table(name="dip")
@EntityListeners({HistoryListener.class, PackageListener.class})
public class DisseminationInfoPackage extends RESTResource<DisseminationInfoPackage> implements RESTSubResourceInterface, InfoPackageInterface{
	
	private RepresentationInfo info = new RepresentationInfo();
	
	@JsonIgnore
	@OneToMany(cascade = CascadeType.ALL, orphanRemoval = true)
	@JoinColumn(name="packageId", referencedColumnName="dlcmId")
	private List<DipDataFile> dataFiles = new ArrayList<>();
	
	@ElementCollection
	@Column(name="aipId")
	@CollectionTable(name = "dipAip", joinColumns = { @JoinColumn(name = "packageId") },
		 			 uniqueConstraints= @UniqueConstraint(columnNames={"packageId","aipId"}))
	private List<String> aipIds = new ArrayList<>();

	
	@Override
	public boolean equals(final Object other) {
		if (!(other instanceof DisseminationInfoPackage)) {
			return false;
		}
		DisseminationInfoPackage castOther = (DisseminationInfoPackage) other;
		return Objects.equals(info, castOther.info) && Objects.equals(dataFiles, castOther.dataFiles)
				&& Objects.equals(aipIds, castOther.aipIds);
	}
	@Override
	public int hashCode() {
		return Objects.hash(info, dataFiles, aipIds);
	}

	public RepresentationInfo getInfo() {
		return info;
	}

	public void setInfo(RepresentationInfo info) {
		this.info = info;
	}

	@Override
	public List<DipDataFile> getDataFiles() {
		return dataFiles;
	}

	public void setDataFiles(List<DipDataFile> dataFiles) {
		this.dataFiles = dataFiles;
	}

	public List<String> getAipIds() {
		return aipIds;
	}
	public void setAipIds(List<String> aipIds) {
		this.aipIds = aipIds;
	}

	@Override
	public Specification<DisseminationInfoPackage> getSpecification() {
		return new DisseminationInfoPackageSpecification(this);
	}

	@Override
	@PrePersist
	public void init() {
		info.init();
	}

	@Override
	public boolean update(RESTResource<DisseminationInfoPackage> item) {
		DisseminationInfoPackage t = (DisseminationInfoPackage)item;
		if(this.getInfo().update(t.getInfo())){
			this.updateTime();
			return true;
		}
		return false;
	}

	@Override
	public boolean isAllDataFileReady() {
	  if (this.getDataFiles().isEmpty()) {
      return false;
    }
		for (DipDataFile df : this.getDataFiles()){
			if(df.getDataFile().getStatus().getValue() != StatusValue.Ready){
				return false;
			}
		}
		return true;
	}
	
	@Override
	public <T> boolean addItem(T t) {
		if (t instanceof DipDataFile)
			return this.addDataFile((DipDataFile) t);
		return false;
	}

	private boolean addDataFile(DipDataFile t) {
		updateTime();
		return this.dataFiles.add(t);
	}

	@Override
	public <T> boolean removeItem(T t) {
		if (t instanceof DipDataFile)
			return this.removeDataFile((DipDataFile) t);
		return false;
	}

	private boolean removeDataFile(DipDataFile t) {
		updateTime();
		return this.dataFiles.remove(t);
	}
	@Override
	public String managedBy() {
		return RESTName.ACCESS;
	}
	
	@Override
  public URI getDownloadUri() throws DLCMException {
    if (this.getInfo().getStatus().getValue() == StatusValue.InPreparation) {
      throw new DLCMException("Package not available");
    }
    if (this.getDataFiles().isEmpty()) {
      throw new DLCMException("Package not accessible");
    }
    return this.getDataFiles().get(0).getDataFile().getFinalData();
  }
	
	@Override
  public void addLinks(ControllerLinkBuilder clb, boolean mainRes, boolean subResOnly) {
    super.addLinks(clb, mainRes, subResOnly);
    if (mainRes) {
      this.add(clb.slash(getDlcmId()).slash(RESTName.DATAFILE).withRel(RESTName.DATAFILE));
      if (this.getInfo().getStatus().getValue() != StatusValue.InPreparation) {
        this.add(clb.slash(getDlcmId()).slash(RESTName.DL).withRel(RESTName.DL));
      }
      this.add(clb.slash(getDlcmId()).slash(RESTName.HISTORY).withRel(RESTName.HISTORY));
    }
  }
}
