package ch.dlcm.model.oais;

import java.util.Objects;

import javax.persistence.Embeddable;
import javax.persistence.Embedded;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import ch.dlcm.model.JPAEmbeddableEntityInterface;
import ch.dlcm.model.Status;
import ch.dlcm.model.Status.StatusType;
import ch.dlcm.model.Status.StatusValue;
import ch.dlcm.tool.StringTool;

@Embeddable
public class RepresentationInfo implements JPAEmbeddableEntityInterface<RepresentationInfo> {

	@Embedded
	private Status status = new Status();

	@NotNull
	@Size(min = 1, max = 256)
	private String name;

	@Size(max = 1024)
	private String description;

	public RepresentationInfo() {
		this.status.setType(StatusType.Package);
	}

	@Override
	public boolean equals(final Object other) {
		if (!(other instanceof RepresentationInfo)) {
			return false;
		}
		RepresentationInfo castOther = (RepresentationInfo) other;
		return Objects.equals(status, castOther.status) && Objects.equals(name, castOther.name)
				&& Objects.equals(description, castOther.description);
	}

	@Override
	public int hashCode() {
		return Objects.hash(status, name, description);
	}

	public Status getStatus() {
		return status;
	}

	public void setStatus(Status status) {
		this.status = status;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public void init() {
		if (status.getValue() == null) {
			status.setValue(StatusValue.InPreparation);
		}
	}

	@Override
	public boolean update(RepresentationInfo i) {
		boolean changed = false;
		if (StringTool.toUpdate(this.getName(), i.getName())) {
			this.setName(i.getName());
			changed = true;
		}
		if (StringTool.toUpdate(this.getDescription(), i.getDescription())) {
			this.setDescription(i.getDescription());
			changed = true;
		}
		if (this.getStatus().getValue() == null ||
		    this.getStatus().getValue() == StatusValue.Ready ||
		    this.getStatus().getValue() == StatusValue.Checked ||
		    this.getStatus().getValue() == StatusValue.InError) {
				this.setStatus(i.getStatus());
				changed = true;
		}
		return changed;
	}

}
