package ch.dlcm.rest;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;

import com.fasterxml.jackson.annotation.JsonIgnore;

public class RESTPage {
  private long currentPage;
  private long sizePage;
  private long totalPages;
  private long totalItems;

  public static final int DEFAULT_SIZE_PAGE = 20;

  public <T> RESTPage() {
    this.currentPage = 0;
    this.sizePage = DEFAULT_SIZE_PAGE;
    this.totalPages = 0;
    this.totalItems = 0;
  }

  public <T> RESTPage(long nbItems) {
    this.currentPage = 0;
    this.sizePage = DEFAULT_SIZE_PAGE;
    this.totalPages = this.getPages(nbItems, DEFAULT_SIZE_PAGE);
    this.totalItems = nbItems;
  }

  public <T> RESTPage(Page<T> page, Pageable pageable) {
    this.currentPage = pageable.getPageNumber();
    this.sizePage = pageable.getPageSize();
    this.totalPages = page.getTotalPages();
    this.totalItems = page.getTotalElements();
  }

  public long getSizePage() {
    return sizePage;
  }

  public long getCurrentPage() {
    return currentPage;
  }

  public long getTotalPages() {
    return totalPages;
  }

  public long getTotalItems() {
    return totalItems;
  }

  private long getPages(long nbItems, long pageSize) {
    if (nbItems % pageSize == 0) {
      return nbItems / pageSize;
    } else {
      return nbItems / pageSize + 1;
    }
  }

  @JsonIgnore
  public long getCurrentItems() {
    if (totalItems < sizePage) {
      return totalItems;
    } else {
      if (currentPage + 1 == totalPages) {
        return totalItems - currentPage * sizePage;
      } else {
        return sizePage;
      }
    }
  }

  @JsonIgnore
  public boolean hasPrevious() {
    if (totalItems == 0 || currentPage == 0) {
      return false;
    }
    return true;
  }

  @JsonIgnore
  public boolean hasNext() {
    if (totalItems == 0 || currentPage + 1 == totalPages) {
      return false;
    }
    return true;
  }
}
