package ch.dlcm.rest;

import java.net.URI;
import java.util.List;
import java.util.UUID;

import javax.persistence.Id;
import javax.persistence.MappedSuperclass;

import org.springframework.data.jpa.domain.Specification;
import org.springframework.hateoas.mvc.ControllerLinkBuilder;

import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.dlcm.exception.DLCMException;
import ch.dlcm.model.DataFileInterface;

@MappedSuperclass
public abstract class RESTResource<T> extends RESTResourceBase {

  @Id
  private String dlcmId;

  protected RESTResource() {
    super();
    dlcmId = UUID.randomUUID().toString();
  }

  public String getDlcmId() {
    return dlcmId;
  }
  
  public void setDlcmId(String dlcmId) {
    this.dlcmId = dlcmId;
  }

  public void addLinks(ControllerLinkBuilder clb, boolean mainRes, boolean sub_res_only) {
    this.add(clb.slash(getDlcmId()).withSelfRel());
    if (!sub_res_only)
      this.add((RESTTool.referenceLink(clb.slash(getDlcmId()).toUriComponentsBuilder(), this.managedBy())).withSelfRel());
    this.add(clb.withRel(RESTAction.LIST));
    if (mainRes)
      this.add((RESTTool.parentLink(clb.toUriComponentsBuilder())).withRel(RESTAction.MODULE));
    else
      this.add((RESTTool.parentLink(clb.toUriComponentsBuilder())).withRel(RESTAction.PARENT));
  }

  @JsonIgnore
  public abstract Specification<T> getSpecification();

  public abstract void init();

  public abstract boolean update(RESTResource<T> item);

  public abstract String managedBy();

  @JsonIgnore
  public URI getDownloadUri() throws DLCMException {
    throw new UnsupportedOperationException();
  }

  @JsonIgnore
  public boolean isModifiable() {
    return true;
  }

  @JsonIgnore
  public <V extends DataFileInterface> List<V> getDataFiles() {
    throw new UnsupportedOperationException();
  }

  protected <V extends RESTResource<V>> boolean exist(List<V> list, V item) {
    for (V i : list) {
      if (i.getDlcmId().equals(item.getDlcmId())) {
        return true;
      }
    }
    return false;
  }

  @Override public boolean equals(Object o) {
    if (this == o)
      return true;
    if (!(o instanceof RESTResource))
      return false;
    if (!super.equals(o))
      return false;

    RESTResource<?> that = (RESTResource<?>) o;

    return getDlcmId() != null ? getDlcmId().equals(that.getDlcmId()) : that.getDlcmId() == null;
  }

  @Override public int hashCode() {
    int result = super.hashCode();
    result = 31 * result + (getDlcmId() != null ? getDlcmId().hashCode() : 0);
    return result;
  }
}