package ch.dlcm.rest;

import ch.dlcm.tool.StringTool;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.springframework.hateoas.ResourceSupport;

import javax.persistence.Column;
import javax.persistence.MappedSuperclass;
import javax.validation.constraints.NotNull;
import java.time.OffsetDateTime;
import java.time.ZoneOffset;

@MappedSuperclass
public abstract class RESTResourceBase extends ResourceSupport {

  @NotNull
  @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = StringTool.DATE_FORMAT)
  @Column(length = 3)
  protected OffsetDateTime creationTime;

  @NotNull
  @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = StringTool.DATE_FORMAT)
  @Column(length = 3)
  protected OffsetDateTime updateTime;

  public RESTResourceBase() {
    updateTime = OffsetDateTime.now(ZoneOffset.UTC);
    creationTime = OffsetDateTime.now(ZoneOffset.UTC);
  }

  public void updateTime() {
    updateTime = OffsetDateTime.now(ZoneOffset.UTC);
  }

  public OffsetDateTime getCreationTime() {
    return creationTime;
  }

  public OffsetDateTime getUpdateTime() {
    return updateTime;
  }

  @Override public boolean equals(Object o) {
    if (this == o)
      return true;
    if (!(o instanceof RESTResourceBase))
      return false;
    if (!super.equals(o))
      return false;

    RESTResourceBase that = (RESTResourceBase) o;

    if (getCreationTime() != null ?
            !getCreationTime().equals(that.getCreationTime()) :
            that.getCreationTime() != null)
      return false;
    return getUpdateTime() != null ?
            getUpdateTime().equals(that.getUpdateTime()) :
            that.getUpdateTime() == null;
  }

  @Override public int hashCode() {
    int result = super.hashCode();
    result = 31 * result + (getCreationTime() != null ? getCreationTime().hashCode() : 0);
    result = 31 * result + (getUpdateTime() != null ? getUpdateTime().hashCode() : 0);
    return result;
  }
}
