package ch.dlcm.rest;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import org.apache.log4j.Logger;
import org.springframework.hateoas.ResourceSupport;
import org.springframework.hateoas.mvc.ControllerLinkBuilder;

import com.fasterxml.jackson.databind.ObjectMapper;

public abstract class RESTResourceNoSql<T> extends ResourceSupport {
	private static final Logger log = Logger.getLogger(RESTResourceNoSql.class);
	private String dlcmId;

	public String getDlcmId() {
		return dlcmId;
	}

	public void setDlcmId(String dlcmId) {
		this.dlcmId = dlcmId;
	}

	public void addLinks(ControllerLinkBuilder clb) {
		this.add(clb.slash(getDlcmId()).withSelfRel());
		this.add(clb.withRel(RESTAction.LIST));
		this.add((RESTTool.parentLink(clb.toUriComponentsBuilder())).withRel(RESTAction.MODULE));
	}

	@Override
	public boolean equals(final Object other) {
		if (!(other instanceof RESTResourceNoSql)) {
			return false;
		}
		@SuppressWarnings("unchecked")
		RESTResourceNoSql<T> castOther = (RESTResourceNoSql<T>) other;
		return Objects.equals(dlcmId, castOther.dlcmId);
	}

	@Override
	public int hashCode() {
		return Objects.hash(dlcmId);
	}
	
	@SuppressWarnings("unchecked")
	protected Map<String, Object> convertString2Map(String s){
		try {
			return new ObjectMapper().readValue(s, HashMap.class);
		} catch (IOException e) {
			log.error(e);
		}
		return new HashMap<>();
	}
	
	public abstract boolean update(RESTResourceNoSql<T> item);
}