package ch.dlcm.tool;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;

import ch.dlcm.rest.RESTPage;

public class CollectionTool {
  private static final Logger logger = Logger.getLogger(CollectionTool.class);
  private static final ObjectMapper mapper = new ObjectMapper();

  public static <E> List<E> toList(Iterable<E> iterable) {
    if (iterable instanceof List) {
      return (List<E>) iterable;
    }
    ArrayList<E> list = new ArrayList<>();
    if (iterable != null) {
      for (E e : iterable) {
        list.add(e);
      }
    }
    return list;
  }

  public static <T> List<T> getList(String jsonString, Class<T> itemClass) {
    List<T> result = new ArrayList<>();
    try {
      JsonNode jsonPostResult = mapper.readTree(jsonString);
      String resourceArrayString = jsonPostResult.get("_data").toString();
      JavaType listItemType = mapper.getTypeFactory().constructParametricType(List.class,
              itemClass);
      result = mapper.readValue(resourceArrayString, listItemType);
    } catch (IOException e) {
      logger.error("Error in building list", e);
    }
    return result;
  }

  public static RESTPage getPage(String jsonString) {
    RESTPage result=null;
    try {
      JsonNode jsonPostResult = mapper.readTree(jsonString);
      String pageString = jsonPostResult.get("_page").toString();
      result = mapper.readValue(pageString, RESTPage.class);
    } catch (IOException e) {
      logger.error("Error in building list", e);
    }
    return result;
  }

}
