package ch.dlcm.tool;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Arrays;

import org.apache.log4j.Logger;
import org.springframework.boot.web.client.RestTemplateBuilder;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.http.client.HttpComponentsAsyncClientHttpRequestFactory;
import org.springframework.http.client.SimpleClientHttpRequestFactory;
import org.springframework.web.client.RequestCallback;
import org.springframework.web.client.ResponseExtractor;
import org.springframework.web.client.RestTemplate;

public class RestClientTool {
  private static final Logger log = Logger.getLogger(CollectionTool.class);

  public static RestTemplate getClient(String url) {
      return getRestTemplateBuilder(url).build();
  }

  public static <T> T downloadContent(URI uri, ResponseExtractor<T> responseExtractor){
    SimpleClientHttpRequestFactory requestFactory = new SimpleClientHttpRequestFactory();
    requestFactory.setBufferRequestBody(false);
    RestTemplate restTemplate = getRestTemplateBuilder(uri.toString()).requestFactory(requestFactory).build();

    // Optional Accept header
    RequestCallback requestCallback = request -> request.getHeaders().setAccept(
            Arrays.asList(MediaType.APPLICATION_OCTET_STREAM, MediaType.ALL));

    return restTemplate.execute(uri, HttpMethod.GET, requestCallback, responseExtractor);
  }

  private static RestTemplateBuilder getRestTemplateBuilder(String url){
    RestTemplateBuilder restTemplateBuilder = new RestTemplateBuilder();
    if (isWithBasicAuthenfication(url)) {
      try {
        URI uri = new URI(url);
        String userInfo = uri.getUserInfo();
        String user = userInfo.substring(0, userInfo.indexOf(":"));
        String password = userInfo.substring(userInfo.indexOf(":") + 1);
        restTemplateBuilder.basicAuthorization(user, password);
      } catch (URISyntaxException e) {
        log.error(url, e);
      }
    }
    // Use Apache HttpComponents in order to have detailed http request/response logging
    restTemplateBuilder.requestFactory(new HttpComponentsAsyncClientHttpRequestFactory());
    return restTemplateBuilder;
  }

  private static boolean isWithBasicAuthenfication(String url) {
    return url.contains("@");
  }
}
