package ch.dlcm.controller.preingest;

import static org.springframework.hateoas.mvc.ControllerLinkBuilder.linkTo;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.methodOn;

import java.net.URI;
import java.util.List;

import org.springframework.core.io.InputStreamResource;
import org.springframework.hateoas.ResourceSupport;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import ch.dlcm.controller.ResourceController;
import ch.dlcm.exception.DLCMException;
import ch.dlcm.model.Access;
import ch.dlcm.model.Deposit;
import ch.dlcm.model.DepositDataFile;
import ch.dlcm.model.Status;
import ch.dlcm.model.Status.StatusType;
import ch.dlcm.model.Status.StatusValue;
import ch.dlcm.rest.RESTAction;
import ch.dlcm.rest.RESTName;
import ch.dlcm.tool.ZipTool;

@RestController
@RequestMapping("/" + RESTName.PREINGEST + "/" + RESTName.DEPOSIT)
public class DepositController extends ResourceController<Deposit> {

	@RequestMapping(value = "/{id}/" + RESTName.SIP, method = RequestMethod.GET)
	public HttpEntity<InputStreamResource> getSIP(@PathVariable String id) throws DLCMException{
		Deposit item = itemRepository.findOne(id);
		if (item == null) {
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}
		if (item.getStatus().getValue() != StatusValue.Completed) {
			throw new DLCMException(messageService.get("preingest.deposit.notready", new Object[] { id }));
		}
		return download(item.getClass().getName(), id, item.getSipUri(), id+".deposit");
	}

	@RequestMapping(value = "/{id}/" + RESTName.DL, method = RequestMethod.GET)
	public HttpEntity<InputStreamResource> getFiles(@PathVariable String id){
		Deposit item = itemRepository.findOne(id);
		if (item == null) {
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}
		ZipTool zipPackage = new ZipTool(System.getProperty("java.io.tmpdir") + "/" + item.getDlcmId() + ".zip");
		zipPackage.zipFiles(this.dlcmConfig.getPreingestPreparationLocation() + "/" + item.getDlcmId());
		return this.download(item.getClass().getName(),id,URI.create(zipPackage.getArchive()), null);
	}

	@RequestMapping(value = "/{id}/" + RESTName.UL, method = RequestMethod.POST)
	public HttpEntity<List<DepositDataFile>> sendFiles(@PathVariable String id,
			@RequestParam("file") List<MultipartFile> files,
			@RequestParam(value = "category", required = false) List<String> dataCategories,
			@RequestParam(value = "type", required = false) List<String> dataTypes,
			@RequestParam(value = "folder", required = false) List<String> relLocations) throws DLCMException{
		DepositDataFile df = new DepositDataFile();
		return upload(id, df, files, dataCategories, dataTypes, relLocations);
	}


	@RequestMapping(value = "/" + RESTName.STATUS, method = RequestMethod.GET)
	public HttpEntity<StatusValue[]> listStatus() {
		return new ResponseEntity<>(Status.listStatusValues(StatusType.Deposit), HttpStatus.OK);
	}

	@RequestMapping(value = "/" + RESTName.ACCESS_TYPES, method = RequestMethod.GET)
	public HttpEntity<Access[]> listAccessTypes() {
		return new ResponseEntity<>(Access.values(), HttpStatus.OK);
	}

	@Override
	protected <W extends ResourceSupport> boolean addOthersLinks(W w) {
		w.add(linkTo(methodOn(this.getClass()).listStatus()).withRel(RESTAction.VALUES));
		w.add(linkTo(methodOn(this.getClass()).listAccessTypes()).withRel(RESTAction.VALUES));
		return true;
	}
}
