package ch.dlcm.preparation.zip;

import java.io.File;
import java.util.List;

import javax.annotation.PostConstruct;

import org.springframework.context.annotation.Profile;
import org.springframework.stereotype.Service;

import ch.dlcm.model.Status.StatusValue;
import ch.dlcm.preparation.PreparationService;
import ch.dlcm.service.DLCMService;
import ch.dlcm.tool.FileTool;
import ch.dlcm.tool.ZipTool;

@Profile("pre-zip")
@Service
public class ZipPreparationService extends DLCMService implements PreparationService {

	private String preparationPath;

	@PostConstruct
	@Override
	protected void loadConfig(){
		this.preparationPath = this.dlcmConfig.getPreingestPreparationLocation();
	}

	@Override
	public String submitPackage(String id, String name) {
		String source = preparationPath + "/" + id;
		ZipTool zipTool = new ZipTool(source+".zip");
		if (zipTool.zipFiles(source)){
			return id;
		}
		return null;
	}

	@Override
	public StatusValue getStatus(String id) {
		File f = new File(preparationPath + "/" + id + ".zip");
		if (f.isFile() && f.exists())
			return StatusValue.Completed;
		return StatusValue.InProgress;
	}


	@Override
	public String getPackageUri(String id) {
		return FileTool.getFileUri(preparationPath + "/" + id + ".zip");
	}

	@Override
	public String sendPackage(String id, String name) {
		return this.sendSIP(id, name);
	}

	@Override
	public List<?> listPackages() {
		throw new UnsupportedOperationException();
	}

}
