package ch.dlcm.service;

import javax.annotation.PostConstruct;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jms.annotation.JmsListener;
import org.springframework.stereotype.Component;

import ch.dlcm.message.DepositMessage;
import ch.dlcm.model.Deposit;
import ch.dlcm.model.Status.StatusValue;
import ch.dlcm.preparation.PreparationService;
import ch.dlcm.repository.DepositRepository;

@Component
public class DepositMgmt extends DLCMService {
	private static final Logger log = Logger.getLogger(DepositMgmt.class);

	@Autowired
	DepositRepository depositRepo;

	@Autowired
	PreparationService depositPackageBuilder;

	@PostConstruct
	@Override
	protected void loadConfig() {
	}

	@JmsListener(destination = "${dlcm.queue.deposit}")
	public void processDeposit(DepositMessage depositPayload) {
		log.info("Reading message " + depositPayload.toString());
		Deposit deposit = this.getOne(depositRepo, depositPayload.getDlcmId());
		if (deposit == null){
			log.error("Cannot find deposit (" + depositPayload.getDlcmId() +")");
			return;
		}
		while (deposit.getStatus().getValue() != StatusValue.InError
				&& deposit.getStatus().getValue() != StatusValue.Paused
				&& deposit.getStatus().getValue() != StatusValue.Completed) {
			try{
				log.info("Deposit: " + deposit.getTitle() + " (" + deposit.getDlcmId() + "): status=" + deposit.getStatus().getValue().toString());
				switch (deposit.getStatus().getValue()) {
				case Approved:
					if (deposit.getPreparationId() == null){
						String preparationId = depositPackageBuilder.submitPackage(deposit.getDlcmId(), deposit.getTitle());
						if (preparationId != null) {
							deposit.setPreparationId(preparationId);
							deposit.setDepositStatus(StatusValue.Submitted);
						} else {
							deposit.setErrorStatus(messageService.get("preingest.deposit.error.submission"));
						}
					}else{
						deposit.setDepositStatus(StatusValue.Submitted);
					}
					break;
				case Submitted:
					String sipId;
					if (!deposit.isAllDataFileReady()){
						deposit.setErrorStatus(messageService.get("preingest.deposit.datafile.notready"));
					}else{
						switch(depositPackageBuilder.getStatus(deposit.getPreparationId())){
						case Completed:
								deposit.setSip(depositPackageBuilder.getPackageUri(deposit.getDlcmId()));
								if ((sipId = depositPackageBuilder.sendPackage(deposit.getDlcmId(), deposit.getTitle())) != null) {
									deposit.setSipId(sipId);
									deposit.setDepositStatus(StatusValue.Completed);
								} else {
									deposit.setErrorStatus(messageService.get("preingest.deposit.error.sending"));
								}
								break;
						case InError:
							deposit.setErrorStatus(messageService.get("preingest.deposit.error.preparation"));
							break;
						case Paused:
							deposit.getStatus().setValue(StatusValue.Paused);
							deposit.getStatus().setInfo(messageService.get("preingest.deposit.preparation.useraction"));
							break;
						default:
							break;
						}
					}
					break;
				default:
					break;
				}
			}catch(Exception e){
			  log.error("Error in processing deposit (" + depositPayload.getDlcmId() + ")", e);
				deposit.setErrorStatus(e.getMessage());
			}
			deposit = depositRepo.save(deposit);
		}
		log.info("Deposit processed: dlcmId="+deposit.getDlcmId()+" status="+deposit.getStatus().getValue().toString());
	}
}
