package ch.dlcm.test.doc.matcher.accession;

import static ch.dlcm.test.executor.DLCMTestDocHelper.endsWithList;
import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.Matchers.hasItems;
import static org.hamcrest.Matchers.hasKey;
import static org.hamcrest.Matchers.hasSize;
import static org.hamcrest.Matchers.hasValue;
import static org.hamcrest.core.AnyOf.anyOf;
import static org.hamcrest.core.IsNull.notNullValue;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import ch.dlcm.rest.RESTAction;
import org.hamcrest.Matcher;

import ch.dlcm.rest.RESTName;
import ch.dlcm.test.executor.DLCMTestDocHelper;

public class AccessDocMatcher {

    private AccessDocMatcher() {
        throw new IllegalAccessError("Utility class");
    }

    public static Map<String, Matcher> getAccessExpectedMatchers() {
        Map<String, Matcher> expectedMap = new HashMap<>();
        expectedMap.put("name", is(notNullValue()));
        expectedMap.put("name", is("access"));
        // _links
        final int SIZE_LINKS = DLCMTestDocHelper.getAccessLinksActions().size();
        expectedMap.put("$._links.*", hasSize(SIZE_LINKS));
        for (String endPoint : DLCMTestDocHelper.getAccessLinksActions()) {
            expectedMap.put("$._links", hasKey(endPoint));
        }
        // _links.resources
        final int SIZE_RESSOURCES = DLCMTestDocHelper.getAccessRessourcesLinksActions().size();
        expectedMap.put("$._links.resources", hasSize(SIZE_RESSOURCES));
        for (int i = 0; i < SIZE_RESSOURCES; i++) {
            expectedMap.put("$._links.resources["+ i +"]", hasKey("href"));
            expectedMap.put("$._links.resources["+ i +"].href", anyOf(endsWithList(DLCMTestDocHelper.getAccessRessourcesLinksActions())));
        }
        return expectedMap;
    }

    public static Map<String, String> getAccessMapLinksDescription() {
        Map<String, String> links = new HashMap<>();
        DLCMTestDocHelper.getAccessLinksActions().forEach(
                action -> links.put(action,"<<" + action + ", Links>> to " + action +" resource")
        );
        return links;
    }

    public static Map<String, Matcher> getAccessOrdersExpectedMatchers() {
        return getExpectedMatchers(DLCMTestDocHelper.getAccessOrderActions(), DLCMTestDocHelper.getAccessOrdersLinksActions());
    }

    public static Map<String, Matcher> getAccessDipExpectedMatchers() {
        Map<String, Matcher> expectedMap = new HashMap<>();
        expectedMap.put("$.*", hasSize(3));
        expectedMap.put("$._data", hasSize(0));
        expectedMap.put("$._page.*", hasSize(4));
        expectedMap.put("$._links.*", hasSize(DLCMTestDocHelper.getAccessDipActions().size()));
        for (String endPoint: DLCMTestDocHelper.getAccessDipActions()) {
            expectedMap.put("$._links", hasKey(endPoint));
            expectedMap.put("$._links." + endPoint, hasKey("href"));
            expectedMap.put("$._links." + endPoint, hasValue(anyOf(endsWithList(DLCMTestDocHelper.getAccessDipLinksActions()))));
        }
        return expectedMap;
    }

    // TODO should be re-used in other matcher?
    private static Map<String, Matcher> getExpectedMatchers(List<String> actions, List linkActions) {
        Map<String, Matcher> expectedMap = new HashMap<>();
        expectedMap.put("$.*", hasSize(3));
        expectedMap.put("$._data", hasSize(0));
        expectedMap.put("$._page.*", hasSize(4));
        expectedMap.put("$._links.*", hasSize(actions.size()));
        for (String endPoint: actions) {
            expectedMap.put("$._links", hasKey(endPoint));
            if (RESTAction.VALUES.equals(endPoint)){
                expectedMap.put("$._links." + endPoint + ".*", hasSize(2));
                expectedMap.put("$._links." + endPoint + "[0]", hasValue(anyOf(endsWithList(linkActions))));
                expectedMap.put("$._links." + endPoint + "[1]", hasValue(anyOf(endsWithList(linkActions))));
            }else {
                expectedMap.put("$._links." + endPoint, hasKey("href"));
            }
        }
        return expectedMap;
    }

    public static Map<String, String> getAccessDipMapLinksDescription() {
        return getMapLinksDescription(DLCMTestDocHelper.getAccessDipActions(), null, RESTName.DIP);
    }

    /**
     * Shared with access Dip and Orders
     * @param radicalPhrase possibility to add more fields when needed
     */
    private static Map<String, String> getMapLinksDescription(List<String> actions, String... radicalPhrase) {
        List<String> safeRadicalPhrase = Arrays.asList(radicalPhrase);
        Map<String, String> fields = new HashMap<>();
        fields.put("data","The " + (safeRadicalPhrase.get(0) == null ? RESTName.DATAFILE: safeRadicalPhrase.get(0)) +" content");
        fields.put("_links", "The " + (safeRadicalPhrase.get(1) == null ? RESTName.DIP: safeRadicalPhrase.get(1))  + " links");
        Map<String, String> links = new HashMap<>();
        actions.forEach(
                action -> links.put(action,"<<" + action + ", Links>> to " + action +" resource")
        );
        return links;
    }

    public static Map<String, String> getAccessOrdersMapLinksDescription() {
        return getMapLinksDescription(DLCMTestDocHelper.getAccessOrderActions(), null, RESTName.ORDER);
    }
    
    public static Map<String, String> getAccessAipMapLinksDescription() {
      return getMapLinksDescription(DLCMTestDocHelper.getAccessAipActions(), null, RESTName.AIP);
    }

    public static Map<String, Matcher> getAccessDipStatusExpectedMatchers() {
        Map<String, Matcher> expectedMap = new HashMap<>();
        expectedMap.put("$.*", hasSize(DLCMTestDocHelper.getAccessDipStatus().size()));
        expectedMap.put("$", hasItems(DLCMTestDocHelper.getAccessDipStatus().toArray()));
        return expectedMap;
    }

    public static Map<String, String> getAccessDipStatusMapFieldDescription() {
        Map<String, String> fields = new HashMap<>();
        fields.put("[]", DLCMTestDocHelper.getAccessDipStatus().toString());
        return fields;
    }

    public static Map<String, Matcher> getAccessAipExpectedMatchers() {
      Map<String, Matcher> expectedMap = new HashMap<>();
      expectedMap.put("$.*", hasSize(3));
      expectedMap.put("$._data", hasSize(0));
      expectedMap.put("$._page.*", hasSize(4));
      expectedMap.put("$._links.*", hasSize(DLCMTestDocHelper.getAccessAipActions().size()));
      for (String endPoint: DLCMTestDocHelper.getAccessAipActions()) {
          expectedMap.put("$._links", hasKey(endPoint));
          expectedMap.put("$._links." + endPoint, hasKey("href"));
          expectedMap.put("$._links." + endPoint, hasValue(anyOf(endsWithList(DLCMTestDocHelper.getAccessAipLinksActions()))));
      }
      return expectedMap;
    }

}
