package ch.dlcm.test.doc.matcher.ingestion;

import static ch.dlcm.test.executor.DLCMTestDocHelper.endsWithList;
import static org.hamcrest.CoreMatchers.anyOf;
import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.CoreMatchers.notNullValue;
import static org.hamcrest.Matchers.hasSize;

import java.util.HashMap;
import java.util.Map;

import ch.dlcm.test.doc.matcher.DLCMDocumentation;
import ch.dlcm.test.executor.DLCMTestDocHelper;
import org.hamcrest.Matcher;

import ch.dlcm.rest.RESTAction;
import ch.dlcm.rest.RESTName;

public final class AdminDocMatcher implements DLCMDocumentation {


    @Override
    public Map<String, Matcher> getExpectedMatchers() {
        return getAdminExpectedMatchers();
    }

    @Override
    public Map<String, String> getMapFieldsDescription() {
        return getAdminMapFieldsDescription();
    }

    @Override
    public Map<String, String> getMapLinksDescription() {
        return getAdminMapLinksDescription();
    }

    public static Map<String, Matcher> getAdminExpectedMatchers(){
        Map<String, Matcher> expectedMap = new HashMap<>();
        final Integer ADMIN_SIZE_RESSOURCES = DLCMTestDocHelper.getAdminRessourcesActions().size();
        expectedMap.put("name", is(notNullValue()));
        expectedMap.put("name", is(RESTName.ADMIN));
        expectedMap.put("$._links.*", hasSize(DLCMTestDocHelper.getAdminRootActions().size()));
        expectedMap.put("$._links.self", is(notNullValue()));
        expectedMap.put("$._links.resources", hasSize(ADMIN_SIZE_RESSOURCES));
        for (int i = 0; i < ADMIN_SIZE_RESSOURCES; i++) {
            expectedMap.put("$._links.resources["+ i +"].href", anyOf(endsWithList(DLCMTestDocHelper.getAdminRessourcesActions())));
        }
        return expectedMap;
    }

    public static Map<String, String> getAdminMapFieldsDescription(){
        Map<String, String> fields = new HashMap<>();
        fields.put("name","The " + RESTName.ADMIN +" name");
        fields.put("_links", "The " + RESTName.ADMIN  + " links");
        fields.put("_links.resources", "The " + RESTName.ADMIN + " resource");
        return fields;
    }

    public static Map<String, String> getAdminMapLinksDescription(){
        Map<String, String> links = new HashMap<>();
        links.put(RESTAction.SELF,"Link to current " + RESTName.ADMIN +" resource");
        links.put(RESTAction.RESOURCES,"<<pre-ingest-resources, Links>> to " + RESTName.ADMIN  + " resources List");
        return links;
    }


}
