package ch.dlcm.test.doc.matcher.ingestion;

import static ch.dlcm.test.executor.DLCMTestDocHelper.endsWithList;
import static org.hamcrest.CoreMatchers.anyOf;
import static org.hamcrest.CoreMatchers.notNullValue;
import static org.hamcrest.Matchers.hasKey;
import static org.hamcrest.Matchers.hasSize;
import static org.hamcrest.collection.IsMapContaining.hasValue;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.hamcrest.Matcher;

import ch.dlcm.rest.RESTAction;
import ch.dlcm.rest.RESTName;
import ch.dlcm.test.doc.matcher.DLCMDocumentation;
import ch.dlcm.test.executor.DLCMTestDocHelper;

public class AdminInstitutionDocMatcher implements DLCMDocumentation {

  @Override
  public Map<String, Matcher> getExpectedMatchers() {
    return getAdminInstitutionExpectedMatchers();
  }

  @Override
  public Map<String, String> getMapFieldsDescription() {
    return getAdminInstitutionMapFieldsDescription();
  }

  @Override
  public Map<String, String> getMapLinksDescription() {
    return getAdminInstitutionMapLinksDescription();
  }

  public static Map<String, Matcher> getAdminInstitutionExpectedMatchers() {
    Map<String, Matcher> expectedMap = new HashMap<>();
    expectedMap.put("$.*", hasSize(3));
    expectedMap.put("$._data", hasSize(4));
    expectedMap.put("$._page.*", hasSize(4));
    expectedMap.put("$._links.*", hasSize(DLCMTestDocHelper.getAdminInstitutionLinksActions().size()));
    for (String endPoint : DLCMTestDocHelper.getAdminInstitutionsRootActions()) {
      expectedMap.put("$._links", hasKey(endPoint));
      expectedMap.put("$._links." + endPoint, hasKey("href"));
      expectedMap.put("$._links." + endPoint, hasValue(anyOf(endsWithList(DLCMTestDocHelper.getAdminInstitutionLinksActions()))));
    }
    return expectedMap;
  }

  public static Map<String, Matcher> getAdminInstitutionGetORPostExpectedMatchers() {
    Map<String, Matcher> expectedMap = new HashMap<>();
    List<String> postLinks = new ArrayList<>(DLCMTestDocHelper.getAdminInstitutionsPostActions());
    for (String field : postLinks) {
      expectedMap.put("$", hasKey(field));
      expectedMap.put("$." + field, notNullValue());
    }
    expectedMap.put("$.*", hasSize(postLinks.size()));

    return expectedMap;
  }

  public static Map<String, String> getAdminInstitutionMapFieldsDescription() {
    Map<String, String> fields = new HashMap<>();
    fields.put("_data", "The " + RESTName.INSTITUTION + " content");
    fields.put("_page", "The " + RESTName.INSTITUTION + " page metadata information");
    fields.put("_links", "The " + RESTName.INSTITUTION + " links");
    return fields;
  }

  public static Map<String, String> getAdminInstitutionPostMapFieldsDescription() {
    Map<String, String> fields = new HashMap<>();
    fields.put(DLCMTestDocHelper.getAdminInstitutionsPostActions().get(0), "The " + RESTName.INSTITUTION + " id");
    fields.put(DLCMTestDocHelper.getAdminInstitutionsPostActions().get(1), "The " + RESTName.INSTITUTION + " name");
    fields.put(DLCMTestDocHelper.getAdminInstitutionsPostActions().get(2), "The " + RESTName.INSTITUTION + " description");
    fields.put(DLCMTestDocHelper.getAdminInstitutionsPostActions().get(3), "The " + RESTName.INSTITUTION + " updated time");
    fields.put(DLCMTestDocHelper.getAdminInstitutionsPostActions().get(4), "The " + RESTName.INSTITUTION + " created time");
    fields.put(DLCMTestDocHelper.getAdminInstitutionsPostActions().get(5), "The " + RESTName.INSTITUTION + " links");
    return fields;
  }

  public static Map<String, String> getAdminInstitutionMapLinksDescription() {
    Map<String, String> links = new HashMap<>();
    links.put(RESTAction.SELF, "Link to current " + RESTName.INSTITUTION + " resource");
    links.put(RESTAction.MODULE, "Link to the parent: " + RESTAction.MODULE);
    links.put(RESTAction.LASTCREATED, "The date of the last " + RESTName.INSTITUTION + " create");
    links.put(RESTAction.LASTUPDATE, "The date of the last " + RESTName.INSTITUTION + " update");
    return links;
  }

  public static Map<String, String> getAdminInstitutionsPostMapLinksDescription() {
    Map<String, String> links = new HashMap<>();
    links.put(RESTAction.SELF, "Link to an existent " + RESTName.INSTITUTION + " id");
    links.put(RESTAction.MODULE, "Link to the parent: " + RESTAction.MODULE);
    links.put(RESTAction.LIST, "Link to the list of " + RESTName.INSTITUTION);
    links.put(RESTName.PERSON, "Link to the " + RESTName.INSTITUTION + " " + RESTName.PERSON);
    links.put(RESTName.UNIT, "Link to the " + RESTName.UNIT);
    return links;
  }

  // TODO perform delete structure for test institution deeply

}
