package ch.dlcm.test.doc.matcher.ingestion;

import ch.dlcm.model.Status;
import ch.dlcm.test.doc.matcher.DLCMDocumentation;
import ch.dlcm.test.executor.DLCMTestDocHelper;
import org.hamcrest.Matcher;

import java.util.HashMap;
import java.util.Map;

import static ch.dlcm.rest.RESTResult.*;
import static org.hamcrest.CoreMatchers.anyOf;
import static org.hamcrest.CoreMatchers.notNullValue;
import static org.hamcrest.Matchers.*;
import static org.hamcrest.collection.IsMapContaining.hasValue;

public class IngestDocMatcher implements DLCMDocumentation{


    @Override
    public Map<String, Matcher> getExpectedMatchers() {
        return getIngestSipResumeExpectedMatchers(null);
    }

    @Override
    public Map<String, String> getMapFieldsDescription() {
        return null;
    }

    @Override
    public Map<String, String> getMapLinksDescription() {
        return null;
    }


    public static Map<String, Matcher> getIngestSipResumeExpectedMatchers(String sipId) {
        Map<String, Matcher> expectedMap = new HashMap<>();
        final int SIZE_FIELDS = DLCMTestDocHelper.getIngestSipResumetActions().size();
        expectedMap.put("$.*", hasSize(SIZE_FIELDS));
        for (String field : DLCMTestDocHelper.getIngestSipResumetActions()) {
            if ("_links".equals(field)){
                expectedMap.put("$._links.*", hasSize(2));
                expectedMap.put("$._links.self", hasValue(anyOf(endsWith(sipId + "/resume"))));
                expectedMap.put("$._links.parent", hasValue(notNullValue()));
            }else if ("status".equals(field)) {
                expectedMap.put("$", is(ActionStatus.Executed));
            }else{
                expectedMap.put("$", hasKey(field));
                expectedMap.put("$." + field, notNullValue());
            }
        }
        return expectedMap;
    }

    public static Map<String, Matcher> getIngestSipExpectedMatchers() {
        Map<String, Matcher> expectedMap = new HashMap<>();
        final int SIZE_SIP = DLCMTestDocHelper.getIngestSipActions().size();
        expectedMap.put("$.*", hasSize(SIZE_SIP));
        for (String field : DLCMTestDocHelper.getIngestSipActions()) {
            if ("depositId".equals(field) || "aipId".equals(field)){
                expectedMap.put("$." + field, nullValue());
            }else{
                expectedMap.put("$." + field, notNullValue());
            }
        }

        final int SIZE_INFO = DLCMTestDocHelper.getIngestSipInfoActions().size();
        expectedMap.put("$.info.*", hasSize(SIZE_INFO));
        for (String field : DLCMTestDocHelper.getIngestSipInfoActions()) {
            expectedMap.put("$.info", hasKey(field));
            if ("status".equals(field)){
                expectedMap.put("$.info.status.value", is(Status.StatusValue.InPreparation.name()));
                expectedMap.put("$.info.status.info", nullValue());
            }
        }
        return expectedMap;

    }

}
