package ch.dlcm.test.doc.matcher.ingestion;

import ch.dlcm.rest.RESTAction;
import ch.dlcm.rest.RESTName;
import ch.dlcm.test.doc.matcher.DLCMDocumentation;
import ch.dlcm.test.executor.DLCMTestDocHelper;
import org.hamcrest.Matcher;

import java.util.HashMap;
import java.util.Map;

import static org.hamcrest.Matchers.*;

public final class PreIngestDocMatcher implements DLCMDocumentation {


    @Override
    public Map<String, Matcher> getExpectedMatchers() {
        return getPreIngestExpectedMatchers();
    }

    @Override
    public Map<String, String> getMapFieldsDescription() {
        return getPreIngestMapFieldsDescription();
    }

    @Override
    public Map<String, String> getMapLinksDescription() {
        return getPreIngestMapLinksDescription();
    }

    public static Map<String, Matcher> getPreIngestExpectedMatchers(){
        Map<String, Matcher> expectedMap = new HashMap<>();
        expectedMap.put("name", is(notNullValue()));
        expectedMap.put("name", is(RESTName.PREINGEST));
        expectedMap.put("$._links.*", hasSize(2));
        expectedMap.put("$._links.self", is(notNullValue()));
        expectedMap.put("$._links.resources.href", endsWith(RESTName.DEPOSIT));
        return expectedMap;
    }

    public static Map<String, String> getPreIngestMapFieldsDescription(){
        Map<String, String> fields = new HashMap<>();
        fields.put("name","The " + RESTName.PREINGEST +" name");
        fields.put("_links", "The " + RESTName.PREINGEST  + " links");
        fields.put("_links.resources", "The " + RESTName.PREINGEST  + " resource");
        return fields;
    }

    public static Map<String, String> getPreIngestMapLinksDescription(){
        Map<String, String> links = new HashMap<>();
        links.put(RESTAction.SELF,"Link to current " + RESTName.PREINGEST +" resource");
        links.put(RESTAction.RESOURCES,"<<pre-ingest-resources, Links>> to " + RESTName.PREINGEST  + " resources List");
        return links;
    }


    public static Map<String, Matcher> getPreIngestDepositsPeopleExpectedMatchers(String parentLink) {
        Map<String, Matcher> expectedMap = new HashMap<>();
        expectedMap.put("$._data", is(notNullValue()));
        expectedMap.put("$._data", hasSize(1));
        expectedMap.put("$._links.*", hasSize(2));
        expectedMap.put("$._links.self", hasValue(endsWith(RESTName.PERSON)));
        expectedMap.put("$._links.parent", hasValue(endsWith(parentLink)));
        return expectedMap;
    }

    public static Map<String, Matcher> getPreIngestAdminPeopleIDExpectedMatchers(String peopleId) {
        Map<String, Matcher> expectedMap = new HashMap<>();

        final int SIZE_PEOPLE = DLCMTestDocHelper.getAdminPeopleActions().size();
        expectedMap.put("$.*", hasSize(SIZE_PEOPLE));
        for (String endPoint : DLCMTestDocHelper.getAdminPeopleActions()) {
            expectedMap.put("$", hasKey(endPoint));
                if(!"orcid".equals(endPoint)){
                    expectedMap.put("$." + endPoint, notNullValue());
                }
        }

        final int SIZE_LINKS = DLCMTestDocHelper.getAdminPeopleLinksActions().size();
        expectedMap.put("$._links.*", hasSize(SIZE_LINKS));
        for (String endPoint : DLCMTestDocHelper.getAdminPeopleLinksActions()) {
            expectedMap.put("$._links", hasKey(endPoint));
            if (RESTAction.SELF.equals(endPoint)){
                expectedMap.put("$._links." + endPoint, hasSize(2));
                expectedMap.put("$._links." + endPoint, hasKey("href"));
                expectedMap.put("$._links." + endPoint, hasItem(hasValue(endsWith((peopleId)))));
            }else {
                expectedMap.put("$._links." + endPoint, hasKey("href"));
                expectedMap.put("$._links." + endPoint, notNullValue());
            }
        }
        return expectedMap;
    }

}
