package ch.dlcm.test.doc.matcher.storagion;

import static ch.dlcm.test.executor.DLCMTestDocHelper.endsWithList;
import static org.hamcrest.CoreMatchers.anyOf;
import static org.hamcrest.Matchers.hasItems;
import static org.hamcrest.Matchers.hasKey;
import static org.hamcrest.Matchers.hasSize;
import static org.hamcrest.Matchers.is;
import static org.hamcrest.Matchers.notNullValue;
import static org.hamcrest.collection.IsMapContaining.hasValue;

import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Map;

import org.hamcrest.Matcher;

import ch.dlcm.rest.RESTName;
import ch.dlcm.test.doc.matcher.DLCMDocumentation;
import ch.dlcm.test.executor.DLCMTestDocHelper;
import org.springframework.test.web.servlet.ResultActions;

public class StoragionDocMatcher implements DLCMDocumentation {


    @Override
    public Map<String, Matcher> getExpectedMatchers() {
        return getStoragionRootExpectedMatchers();
    }

    @Override
    public Map<String, String> getMapFieldsDescription() {
        return getStoragionRootMapLinksDescription();
    }

    @Override
    public Map<String, String> getMapLinksDescription() {
        return null;
    }

    public static Map<String, Matcher> getStoragionRootExpectedMatchers() {
        Map<String, Matcher> expectedMap = new HashMap<>();
        expectedMap.put("name", is(notNullValue()));
        expectedMap.put("name", is("DLCM Backend - Storagion"));
        final int SIZE_LINKS = DLCMTestDocHelper.getStoragionRootLinksActions().size();
        expectedMap.put("$._links.*", hasSize(SIZE_LINKS));
        for (String endPoint : DLCMTestDocHelper.getStoragionRootLinksActions()) {
            expectedMap.put("$._links", hasKey(endPoint));
            expectedMap.put("$._links." + endPoint, hasKey("href"));
            expectedMap.put("$._links." + endPoint, hasValue(anyOf(endsWithList(DLCMTestDocHelper.getStoragionRootLinksActions()))));
        }
        return expectedMap;
    }

    public static Map<String, String> getStoragionRootMapLinksDescription() {
        Map<String, String> links = new HashMap<>();
        DLCMTestDocHelper.getStoragionRootLinksActions().forEach(
                action -> links.put(action, "<<" + action + ", Links>> to " + action + " resource")
        );
        return links;
    }

    public static Map<String, Matcher> getDataMgmtDIExpectedMatchers(ResultActions request) throws UnsupportedEncodingException {
        Map<String, Matcher> expectedMap = new HashMap<>();
        final int SIZE_LINKS = DLCMTestDocHelper.getDataMgmtDIActions(request).size();
        expectedMap.put("$._links.*", hasSize(SIZE_LINKS));
        for (String endPoint : DLCMTestDocHelper.getDataMgmtDIActions(request)) {
            expectedMap.put("$._links", hasKey(endPoint));
            expectedMap.put("$._links." + endPoint, hasKey("href"));
            expectedMap.put("$._links." + endPoint, hasValue(anyOf(endsWithList(DLCMTestDocHelper.getDataMgmtDILinksActions()))));
        }
        return expectedMap;
    }

    public static Map<String, Matcher> getArchivalStorageExpectedMatchers() {
        Map<String, Matcher> expectedMap = new HashMap<>();
        expectedMap.put("$.name", is(RESTName.ARCHIVALSTORAGE));
        final int SIZE_LINKS = DLCMTestDocHelper.getArchivalStorageLinksActions().size();
        expectedMap.put("$._links.*", hasSize(SIZE_LINKS));
        for (String endPoint : DLCMTestDocHelper.getArchivalStorageLinksActions()) {
            expectedMap.put("$._links", hasKey(endPoint));
            expectedMap.put("$._links." + endPoint, hasKey("href"));
            expectedMap.put("$._links." + endPoint, hasValue(anyOf(endsWithList(DLCMTestDocHelper.getArchivalStorageActions()))));
        }
        return expectedMap;
    }

    public static Map<String, Matcher> getArchivalStorageAipExpectedMatchers() {
        Map<String, Matcher> expectedMap = new HashMap<>();
        expectedMap.put("$._data", hasSize(0));
        final int SIZE_LINKS = DLCMTestDocHelper.getArchivalStorageAipLinksActions().size();
        expectedMap.put("$._links.*", hasSize(SIZE_LINKS));
        for (String endPoint : DLCMTestDocHelper.getArchivalStorageAipLinksActions()) {
            expectedMap.put("$._links", hasKey(endPoint));
            expectedMap.put("$._links." + endPoint, hasKey("href"));
            expectedMap.put("$._links." + endPoint, hasValue(anyOf(endsWithList(DLCMTestDocHelper.getArchivalStorageAipActions()))));
        }
        return expectedMap;
    }

    public static Map<String, Matcher> getArchivalStorageAipStatusExpectedMatchers() {
        Map<String, Matcher> expectedMap = new HashMap<>();
        expectedMap.put("$.*", hasSize(DLCMTestDocHelper.getArchivalStorageAipStatus().size()));
        expectedMap.put("$", hasItems(DLCMTestDocHelper.getArchivalStorageAipStatus().toArray()));
        return expectedMap;

    }
}
