package ch.dlcm.test.executor;

import static org.hamcrest.CoreMatchers.endsWith;

import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.hamcrest.Matcher;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.http.MediaType;

import ch.dlcm.model.Status;
import ch.dlcm.rest.RESTAction;
import ch.dlcm.rest.RESTName;
import org.springframework.test.web.servlet.ResultActions;

/**
 * Helpers to rest DLCM cycle tests.
 */
public class DLCMTestDocHelper {


    public static final MediaType JSON_MEDIA_TYPE = new MediaType("application","json", Charset.forName("UTF-8"));
    public static final MediaType HAL_JSON_MEDIA_TYPE = new MediaType("application","hal+json", Charset.forName("UTF-8"));
    public static final MediaType HTML_MEDIA_TYPE = new MediaType("text","html", Charset.forName("UTF-8"));

    /* Applications routes*/
    public static final String APPLICATION_ROOT = "/";
    public static final String APPLICATION = "Application";


    //  ================================
    //  ADMINISTRATION
    //  ================================
    /**
     * List of ingestion links to test
     */
    public static List<String> getAdministrationRootActions(){
        return Arrays.asList(
                RESTAction.SELF,
                RESTAction.DASHBOARD,
                RESTName.ADMIN);
    }
    

    //	================================
    //	INGESTION
    //	================================
    /**
     * List of ingestion links to test
     */
    public static List<String> getIngestionRootActions(){
        return Arrays.asList(
                RESTAction.SELF,
                RESTAction.DASHBOARD,
                RESTName.PREINGEST,
                RESTName.INGEST,
                RESTName.ADMIN);
    }

    /**
     * List of Admin/institutions links to test
     */
    public static List<String> getAdminInstitutionsRootActions() {
        return Arrays.asList(
                RESTAction.SELF,
                RESTAction.MODULE,
                RESTAction.LASTUPDATE,
                RESTAction.LASTCREATED);
    }

    /**
     * List of created Admin/institutions links to test
     */
    public static List<String> getAdminInstitutionsPostActions() {
        return Arrays.asList(
                "dlcmId",
                "name",
                "description",
                "updateTime",
                "creationTime",
                "_links");
    }

    public static List<String> getAdminRootActions() {
        return Arrays.asList(
                RESTAction.SELF,
                RESTAction.RESOURCES);
    }

    public static List<String> getAdminRessourcesActions() {
        return Arrays.asList(
                RESTName.PERSON,
                RESTName.INSTITUTION,
                RESTName.UNIT,
                RESTName.SUBMISSION_AGREEMENT,
                RESTName.LICENSE,
                RESTName.LANGUAGE,
                RESTName.ROLE,
                RESTName.FUNDING_AGENCY);
    }

    /**
     * List of Access/dip links to test
     */
    public static List<String> getAdminInstitutionLinksActions() {
        String dipPrefix = RESTName.ADMIN +"/"+ RESTName.INSTITUTION;
        return Arrays.asList(
                RESTName.ADMIN,
                dipPrefix,
                dipPrefix + "?sort=creationTime,desc",
                dipPrefix + "?sort=updateTime,desc");
    }


    /**
     * List of admin/people links to test
     */
    public static List<String> getAdminPeopleActions() {
        return Arrays.asList(
                "dlcmId",
                "firstName",
                "lastName",
                "updateTime",
                "creationTime",
                "_links",
                "orcid");
    }

    /**
     * List of admin/people links to test
     */
    public static List<String> getAdminPeopleLinksActions() {
        return Arrays.asList(
                RESTAction.SELF,
                RESTAction.LIST,
                RESTAction.PARENT);
    }

    /**
     * List of ingest/sip links to test
     */
    public static List<String> getIngestSipActions() {
        return Arrays.asList(
                "dlcmId",
                "info",
                "updateTime",
                "creationTime",
                "depositId",
                "aipId",
                "allDataFileReady",
                "_links");
    }

    /**
     * List of ingest/sip links to test
     */
    public static List<String> getIngestSipInfoActions() {
        return Arrays.asList(
                "status",
                "name",
                "description");
    }

    //	================================
    //	STORAGION
    //	================================

    /**
     * List of root Storagion links
     */
    public static List<String> getStoragionRootLinksActions() {
        return Arrays.asList(
                RESTAction.SELF,
                RESTAction.DASHBOARD,
                RESTName.DATAMGMT,
                RESTName.ARCHIVALSTORAGE);
    }

    public static List<String> getDataMgmtActions() {
        return Arrays.asList(
                RESTAction.SELF,
                RESTAction.RESOURCES
        );
    }

    public static List<String> getDataMgmtLinksActions() {
        return Arrays.asList(RESTName.DATAMGMT);
    }

    public static List<String> getDataMgmtResourcesActions() {
        return Arrays.asList(
                RESTName.DI,
                RESTName.SETTING);
    }

    public static List<String> getDataMgmtDIActions(ResultActions request) throws UnsupportedEncodingException {
        List<String> list = Arrays.asList(
                RESTAction.SELF,
                RESTAction.MODULE,
                RESTAction.LASTUPDATE,
                RESTAction.LASTCREATED,
                RESTAction.SEARCH);
        if (hasNextOnLinks(request)){
            ArrayList listWithNext = new ArrayList();
            listWithNext.addAll(list);
            listWithNext.add(RESTAction.NEXT);
            return listWithNext;
        }
        return list;
    }

    public static List<String> getDataMgmtDILinksActions() {
        String diPrefix = RESTName.DATAMGMT +"/"+ RESTName.DI;
        return Arrays.asList(
                RESTName.DATAMGMT,
                diPrefix,
                diPrefix + "?sort=creationTime,desc",
                diPrefix + "?sort=updateTime,desc",
                diPrefix + "/" + RESTAction.SEARCH,
                diPrefix + "?size=20&page=1");
    }

    public static List<String> getArchivalStorageActions() {
        return Arrays.asList(
                RESTName.ARCHIVALSTORAGE,
                RESTName.ARCHIVALSTORAGE + "/" + RESTName.AIP);
    }

    public static List<String> getArchivalStorageLinksActions() {
        return Arrays.asList(
                RESTAction.SELF,
                RESTAction.RESOURCES
        );
    }

    /**
     * List of archival-storage/aip links to test
     */
    public static List<String> getArchivalStorageAipLinksActions() {
        return Arrays.asList(
                RESTAction.SELF,
                RESTAction.MODULE,
                RESTAction.LASTUPDATE,
                RESTAction.LASTCREATED,
                RESTAction.VALUES);
    }

    public static List<String> getArchivalStorageAipActions() {
        String aipPrefix = RESTName.ARCHIVALSTORAGE +"/"+ RESTName.AIP;
        return Arrays.asList(
                RESTName.ARCHIVALSTORAGE,
                aipPrefix,
                aipPrefix + "?sort=creationTime,desc",
                aipPrefix + "?sort=updateTime,desc",
                aipPrefix + "/" + RESTName.STATUS);
    }

    /**
     * List of Archival-sorage/aip/status. That's package list status
     */
    public static List<String> getArchivalStorageAipStatus() {
        return getPackageStatus();
    }


    //	================================
    //	ACCESSION
    //	================================
    /**
     * List of root Accession links
     */
    public static List<String> getAccessionRootLinksActions() {
        return Arrays.asList(
                RESTAction.SELF,
                RESTAction.DASHBOARD,
                RESTName.ACCESS);
    }

    /**
     * List of access links
     */
    public static List<String> getAccessLinksActions() {
        return Arrays.asList(
                RESTAction.SELF,
                RESTAction.RESOURCES);
    }

    /**
     * List of access ressources links
     */
    public static List<String> getAccessRessourcesLinksActions() {
        return Arrays.asList(
                RESTName.DIP,
                RESTName.ORDER,
                RESTName.AIP);
    }

    /**
     * List of Access/dip links to test
     */
    private static List<String> getAccessPackageActions() {
      return Arrays.asList(
              RESTAction.SELF,
              RESTAction.MODULE,
              RESTAction.LASTUPDATE,
              RESTAction.LASTCREATED,
              RESTAction.VALUES);
    }

    public static List<String> getAccessOrderActions() {
      return getAccessPackageActions();
    }

    public static List<String> getAccessDipActions() {
        return getAccessPackageActions();
    }
    
    public static List<String> getAccessAipActions() {
      return getAccessPackageActions();
    }


    /**
     * List of Access/dip links to test
     */
    
    private static List<String> getAccessLinksActions(String resource) {
        String prefix = RESTName.ACCESS +"/"+ resource;
        return Arrays.asList(
                RESTName.ACCESS,
                prefix,
                prefix + "?sort=creationTime,desc",
                prefix + "?sort=updateTime,desc",
                RESTName.STATUS);
    }

    public static List<String> getAccessDipLinksActions() {
        return getAccessLinksActions(RESTName.DIP);
    }
    
    public static List<String> getAccessAipLinksActions() {
      return getAccessLinksActions(RESTName.AIP);
    }
    
    public static List<String> getAccessOrdersLinksActions() {
      String prefix = RESTName.ACCESS +"/"+ RESTName.ORDER;
      return Arrays.asList(
          RESTName.ACCESS,
          prefix,
          prefix + "?sort=creationTime,desc",
          prefix + "?sort=updateTime,desc",
          RESTName.STATUS,
          RESTName.QUERYTYPE);
    }

    /**
     * List of Access/dip/status. The same than Archival-sorage/aip/status
     */
    public static List<String> getAccessDipStatus() {
        return getPackageStatus();
    }
    
    public static List<String> getAccessAipStatus() {
      return getPackageStatus();
    }

    private static List<String> getPackageStatus(){
        return Arrays.asList(
                Status.StatusValue.InPreparation.name(),
                Status.StatusValue.Ready.name(),
                Status.StatusValue.Checked.name(),
                Status.StatusValue.InProgress.name(),
                Status.StatusValue.Stored.name(),
                Status.StatusValue.Indexing.name(),
                Status.StatusValue.Completed.name(),
                Status.StatusValue.Reindexing.name(),
                Status.StatusValue.Downloading.name(),
                Status.StatusValue.InError.name()
        );
    }

    //	================================
    //	DIVERS
    //	================================
    /**
     * A interable endWith matchers used to test the content of actions
     * @param actions A list of actions
     */
    public static Iterable endsWithList(List<String> actions) {
        List<Matcher<String>> list = new ArrayList<>();
        for(String action:actions){
            if (action.equals(RESTAction.SELF)){
                list.add(endsWith("/"));
            }else{
                list.add(endsWith(action));
            }
        }
        return list;
    }

    public static List<String> getIngestSipResumetActions() {
        return Arrays.asList(
                "dlcmId",
                "status",
                "message",
                "_links"
        );
    }

    /**
     * Helper that check for next link on "_links"
     */
    public static boolean hasNextOnLinks(ResultActions request) throws UnsupportedEncodingException {
      Integer totalPages = new JSONObject(request.andReturn().getResponse().getContentAsString()).getJSONObject( "_page").getInt("totalPages");
      Integer totalItems = new JSONObject(request.andReturn().getResponse().getContentAsString()).getJSONObject( "_page").getInt("totalItems");
      Integer currentPage = new JSONObject(request.andReturn().getResponse().getContentAsString()).getJSONObject( "_page").getInt("currentPage");
      return ( totalItems != 0 ) && ( currentPage + 1 != totalPages );
    }

}
