package ch.dlcm.tools;

import java.nio.file.Files;
import java.nio.file.Path;
import java.util.List;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.CommandLineRunner;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.stereotype.Component;

import ch.dlcm.model.DepositDataFile;
import ch.dlcm.rest.RESTName;

@Component
public class DepositUploader extends Tools implements CommandLineRunner {
	private static final Logger log = Logger.getLogger(DepositUploader.class);

	@Value("${dlcm.import.deposit}")
	private String importFolder;

	@Override
	public void run(String... arg0) throws Exception {
		if (!importFolder.isEmpty()) {
			log.info("Scanning folder: " + importFolder);
			for (Path d : scanFolder(importFolder, Files::isDirectory)) {
				int count = 0;
				int total = 0;
				log.info("Importing " + d);
				String id = createDeposit(d.getFileName().toString());
				if (id != null) {
					for (Path dataCat : scanFolder(d.toString(), Files::isDirectory)) {
						for (Path dataType : scanFolder(dataCat.toString(), Files::isDirectory)) {
							List<Path> fileList = scanAll(dataType.toString());
							total += fileList.size();
							for (Path f : fileList) {
								String relLocation = dataType.relativize(f.getParent()).toString();
								String dfId = addDataFile(
										this.preIngestUrl + "/" + RESTName.DEPOSIT + "/" + id + "/" + RESTName.UL, f,
										dataCat.getFileName().toString(), dataType.getFileName().toString(),
										relLocation, new ParameterizedTypeReference<DepositDataFile[]>() {});
								if (dfId == null) {
									log.error("Cannot add data file (" + f.toString() + ") to deposit (ID=" + id + ")");
									break;
								}
								count++;
							}
						}
					}
				}
				log.info("Importing end " + d + ": " + count + " uploaded file(s) of " + total + " data file(s)");
			}
		}
	}

}